// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

// Package sdk provides the OpenTelemetry default SDK for Go.
package sdk // import "go.opentelemetry.io/otel/sdk"

// Version is the current release version of the OpenTelemetry SDK in use.
func Version() string {
	return "1.44.0"
}
