// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: udpa/annotations/status.proto

package annotations

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type PackageVersionStatus int32

const (
	PackageVersionStatus_UNKNOWN                      PackageVersionStatus = 0
	PackageVersionStatus_FROZEN                       PackageVersionStatus = 1
	PackageVersionStatus_ACTIVE                       PackageVersionStatus = 2
	PackageVersionStatus_NEXT_MAJOR_VERSION_CANDIDATE PackageVersionStatus = 3
)

// Enum value maps for PackageVersionStatus.
var (
	PackageVersionStatus_name = map[int32]string{
		0: "UNKNOWN",
		1: "FROZEN",
		2: "ACTIVE",
		3: "NEXT_MAJOR_VERSION_CANDIDATE",
	}
	PackageVersionStatus_value = map[string]int32{
		"UNKNOWN":                      0,
		"FROZEN":                       1,
		"ACTIVE":                       2,
		"NEXT_MAJOR_VERSION_CANDIDATE": 3,
	}
)

func (x PackageVersionStatus) Enum() *PackageVersionStatus {
	p := new(PackageVersionStatus)
	*p = x
	return p
}

func (x PackageVersionStatus) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (PackageVersionStatus) Descriptor() protoreflect.EnumDescriptor {
	return file_udpa_annotations_status_proto_enumTypes[0].Descriptor()
}

func (PackageVersionStatus) Type() protoreflect.EnumType {
	return &file_udpa_annotations_status_proto_enumTypes[0]
}

func (x PackageVersionStatus) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use PackageVersionStatus.Descriptor instead.
func (PackageVersionStatus) EnumDescriptor() ([]byte, []int) {
	return file_udpa_annotations_status_proto_rawDescGZIP(), []int{0}
}

type StatusAnnotation struct {
	state                protoimpl.MessageState `protogen:"open.v1"`
	WorkInProgress       bool                   `protobuf:"varint,1,opt,name=work_in_progress,json=workInProgress,proto3" json:"work_in_progress,omitempty"`
	PackageVersionStatus PackageVersionStatus   `protobuf:"varint,2,opt,name=package_version_status,json=packageVersionStatus,proto3,enum=udpa.annotations.PackageVersionStatus" json:"package_version_status,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *StatusAnnotation) Reset() {
	*x = StatusAnnotation{}
	mi := &file_udpa_annotations_status_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatusAnnotation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatusAnnotation) ProtoMessage() {}

func (x *StatusAnnotation) ProtoReflect() protoreflect.Message {
	mi := &file_udpa_annotations_status_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatusAnnotation.ProtoReflect.Descriptor instead.
func (*StatusAnnotation) Descriptor() ([]byte, []int) {
	return file_udpa_annotations_status_proto_rawDescGZIP(), []int{0}
}

func (x *StatusAnnotation) GetWorkInProgress() bool {
	if x != nil {
		return x.WorkInProgress
	}
	return false
}

func (x *StatusAnnotation) GetPackageVersionStatus() PackageVersionStatus {
	if x != nil {
		return x.PackageVersionStatus
	}
	return PackageVersionStatus_UNKNOWN
}

var file_udpa_annotations_status_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.FileOptions)(nil),
		ExtensionType: (*StatusAnnotation)(nil),
		Field:         222707719,
		Name:          "udpa.annotations.file_status",
		Tag:           "bytes,222707719,opt,name=file_status",
		Filename:      "udpa/annotations/status.proto",
	},
}

// Extension fields to descriptorpb.FileOptions.
var (
	// optional udpa.annotations.StatusAnnotation file_status = 222707719;
	E_FileStatus = &file_udpa_annotations_status_proto_extTypes[0]
)

var File_udpa_annotations_status_proto protoreflect.FileDescriptor

const file_udpa_annotations_status_proto_rawDesc = "" +
	"\n" +
	"\x1dudpa/annotations/status.proto\x12\x10udpa.annotations\x1a google/protobuf/descriptor.proto\"\x9a\x01\n" +
	"\x10StatusAnnotation\x12(\n" +
	"\x10work_in_progress\x18\x01 \x01(\bR\x0eworkInProgress\x12\\\n" +
	"\x16package_version_status\x18\x02 \x01(\x0e2&.udpa.annotations.PackageVersionStatusR\x14packageVersionStatus*]\n" +
	"\x14PackageVersionStatus\x12\v\n" +
	"\aUNKNOWN\x10\x00\x12\n" +
	"\n" +
	"\x06FROZEN\x10\x01\x12\n" +
	"\n" +
	"\x06ACTIVE\x10\x02\x12 \n" +
	"\x1cNEXT_MAJOR_VERSION_CANDIDATE\x10\x03:d\n" +
	"\vfile_status\x12\x1c.google.protobuf.FileOptions\x18\x87\x80\x99j \x01(\v2\".udpa.annotations.StatusAnnotationR\n" +
	"fileStatusB)Z'github.com/cncf/xds/go/udpa/annotationsb\x06proto3"

var (
	file_udpa_annotations_status_proto_rawDescOnce sync.Once
	file_udpa_annotations_status_proto_rawDescData []byte
)

func file_udpa_annotations_status_proto_rawDescGZIP() []byte {
	file_udpa_annotations_status_proto_rawDescOnce.Do(func() {
		file_udpa_annotations_status_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_udpa_annotations_status_proto_rawDesc), len(file_udpa_annotations_status_proto_rawDesc)))
	})
	return file_udpa_annotations_status_proto_rawDescData
}

var file_udpa_annotations_status_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_udpa_annotations_status_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_udpa_annotations_status_proto_goTypes = []any{
	(PackageVersionStatus)(0),        // 0: udpa.annotations.PackageVersionStatus
	(*StatusAnnotation)(nil),         // 1: udpa.annotations.StatusAnnotation
	(*descriptorpb.FileOptions)(nil), // 2: google.protobuf.FileOptions
}
var file_udpa_annotations_status_proto_depIdxs = []int32{
	0, // 0: udpa.annotations.StatusAnnotation.package_version_status:type_name -> udpa.annotations.PackageVersionStatus
	2, // 1: udpa.annotations.file_status:extendee -> google.protobuf.FileOptions
	1, // 2: udpa.annotations.file_status:type_name -> udpa.annotations.StatusAnnotation
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	2, // [2:3] is the sub-list for extension type_name
	1, // [1:2] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_udpa_annotations_status_proto_init() }
func file_udpa_annotations_status_proto_init() {
	if File_udpa_annotations_status_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_udpa_annotations_status_proto_rawDesc), len(file_udpa_annotations_status_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 1,
			NumServices:   0,
		},
		GoTypes:           file_udpa_annotations_status_proto_goTypes,
		DependencyIndexes: file_udpa_annotations_status_proto_depIdxs,
		EnumInfos:         file_udpa_annotations_status_proto_enumTypes,
		MessageInfos:      file_udpa_annotations_status_proto_msgTypes,
		ExtensionInfos:    file_udpa_annotations_status_proto_extTypes,
	}.Build()
	File_udpa_annotations_status_proto = out.File
	file_udpa_annotations_status_proto_goTypes = nil
	file_udpa_annotations_status_proto_depIdxs = nil
}
