// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/tap/v3/common.proto

package tapv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/common/matcher/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v32 "github.com/envoyproxy/go-control-plane/envoy/config/route/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Output format. All output is in the form of one or more :ref:`TraceWrapper
// <envoy_v3_api_msg_data.tap.v3.TraceWrapper>` messages. This enumeration indicates
// how those messages are written. Note that not all sinks support all output formats. See
// individual sink documentation for more information.
type OutputSink_Format int32

const (
	// Each message will be written as JSON. Any :ref:`body <envoy_v3_api_msg_data.tap.v3.Body>`
	// data will be present in the :ref:`as_bytes
	// <envoy_v3_api_field_data.tap.v3.Body.as_bytes>` field. This means that body data will be
	// base64 encoded as per the `proto3 JSON mappings
	// <https://developers.google.com/protocol-buffers/docs/proto3#json>`_.
	OutputSink_JSON_BODY_AS_BYTES OutputSink_Format = 0
	// Each message will be written as JSON. Any :ref:`body <envoy_v3_api_msg_data.tap.v3.Body>`
	// data will be present in the :ref:`as_string
	// <envoy_v3_api_field_data.tap.v3.Body.as_string>` field. This means that body data will be
	// string encoded as per the `proto3 JSON mappings
	// <https://developers.google.com/protocol-buffers/docs/proto3#json>`_. This format type is
	// useful when it is known that that body is human readable (e.g., JSON over HTTP) and the
	// user wishes to view it directly without being forced to base64 decode the body.
	OutputSink_JSON_BODY_AS_STRING OutputSink_Format = 1
	// Binary proto format. Note that binary proto is not self-delimiting. If a sink writes
	// multiple binary messages without any length information the data stream will not be
	// useful. However, for certain sinks that are self-delimiting (e.g., one message per file)
	// this output format makes consumption simpler.
	OutputSink_PROTO_BINARY OutputSink_Format = 2
	// Messages are written as a sequence tuples, where each tuple is the message length encoded
	// as a `protobuf 32-bit varint
	// <https://developers.google.com/protocol-buffers/docs/reference/cpp/google.protobuf.io.coded_stream>`_
	// followed by the binary message. The messages can be read back using the language specific
	// protobuf coded stream implementation to obtain the message length and the message.
	OutputSink_PROTO_BINARY_LENGTH_DELIMITED OutputSink_Format = 3
	// Text proto format.
	OutputSink_PROTO_TEXT OutputSink_Format = 4
)

// Enum value maps for OutputSink_Format.
var (
	OutputSink_Format_name = map[int32]string{
		0: "JSON_BODY_AS_BYTES",
		1: "JSON_BODY_AS_STRING",
		2: "PROTO_BINARY",
		3: "PROTO_BINARY_LENGTH_DELIMITED",
		4: "PROTO_TEXT",
	}
	OutputSink_Format_value = map[string]int32{
		"JSON_BODY_AS_BYTES":            0,
		"JSON_BODY_AS_STRING":           1,
		"PROTO_BINARY":                  2,
		"PROTO_BINARY_LENGTH_DELIMITED": 3,
		"PROTO_TEXT":                    4,
	}
)

func (x OutputSink_Format) Enum() *OutputSink_Format {
	p := new(OutputSink_Format)
	*p = x
	return p
}

func (x OutputSink_Format) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (OutputSink_Format) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_config_tap_v3_common_proto_enumTypes[0].Descriptor()
}

func (OutputSink_Format) Type() protoreflect.EnumType {
	return &file_envoy_config_tap_v3_common_proto_enumTypes[0]
}

func (x OutputSink_Format) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use OutputSink_Format.Descriptor instead.
func (OutputSink_Format) EnumDescriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{5, 0}
}

// Tap configuration.
type TapConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The match configuration. If the configuration matches the data source being tapped, a tap will
	// occur, with the result written to the configured output.
	// Exactly one of :ref:`match <envoy_v3_api_field_config.tap.v3.TapConfig.match>` and
	// :ref:`match_config <envoy_v3_api_field_config.tap.v3.TapConfig.match_config>` must be set. If both
	// are set, the :ref:`match <envoy_v3_api_field_config.tap.v3.TapConfig.match>` will be used.
	//
	// Deprecated: Marked as deprecated in envoy/config/tap/v3/common.proto.
	MatchConfig *MatchPredicate `protobuf:"bytes,1,opt,name=match_config,json=matchConfig,proto3" json:"match_config,omitempty"`
	// The match configuration. If the configuration matches the data source being tapped, a tap will
	// occur, with the result written to the configured output.
	// Exactly one of :ref:`match <envoy_v3_api_field_config.tap.v3.TapConfig.match>` and
	// :ref:`match_config <envoy_v3_api_field_config.tap.v3.TapConfig.match_config>` must be set. If both
	// are set, the :ref:`match <envoy_v3_api_field_config.tap.v3.TapConfig.match>` will be used.
	Match *v3.MatchPredicate `protobuf:"bytes,4,opt,name=match,proto3" json:"match,omitempty"`
	// The tap output configuration. If a match configuration matches a data source being tapped,
	// a tap will occur and the data will be written to the configured output.
	OutputConfig *OutputConfig `protobuf:"bytes,2,opt,name=output_config,json=outputConfig,proto3" json:"output_config,omitempty"`
	// [#not-implemented-hide:] Specify if Tap matching is enabled. The % of requests\connections for
	// which the tap matching is enabled. When not enabled, the request\connection will not be
	// recorded.
	//
	// .. note::
	//
	//	This field defaults to 100/:ref:`HUNDRED
	//	<envoy_v3_api_enum_type.v3.FractionalPercent.DenominatorType>`.
	TapEnabled    *v31.RuntimeFractionalPercent `protobuf:"bytes,3,opt,name=tap_enabled,json=tapEnabled,proto3" json:"tap_enabled,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TapConfig) Reset() {
	*x = TapConfig{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TapConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TapConfig) ProtoMessage() {}

func (x *TapConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TapConfig.ProtoReflect.Descriptor instead.
func (*TapConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{0}
}

// Deprecated: Marked as deprecated in envoy/config/tap/v3/common.proto.
func (x *TapConfig) GetMatchConfig() *MatchPredicate {
	if x != nil {
		return x.MatchConfig
	}
	return nil
}

func (x *TapConfig) GetMatch() *v3.MatchPredicate {
	if x != nil {
		return x.Match
	}
	return nil
}

func (x *TapConfig) GetOutputConfig() *OutputConfig {
	if x != nil {
		return x.OutputConfig
	}
	return nil
}

func (x *TapConfig) GetTapEnabled() *v31.RuntimeFractionalPercent {
	if x != nil {
		return x.TapEnabled
	}
	return nil
}

// Tap match configuration. This is a recursive structure which allows complex nested match
// configurations to be built using various logical operators.
// [#next-free-field: 11]
type MatchPredicate struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Rule:
	//
	//	*MatchPredicate_OrMatch
	//	*MatchPredicate_AndMatch
	//	*MatchPredicate_NotMatch
	//	*MatchPredicate_AnyMatch
	//	*MatchPredicate_HttpRequestHeadersMatch
	//	*MatchPredicate_HttpRequestTrailersMatch
	//	*MatchPredicate_HttpResponseHeadersMatch
	//	*MatchPredicate_HttpResponseTrailersMatch
	//	*MatchPredicate_HttpRequestGenericBodyMatch
	//	*MatchPredicate_HttpResponseGenericBodyMatch
	Rule          isMatchPredicate_Rule `protobuf_oneof:"rule"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MatchPredicate) Reset() {
	*x = MatchPredicate{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MatchPredicate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MatchPredicate) ProtoMessage() {}

func (x *MatchPredicate) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MatchPredicate.ProtoReflect.Descriptor instead.
func (*MatchPredicate) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{1}
}

func (x *MatchPredicate) GetRule() isMatchPredicate_Rule {
	if x != nil {
		return x.Rule
	}
	return nil
}

func (x *MatchPredicate) GetOrMatch() *MatchPredicate_MatchSet {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_OrMatch); ok {
			return x.OrMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetAndMatch() *MatchPredicate_MatchSet {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_AndMatch); ok {
			return x.AndMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetNotMatch() *MatchPredicate {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_NotMatch); ok {
			return x.NotMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetAnyMatch() bool {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_AnyMatch); ok {
			return x.AnyMatch
		}
	}
	return false
}

func (x *MatchPredicate) GetHttpRequestHeadersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpRequestHeadersMatch); ok {
			return x.HttpRequestHeadersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpRequestTrailersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpRequestTrailersMatch); ok {
			return x.HttpRequestTrailersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpResponseHeadersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpResponseHeadersMatch); ok {
			return x.HttpResponseHeadersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpResponseTrailersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpResponseTrailersMatch); ok {
			return x.HttpResponseTrailersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpRequestGenericBodyMatch() *HttpGenericBodyMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpRequestGenericBodyMatch); ok {
			return x.HttpRequestGenericBodyMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpResponseGenericBodyMatch() *HttpGenericBodyMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpResponseGenericBodyMatch); ok {
			return x.HttpResponseGenericBodyMatch
		}
	}
	return nil
}

type isMatchPredicate_Rule interface {
	isMatchPredicate_Rule()
}

type MatchPredicate_OrMatch struct {
	// A set that describes a logical OR. If any member of the set matches, the match configuration
	// matches.
	OrMatch *MatchPredicate_MatchSet `protobuf:"bytes,1,opt,name=or_match,json=orMatch,proto3,oneof"`
}

type MatchPredicate_AndMatch struct {
	// A set that describes a logical AND. If all members of the set match, the match configuration
	// matches.
	AndMatch *MatchPredicate_MatchSet `protobuf:"bytes,2,opt,name=and_match,json=andMatch,proto3,oneof"`
}

type MatchPredicate_NotMatch struct {
	// A negation match. The match configuration will match if the negated match condition matches.
	NotMatch *MatchPredicate `protobuf:"bytes,3,opt,name=not_match,json=notMatch,proto3,oneof"`
}

type MatchPredicate_AnyMatch struct {
	// The match configuration will always match.
	AnyMatch bool `protobuf:"varint,4,opt,name=any_match,json=anyMatch,proto3,oneof"`
}

type MatchPredicate_HttpRequestHeadersMatch struct {
	// HTTP request headers match configuration.
	HttpRequestHeadersMatch *HttpHeadersMatch `protobuf:"bytes,5,opt,name=http_request_headers_match,json=httpRequestHeadersMatch,proto3,oneof"`
}

type MatchPredicate_HttpRequestTrailersMatch struct {
	// HTTP request trailers match configuration.
	HttpRequestTrailersMatch *HttpHeadersMatch `protobuf:"bytes,6,opt,name=http_request_trailers_match,json=httpRequestTrailersMatch,proto3,oneof"`
}

type MatchPredicate_HttpResponseHeadersMatch struct {
	// HTTP response headers match configuration.
	HttpResponseHeadersMatch *HttpHeadersMatch `protobuf:"bytes,7,opt,name=http_response_headers_match,json=httpResponseHeadersMatch,proto3,oneof"`
}

type MatchPredicate_HttpResponseTrailersMatch struct {
	// HTTP response trailers match configuration.
	HttpResponseTrailersMatch *HttpHeadersMatch `protobuf:"bytes,8,opt,name=http_response_trailers_match,json=httpResponseTrailersMatch,proto3,oneof"`
}

type MatchPredicate_HttpRequestGenericBodyMatch struct {
	// HTTP request generic body match configuration.
	HttpRequestGenericBodyMatch *HttpGenericBodyMatch `protobuf:"bytes,9,opt,name=http_request_generic_body_match,json=httpRequestGenericBodyMatch,proto3,oneof"`
}

type MatchPredicate_HttpResponseGenericBodyMatch struct {
	// HTTP response generic body match configuration.
	HttpResponseGenericBodyMatch *HttpGenericBodyMatch `protobuf:"bytes,10,opt,name=http_response_generic_body_match,json=httpResponseGenericBodyMatch,proto3,oneof"`
}

func (*MatchPredicate_OrMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_AndMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_NotMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_AnyMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpRequestHeadersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpRequestTrailersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpResponseHeadersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpResponseTrailersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpRequestGenericBodyMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpResponseGenericBodyMatch) isMatchPredicate_Rule() {}

// HTTP headers match configuration.
type HttpHeadersMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// HTTP headers to match.
	Headers       []*v32.HeaderMatcher `protobuf:"bytes,1,rep,name=headers,proto3" json:"headers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpHeadersMatch) Reset() {
	*x = HttpHeadersMatch{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpHeadersMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpHeadersMatch) ProtoMessage() {}

func (x *HttpHeadersMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpHeadersMatch.ProtoReflect.Descriptor instead.
func (*HttpHeadersMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{2}
}

func (x *HttpHeadersMatch) GetHeaders() []*v32.HeaderMatcher {
	if x != nil {
		return x.Headers
	}
	return nil
}

// HTTP generic body match configuration.
// List of text strings and hex strings to be located in HTTP body.
// All specified strings must be found in the HTTP body for positive match.
// The search may be limited to specified number of bytes from the body start.
//
// .. attention::
//
//	Searching for patterns in HTTP body is potentially cpu intensive. For each specified pattern, http body is scanned byte by byte to find a match.
//	If multiple patterns are specified, the process is repeated for each pattern. If location of a pattern is known, ``bytes_limit`` should be specified
//	to scan only part of the http body.
type HttpGenericBodyMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Limits search to specified number of bytes - default zero (no limit - match entire captured buffer).
	BytesLimit uint32 `protobuf:"varint,1,opt,name=bytes_limit,json=bytesLimit,proto3" json:"bytes_limit,omitempty"`
	// List of patterns to match.
	Patterns      []*HttpGenericBodyMatch_GenericTextMatch `protobuf:"bytes,2,rep,name=patterns,proto3" json:"patterns,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpGenericBodyMatch) Reset() {
	*x = HttpGenericBodyMatch{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpGenericBodyMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpGenericBodyMatch) ProtoMessage() {}

func (x *HttpGenericBodyMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpGenericBodyMatch.ProtoReflect.Descriptor instead.
func (*HttpGenericBodyMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{3}
}

func (x *HttpGenericBodyMatch) GetBytesLimit() uint32 {
	if x != nil {
		return x.BytesLimit
	}
	return 0
}

func (x *HttpGenericBodyMatch) GetPatterns() []*HttpGenericBodyMatch_GenericTextMatch {
	if x != nil {
		return x.Patterns
	}
	return nil
}

// Tap output configuration.
// [#next-free-field: 6]
type OutputConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Output sinks for tap data. Currently a single sink is allowed in the list. Once multiple
	// sink types are supported this constraint will be relaxed.
	Sinks []*OutputSink `protobuf:"bytes,1,rep,name=sinks,proto3" json:"sinks,omitempty"`
	// For buffered tapping, the maximum amount of received body that will be buffered prior to
	// truncation. If truncation occurs, the :ref:`truncated
	// <envoy_v3_api_field_data.tap.v3.Body.truncated>` field will be set. If not specified, the
	// default is 1KiB.
	MaxBufferedRxBytes *wrapperspb.UInt32Value `protobuf:"bytes,2,opt,name=max_buffered_rx_bytes,json=maxBufferedRxBytes,proto3" json:"max_buffered_rx_bytes,omitempty"`
	// For buffered tapping, the maximum amount of transmitted body that will be buffered prior to
	// truncation. If truncation occurs, the :ref:`truncated
	// <envoy_v3_api_field_data.tap.v3.Body.truncated>` field will be set. If not specified, the
	// default is 1KiB.
	MaxBufferedTxBytes *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=max_buffered_tx_bytes,json=maxBufferedTxBytes,proto3" json:"max_buffered_tx_bytes,omitempty"`
	// Indicates whether taps produce a single buffered message per tap, or multiple streamed
	// messages per tap in the emitted :ref:`TraceWrapper
	// <envoy_v3_api_msg_data.tap.v3.TraceWrapper>` messages. Note that streamed tapping does not
	// mean that no buffering takes place. Buffering may be required if data is processed before a
	// match can be determined. See the HTTP tap filter :ref:`streaming
	// <config_http_filters_tap_streaming>` documentation for more information.
	Streaming bool `protobuf:"varint,4,opt,name=streaming,proto3" json:"streaming,omitempty"`
	// Tapped messages will be sent on each read/write event for streamed tapping by default.
	// But this behavior could be controlled by setting this field.
	// If set then the tapped messages will be send once the threshold is reached.
	// This could be used to avoid high frequent sending.
	MinStreamedSentBytes *wrapperspb.UInt32Value `protobuf:"bytes,5,opt,name=min_streamed_sent_bytes,json=minStreamedSentBytes,proto3" json:"min_streamed_sent_bytes,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *OutputConfig) Reset() {
	*x = OutputConfig{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OutputConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OutputConfig) ProtoMessage() {}

func (x *OutputConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OutputConfig.ProtoReflect.Descriptor instead.
func (*OutputConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{4}
}

func (x *OutputConfig) GetSinks() []*OutputSink {
	if x != nil {
		return x.Sinks
	}
	return nil
}

func (x *OutputConfig) GetMaxBufferedRxBytes() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxBufferedRxBytes
	}
	return nil
}

func (x *OutputConfig) GetMaxBufferedTxBytes() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxBufferedTxBytes
	}
	return nil
}

func (x *OutputConfig) GetStreaming() bool {
	if x != nil {
		return x.Streaming
	}
	return false
}

func (x *OutputConfig) GetMinStreamedSentBytes() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MinStreamedSentBytes
	}
	return nil
}

// Tap output sink configuration.
// [#next-free-field: 7]
type OutputSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Sink output format.
	Format OutputSink_Format `protobuf:"varint,1,opt,name=format,proto3,enum=envoy.config.tap.v3.OutputSink_Format" json:"format,omitempty"`
	// Types that are valid to be assigned to OutputSinkType:
	//
	//	*OutputSink_StreamingAdmin
	//	*OutputSink_FilePerTap
	//	*OutputSink_StreamingGrpc
	//	*OutputSink_BufferedAdmin
	//	*OutputSink_CustomSink
	OutputSinkType isOutputSink_OutputSinkType `protobuf_oneof:"output_sink_type"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *OutputSink) Reset() {
	*x = OutputSink{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OutputSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OutputSink) ProtoMessage() {}

func (x *OutputSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OutputSink.ProtoReflect.Descriptor instead.
func (*OutputSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{5}
}

func (x *OutputSink) GetFormat() OutputSink_Format {
	if x != nil {
		return x.Format
	}
	return OutputSink_JSON_BODY_AS_BYTES
}

func (x *OutputSink) GetOutputSinkType() isOutputSink_OutputSinkType {
	if x != nil {
		return x.OutputSinkType
	}
	return nil
}

func (x *OutputSink) GetStreamingAdmin() *StreamingAdminSink {
	if x != nil {
		if x, ok := x.OutputSinkType.(*OutputSink_StreamingAdmin); ok {
			return x.StreamingAdmin
		}
	}
	return nil
}

func (x *OutputSink) GetFilePerTap() *FilePerTapSink {
	if x != nil {
		if x, ok := x.OutputSinkType.(*OutputSink_FilePerTap); ok {
			return x.FilePerTap
		}
	}
	return nil
}

func (x *OutputSink) GetStreamingGrpc() *StreamingGrpcSink {
	if x != nil {
		if x, ok := x.OutputSinkType.(*OutputSink_StreamingGrpc); ok {
			return x.StreamingGrpc
		}
	}
	return nil
}

func (x *OutputSink) GetBufferedAdmin() *BufferedAdminSink {
	if x != nil {
		if x, ok := x.OutputSinkType.(*OutputSink_BufferedAdmin); ok {
			return x.BufferedAdmin
		}
	}
	return nil
}

func (x *OutputSink) GetCustomSink() *v31.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.OutputSinkType.(*OutputSink_CustomSink); ok {
			return x.CustomSink
		}
	}
	return nil
}

type isOutputSink_OutputSinkType interface {
	isOutputSink_OutputSinkType()
}

type OutputSink_StreamingAdmin struct {
	// Tap output will be streamed out the :http:post:`/tap` admin endpoint.
	//
	// .. attention::
	//
	//	It is only allowed to specify the streaming admin output sink if the tap is being
	//	configured from the :http:post:`/tap` admin endpoint. Thus, if an extension has
	//	been configured to receive tap configuration from some other source (e.g., static
	//	file, XDS, etc.) configuring the streaming admin output type will fail.
	StreamingAdmin *StreamingAdminSink `protobuf:"bytes,2,opt,name=streaming_admin,json=streamingAdmin,proto3,oneof"`
}

type OutputSink_FilePerTap struct {
	// Tap output will be written to a file per tap sink.
	FilePerTap *FilePerTapSink `protobuf:"bytes,3,opt,name=file_per_tap,json=filePerTap,proto3,oneof"`
}

type OutputSink_StreamingGrpc struct {
	// [#not-implemented-hide:]
	// GrpcService to stream data to. The format argument must be PROTO_BINARY.
	// [#comment: TODO(samflattery): remove cleanup in uber_per_filter.cc once implemented]
	StreamingGrpc *StreamingGrpcSink `protobuf:"bytes,4,opt,name=streaming_grpc,json=streamingGrpc,proto3,oneof"`
}

type OutputSink_BufferedAdmin struct {
	// Tap output will be buffered in a single block before flushing to the :http:post:`/tap` admin endpoint
	//
	// .. attention::
	//
	//	It is only allowed to specify the buffered admin output sink if the tap is being
	//	configured from the :http:post:`/tap` admin endpoint. Thus, if an extension has
	//	been configured to receive tap configuration from some other source (e.g., static
	//	file, XDS, etc.) configuring the buffered admin output type will fail.
	BufferedAdmin *BufferedAdminSink `protobuf:"bytes,5,opt,name=buffered_admin,json=bufferedAdmin,proto3,oneof"`
}

type OutputSink_CustomSink struct {
	// Tap output filter will be defined by an extension type
	CustomSink *v31.TypedExtensionConfig `protobuf:"bytes,6,opt,name=custom_sink,json=customSink,proto3,oneof"`
}

func (*OutputSink_StreamingAdmin) isOutputSink_OutputSinkType() {}

func (*OutputSink_FilePerTap) isOutputSink_OutputSinkType() {}

func (*OutputSink_StreamingGrpc) isOutputSink_OutputSinkType() {}

func (*OutputSink_BufferedAdmin) isOutputSink_OutputSinkType() {}

func (*OutputSink_CustomSink) isOutputSink_OutputSinkType() {}

// Streaming admin sink configuration.
type StreamingAdminSink struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamingAdminSink) Reset() {
	*x = StreamingAdminSink{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamingAdminSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamingAdminSink) ProtoMessage() {}

func (x *StreamingAdminSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamingAdminSink.ProtoReflect.Descriptor instead.
func (*StreamingAdminSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{6}
}

// BufferedAdminSink configures a tap output to collect traces without returning them until
// one of multiple criteria are satisfied.
// Similar to StreamingAdminSink, it is only allowed to specify the buffered admin output
// sink if the tap is being configured from the “/tap“ admin endpoint.
type BufferedAdminSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Stop collecting traces when the specified number are collected.
	// If other criteria for ending collection are reached first, this value will not be used.
	MaxTraces uint64 `protobuf:"varint,1,opt,name=max_traces,json=maxTraces,proto3" json:"max_traces,omitempty"`
	// Acts as a fallback to prevent the client from waiting for long periods of time.
	// After timeout has occurred, a buffer flush will be triggered, returning the traces buffered so far.
	// This may result in returning fewer traces than were requested, and in the case that no traces are
	// buffered during this time, no traces will be returned.
	// Specifying 0 for the timeout value (or not specifying a value at all) indicates an infinite timeout.
	Timeout       *durationpb.Duration `protobuf:"bytes,2,opt,name=timeout,proto3" json:"timeout,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BufferedAdminSink) Reset() {
	*x = BufferedAdminSink{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BufferedAdminSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BufferedAdminSink) ProtoMessage() {}

func (x *BufferedAdminSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BufferedAdminSink.ProtoReflect.Descriptor instead.
func (*BufferedAdminSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{7}
}

func (x *BufferedAdminSink) GetMaxTraces() uint64 {
	if x != nil {
		return x.MaxTraces
	}
	return 0
}

func (x *BufferedAdminSink) GetTimeout() *durationpb.Duration {
	if x != nil {
		return x.Timeout
	}
	return nil
}

// The file per tap sink outputs a discrete file for every tapped stream.
type FilePerTapSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Path prefix. The output file will be of the form <path_prefix>_<id>.pb, where <id> is an
	// identifier distinguishing the recorded trace for stream instances (the Envoy
	// connection ID, HTTP stream ID, etc.).
	PathPrefix    string `protobuf:"bytes,1,opt,name=path_prefix,json=pathPrefix,proto3" json:"path_prefix,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FilePerTapSink) Reset() {
	*x = FilePerTapSink{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FilePerTapSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FilePerTapSink) ProtoMessage() {}

func (x *FilePerTapSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FilePerTapSink.ProtoReflect.Descriptor instead.
func (*FilePerTapSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{8}
}

func (x *FilePerTapSink) GetPathPrefix() string {
	if x != nil {
		return x.PathPrefix
	}
	return ""
}

// [#not-implemented-hide:] Streaming gRPC sink configuration sends the taps to an external gRPC
// server.
type StreamingGrpcSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Opaque identifier, that will be sent back to the streaming grpc server.
	TapId string `protobuf:"bytes,1,opt,name=tap_id,json=tapId,proto3" json:"tap_id,omitempty"`
	// The gRPC server that hosts the Tap Sink Service.
	GrpcService   *v31.GrpcService `protobuf:"bytes,2,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamingGrpcSink) Reset() {
	*x = StreamingGrpcSink{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamingGrpcSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamingGrpcSink) ProtoMessage() {}

func (x *StreamingGrpcSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamingGrpcSink.ProtoReflect.Descriptor instead.
func (*StreamingGrpcSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{9}
}

func (x *StreamingGrpcSink) GetTapId() string {
	if x != nil {
		return x.TapId
	}
	return ""
}

func (x *StreamingGrpcSink) GetGrpcService() *v31.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

// A set of match configurations used for logical operations.
type MatchPredicate_MatchSet struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The list of rules that make up the set.
	Rules         []*MatchPredicate `protobuf:"bytes,1,rep,name=rules,proto3" json:"rules,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MatchPredicate_MatchSet) Reset() {
	*x = MatchPredicate_MatchSet{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MatchPredicate_MatchSet) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MatchPredicate_MatchSet) ProtoMessage() {}

func (x *MatchPredicate_MatchSet) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MatchPredicate_MatchSet.ProtoReflect.Descriptor instead.
func (*MatchPredicate_MatchSet) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{1, 0}
}

func (x *MatchPredicate_MatchSet) GetRules() []*MatchPredicate {
	if x != nil {
		return x.Rules
	}
	return nil
}

type HttpGenericBodyMatch_GenericTextMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Rule:
	//
	//	*HttpGenericBodyMatch_GenericTextMatch_StringMatch
	//	*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch
	Rule          isHttpGenericBodyMatch_GenericTextMatch_Rule `protobuf_oneof:"rule"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpGenericBodyMatch_GenericTextMatch) Reset() {
	*x = HttpGenericBodyMatch_GenericTextMatch{}
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpGenericBodyMatch_GenericTextMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpGenericBodyMatch_GenericTextMatch) ProtoMessage() {}

func (x *HttpGenericBodyMatch_GenericTextMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_tap_v3_common_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpGenericBodyMatch_GenericTextMatch.ProtoReflect.Descriptor instead.
func (*HttpGenericBodyMatch_GenericTextMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_tap_v3_common_proto_rawDescGZIP(), []int{3, 0}
}

func (x *HttpGenericBodyMatch_GenericTextMatch) GetRule() isHttpGenericBodyMatch_GenericTextMatch_Rule {
	if x != nil {
		return x.Rule
	}
	return nil
}

func (x *HttpGenericBodyMatch_GenericTextMatch) GetStringMatch() string {
	if x != nil {
		if x, ok := x.Rule.(*HttpGenericBodyMatch_GenericTextMatch_StringMatch); ok {
			return x.StringMatch
		}
	}
	return ""
}

func (x *HttpGenericBodyMatch_GenericTextMatch) GetBinaryMatch() []byte {
	if x != nil {
		if x, ok := x.Rule.(*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch); ok {
			return x.BinaryMatch
		}
	}
	return nil
}

type isHttpGenericBodyMatch_GenericTextMatch_Rule interface {
	isHttpGenericBodyMatch_GenericTextMatch_Rule()
}

type HttpGenericBodyMatch_GenericTextMatch_StringMatch struct {
	// Text string to be located in HTTP body.
	StringMatch string `protobuf:"bytes,1,opt,name=string_match,json=stringMatch,proto3,oneof"`
}

type HttpGenericBodyMatch_GenericTextMatch_BinaryMatch struct {
	// Sequence of bytes to be located in HTTP body.
	BinaryMatch []byte `protobuf:"bytes,2,opt,name=binary_match,json=binaryMatch,proto3,oneof"`
}

func (*HttpGenericBodyMatch_GenericTextMatch_StringMatch) isHttpGenericBodyMatch_GenericTextMatch_Rule() {
}

func (*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch) isHttpGenericBodyMatch_GenericTextMatch_Rule() {
}

var File_envoy_config_tap_v3_common_proto protoreflect.FileDescriptor

const file_envoy_config_tap_v3_common_proto_rawDesc = "" +
	"\n" +
	" envoy/config/tap/v3/common.proto\x12\x13envoy.config.tap.v3\x1a,envoy/config/common/matcher/v3/matcher.proto\x1a\x1fenvoy/config/core/v3/base.proto\x1a$envoy/config/core/v3/extension.proto\x1a'envoy/config/core/v3/grpc_service.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xf5\x02\n" +
	"\tTapConfig\x12S\n" +
	"\fmatch_config\x18\x01 \x01(\v2#.envoy.config.tap.v3.MatchPredicateB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\vmatchConfig\x12D\n" +
	"\x05match\x18\x04 \x01(\v2..envoy.config.common.matcher.v3.MatchPredicateR\x05match\x12P\n" +
	"\routput_config\x18\x02 \x01(\v2!.envoy.config.tap.v3.OutputConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\foutputConfig\x12O\n" +
	"\vtap_enabled\x18\x03 \x01(\v2..envoy.config.core.v3.RuntimeFractionalPercentR\n" +
	"tapEnabled:*\x9aň\x1e%\n" +
	"#envoy.service.tap.v2alpha.TapConfig\"\xe6\b\n" +
	"\x0eMatchPredicate\x12I\n" +
	"\bor_match\x18\x01 \x01(\v2,.envoy.config.tap.v3.MatchPredicate.MatchSetH\x00R\aorMatch\x12K\n" +
	"\tand_match\x18\x02 \x01(\v2,.envoy.config.tap.v3.MatchPredicate.MatchSetH\x00R\bandMatch\x12B\n" +
	"\tnot_match\x18\x03 \x01(\v2#.envoy.config.tap.v3.MatchPredicateH\x00R\bnotMatch\x12&\n" +
	"\tany_match\x18\x04 \x01(\bB\a\xfaB\x04j\x02\b\x01H\x00R\banyMatch\x12d\n" +
	"\x1ahttp_request_headers_match\x18\x05 \x01(\v2%.envoy.config.tap.v3.HttpHeadersMatchH\x00R\x17httpRequestHeadersMatch\x12f\n" +
	"\x1bhttp_request_trailers_match\x18\x06 \x01(\v2%.envoy.config.tap.v3.HttpHeadersMatchH\x00R\x18httpRequestTrailersMatch\x12f\n" +
	"\x1bhttp_response_headers_match\x18\a \x01(\v2%.envoy.config.tap.v3.HttpHeadersMatchH\x00R\x18httpResponseHeadersMatch\x12h\n" +
	"\x1chttp_response_trailers_match\x18\b \x01(\v2%.envoy.config.tap.v3.HttpHeadersMatchH\x00R\x19httpResponseTrailersMatch\x12q\n" +
	"\x1fhttp_request_generic_body_match\x18\t \x01(\v2).envoy.config.tap.v3.HttpGenericBodyMatchH\x00R\x1bhttpRequestGenericBodyMatch\x12s\n" +
	" http_response_generic_body_match\x18\n" +
	" \x01(\v2).envoy.config.tap.v3.HttpGenericBodyMatchH\x00R\x1chttpResponseGenericBodyMatch\x1a\x89\x01\n" +
	"\bMatchSet\x12C\n" +
	"\x05rules\x18\x01 \x03(\v2#.envoy.config.tap.v3.MatchPredicateB\b\xfaB\x05\x92\x01\x02\b\x02R\x05rules:8\x9aň\x1e3\n" +
	"1envoy.service.tap.v2alpha.MatchPredicate.MatchSet:/\x9aň\x1e*\n" +
	"(envoy.service.tap.v2alpha.MatchPredicateB\v\n" +
	"\x04rule\x12\x03\xf8B\x01\"\x85\x01\n" +
	"\x10HttpHeadersMatch\x12>\n" +
	"\aheaders\x18\x01 \x03(\v2$.envoy.config.route.v3.HeaderMatcherR\aheaders:1\x9aň\x1e,\n" +
	"*envoy.service.tap.v2alpha.HttpHeadersMatch\"\x96\x02\n" +
	"\x14HttpGenericBodyMatch\x12\x1f\n" +
	"\vbytes_limit\x18\x01 \x01(\rR\n" +
	"bytesLimit\x12`\n" +
	"\bpatterns\x18\x02 \x03(\v2:.envoy.config.tap.v3.HttpGenericBodyMatch.GenericTextMatchB\b\xfaB\x05\x92\x01\x02\b\x01R\bpatterns\x1a{\n" +
	"\x10GenericTextMatch\x12,\n" +
	"\fstring_match\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\vstringMatch\x12,\n" +
	"\fbinary_match\x18\x02 \x01(\fB\a\xfaB\x04z\x02\x10\x01H\x00R\vbinaryMatchB\v\n" +
	"\x04rule\x12\x03\xf8B\x01\"\x95\x03\n" +
	"\fOutputConfig\x12A\n" +
	"\x05sinks\x18\x01 \x03(\v2\x1f.envoy.config.tap.v3.OutputSinkB\n" +
	"\xfaB\a\x92\x01\x04\b\x01\x10\x01R\x05sinks\x12O\n" +
	"\x15max_buffered_rx_bytes\x18\x02 \x01(\v2\x1c.google.protobuf.UInt32ValueR\x12maxBufferedRxBytes\x12O\n" +
	"\x15max_buffered_tx_bytes\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueR\x12maxBufferedTxBytes\x12\x1c\n" +
	"\tstreaming\x18\x04 \x01(\bR\tstreaming\x12S\n" +
	"\x17min_streamed_sent_bytes\x18\x05 \x01(\v2\x1c.google.protobuf.UInt32ValueR\x14minStreamedSentBytes:-\x9aň\x1e(\n" +
	"&envoy.service.tap.v2alpha.OutputConfig\"\xaa\x05\n" +
	"\n" +
	"OutputSink\x12H\n" +
	"\x06format\x18\x01 \x01(\x0e2&.envoy.config.tap.v3.OutputSink.FormatB\b\xfaB\x05\x82\x01\x02\x10\x01R\x06format\x12R\n" +
	"\x0fstreaming_admin\x18\x02 \x01(\v2'.envoy.config.tap.v3.StreamingAdminSinkH\x00R\x0estreamingAdmin\x12G\n" +
	"\ffile_per_tap\x18\x03 \x01(\v2#.envoy.config.tap.v3.FilePerTapSinkH\x00R\n" +
	"filePerTap\x12O\n" +
	"\x0estreaming_grpc\x18\x04 \x01(\v2&.envoy.config.tap.v3.StreamingGrpcSinkH\x00R\rstreamingGrpc\x12O\n" +
	"\x0ebuffered_admin\x18\x05 \x01(\v2&.envoy.config.tap.v3.BufferedAdminSinkH\x00R\rbufferedAdmin\x12M\n" +
	"\vcustom_sink\x18\x06 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigH\x00R\n" +
	"customSink\"~\n" +
	"\x06Format\x12\x16\n" +
	"\x12JSON_BODY_AS_BYTES\x10\x00\x12\x17\n" +
	"\x13JSON_BODY_AS_STRING\x10\x01\x12\x10\n" +
	"\fPROTO_BINARY\x10\x02\x12!\n" +
	"\x1dPROTO_BINARY_LENGTH_DELIMITED\x10\x03\x12\x0e\n" +
	"\n" +
	"PROTO_TEXT\x10\x04:+\x9aň\x1e&\n" +
	"$envoy.service.tap.v2alpha.OutputSinkB\x17\n" +
	"\x10output_sink_type\x12\x03\xf8B\x01\"I\n" +
	"\x12StreamingAdminSink:3\x9aň\x1e.\n" +
	",envoy.service.tap.v2alpha.StreamingAdminSink\"p\n" +
	"\x11BufferedAdminSink\x12&\n" +
	"\n" +
	"max_traces\x18\x01 \x01(\x04B\a\xfaB\x042\x02 \x00R\tmaxTraces\x123\n" +
	"\atimeout\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\atimeout\"k\n" +
	"\x0eFilePerTapSink\x12(\n" +
	"\vpath_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"pathPrefix:/\x9aň\x1e*\n" +
	"(envoy.service.tap.v2alpha.FilePerTapSink\"\xae\x01\n" +
	"\x11StreamingGrpcSink\x12\x15\n" +
	"\x06tap_id\x18\x01 \x01(\tR\x05tapId\x12N\n" +
	"\fgrpc_service\x18\x02 \x01(\v2!.envoy.config.core.v3.GrpcServiceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vgrpcService:2\x9aň\x1e-\n" +
	"+envoy.service.tap.v2alpha.StreamingGrpcSinkB|\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"!io.envoyproxy.envoy.config.tap.v3B\vCommonProtoP\x01Z@github.com/envoyproxy/go-control-plane/envoy/config/tap/v3;tapv3b\x06proto3"

var (
	file_envoy_config_tap_v3_common_proto_rawDescOnce sync.Once
	file_envoy_config_tap_v3_common_proto_rawDescData []byte
)

func file_envoy_config_tap_v3_common_proto_rawDescGZIP() []byte {
	file_envoy_config_tap_v3_common_proto_rawDescOnce.Do(func() {
		file_envoy_config_tap_v3_common_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_tap_v3_common_proto_rawDesc), len(file_envoy_config_tap_v3_common_proto_rawDesc)))
	})
	return file_envoy_config_tap_v3_common_proto_rawDescData
}

var file_envoy_config_tap_v3_common_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_config_tap_v3_common_proto_msgTypes = make([]protoimpl.MessageInfo, 12)
var file_envoy_config_tap_v3_common_proto_goTypes = []any{
	(OutputSink_Format)(0),                        // 0: envoy.config.tap.v3.OutputSink.Format
	(*TapConfig)(nil),                             // 1: envoy.config.tap.v3.TapConfig
	(*MatchPredicate)(nil),                        // 2: envoy.config.tap.v3.MatchPredicate
	(*HttpHeadersMatch)(nil),                      // 3: envoy.config.tap.v3.HttpHeadersMatch
	(*HttpGenericBodyMatch)(nil),                  // 4: envoy.config.tap.v3.HttpGenericBodyMatch
	(*OutputConfig)(nil),                          // 5: envoy.config.tap.v3.OutputConfig
	(*OutputSink)(nil),                            // 6: envoy.config.tap.v3.OutputSink
	(*StreamingAdminSink)(nil),                    // 7: envoy.config.tap.v3.StreamingAdminSink
	(*BufferedAdminSink)(nil),                     // 8: envoy.config.tap.v3.BufferedAdminSink
	(*FilePerTapSink)(nil),                        // 9: envoy.config.tap.v3.FilePerTapSink
	(*StreamingGrpcSink)(nil),                     // 10: envoy.config.tap.v3.StreamingGrpcSink
	(*MatchPredicate_MatchSet)(nil),               // 11: envoy.config.tap.v3.MatchPredicate.MatchSet
	(*HttpGenericBodyMatch_GenericTextMatch)(nil), // 12: envoy.config.tap.v3.HttpGenericBodyMatch.GenericTextMatch
	(*v3.MatchPredicate)(nil),                     // 13: envoy.config.common.matcher.v3.MatchPredicate
	(*v31.RuntimeFractionalPercent)(nil),          // 14: envoy.config.core.v3.RuntimeFractionalPercent
	(*v32.HeaderMatcher)(nil),                     // 15: envoy.config.route.v3.HeaderMatcher
	(*wrapperspb.UInt32Value)(nil),                // 16: google.protobuf.UInt32Value
	(*v31.TypedExtensionConfig)(nil),              // 17: envoy.config.core.v3.TypedExtensionConfig
	(*durationpb.Duration)(nil),                   // 18: google.protobuf.Duration
	(*v31.GrpcService)(nil),                       // 19: envoy.config.core.v3.GrpcService
}
var file_envoy_config_tap_v3_common_proto_depIdxs = []int32{
	2,  // 0: envoy.config.tap.v3.TapConfig.match_config:type_name -> envoy.config.tap.v3.MatchPredicate
	13, // 1: envoy.config.tap.v3.TapConfig.match:type_name -> envoy.config.common.matcher.v3.MatchPredicate
	5,  // 2: envoy.config.tap.v3.TapConfig.output_config:type_name -> envoy.config.tap.v3.OutputConfig
	14, // 3: envoy.config.tap.v3.TapConfig.tap_enabled:type_name -> envoy.config.core.v3.RuntimeFractionalPercent
	11, // 4: envoy.config.tap.v3.MatchPredicate.or_match:type_name -> envoy.config.tap.v3.MatchPredicate.MatchSet
	11, // 5: envoy.config.tap.v3.MatchPredicate.and_match:type_name -> envoy.config.tap.v3.MatchPredicate.MatchSet
	2,  // 6: envoy.config.tap.v3.MatchPredicate.not_match:type_name -> envoy.config.tap.v3.MatchPredicate
	3,  // 7: envoy.config.tap.v3.MatchPredicate.http_request_headers_match:type_name -> envoy.config.tap.v3.HttpHeadersMatch
	3,  // 8: envoy.config.tap.v3.MatchPredicate.http_request_trailers_match:type_name -> envoy.config.tap.v3.HttpHeadersMatch
	3,  // 9: envoy.config.tap.v3.MatchPredicate.http_response_headers_match:type_name -> envoy.config.tap.v3.HttpHeadersMatch
	3,  // 10: envoy.config.tap.v3.MatchPredicate.http_response_trailers_match:type_name -> envoy.config.tap.v3.HttpHeadersMatch
	4,  // 11: envoy.config.tap.v3.MatchPredicate.http_request_generic_body_match:type_name -> envoy.config.tap.v3.HttpGenericBodyMatch
	4,  // 12: envoy.config.tap.v3.MatchPredicate.http_response_generic_body_match:type_name -> envoy.config.tap.v3.HttpGenericBodyMatch
	15, // 13: envoy.config.tap.v3.HttpHeadersMatch.headers:type_name -> envoy.config.route.v3.HeaderMatcher
	12, // 14: envoy.config.tap.v3.HttpGenericBodyMatch.patterns:type_name -> envoy.config.tap.v3.HttpGenericBodyMatch.GenericTextMatch
	6,  // 15: envoy.config.tap.v3.OutputConfig.sinks:type_name -> envoy.config.tap.v3.OutputSink
	16, // 16: envoy.config.tap.v3.OutputConfig.max_buffered_rx_bytes:type_name -> google.protobuf.UInt32Value
	16, // 17: envoy.config.tap.v3.OutputConfig.max_buffered_tx_bytes:type_name -> google.protobuf.UInt32Value
	16, // 18: envoy.config.tap.v3.OutputConfig.min_streamed_sent_bytes:type_name -> google.protobuf.UInt32Value
	0,  // 19: envoy.config.tap.v3.OutputSink.format:type_name -> envoy.config.tap.v3.OutputSink.Format
	7,  // 20: envoy.config.tap.v3.OutputSink.streaming_admin:type_name -> envoy.config.tap.v3.StreamingAdminSink
	9,  // 21: envoy.config.tap.v3.OutputSink.file_per_tap:type_name -> envoy.config.tap.v3.FilePerTapSink
	10, // 22: envoy.config.tap.v3.OutputSink.streaming_grpc:type_name -> envoy.config.tap.v3.StreamingGrpcSink
	8,  // 23: envoy.config.tap.v3.OutputSink.buffered_admin:type_name -> envoy.config.tap.v3.BufferedAdminSink
	17, // 24: envoy.config.tap.v3.OutputSink.custom_sink:type_name -> envoy.config.core.v3.TypedExtensionConfig
	18, // 25: envoy.config.tap.v3.BufferedAdminSink.timeout:type_name -> google.protobuf.Duration
	19, // 26: envoy.config.tap.v3.StreamingGrpcSink.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	2,  // 27: envoy.config.tap.v3.MatchPredicate.MatchSet.rules:type_name -> envoy.config.tap.v3.MatchPredicate
	28, // [28:28] is the sub-list for method output_type
	28, // [28:28] is the sub-list for method input_type
	28, // [28:28] is the sub-list for extension type_name
	28, // [28:28] is the sub-list for extension extendee
	0,  // [0:28] is the sub-list for field type_name
}

func init() { file_envoy_config_tap_v3_common_proto_init() }
func file_envoy_config_tap_v3_common_proto_init() {
	if File_envoy_config_tap_v3_common_proto != nil {
		return
	}
	file_envoy_config_tap_v3_common_proto_msgTypes[1].OneofWrappers = []any{
		(*MatchPredicate_OrMatch)(nil),
		(*MatchPredicate_AndMatch)(nil),
		(*MatchPredicate_NotMatch)(nil),
		(*MatchPredicate_AnyMatch)(nil),
		(*MatchPredicate_HttpRequestHeadersMatch)(nil),
		(*MatchPredicate_HttpRequestTrailersMatch)(nil),
		(*MatchPredicate_HttpResponseHeadersMatch)(nil),
		(*MatchPredicate_HttpResponseTrailersMatch)(nil),
		(*MatchPredicate_HttpRequestGenericBodyMatch)(nil),
		(*MatchPredicate_HttpResponseGenericBodyMatch)(nil),
	}
	file_envoy_config_tap_v3_common_proto_msgTypes[5].OneofWrappers = []any{
		(*OutputSink_StreamingAdmin)(nil),
		(*OutputSink_FilePerTap)(nil),
		(*OutputSink_StreamingGrpc)(nil),
		(*OutputSink_BufferedAdmin)(nil),
		(*OutputSink_CustomSink)(nil),
	}
	file_envoy_config_tap_v3_common_proto_msgTypes[11].OneofWrappers = []any{
		(*HttpGenericBodyMatch_GenericTextMatch_StringMatch)(nil),
		(*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_tap_v3_common_proto_rawDesc), len(file_envoy_config_tap_v3_common_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   12,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_tap_v3_common_proto_goTypes,
		DependencyIndexes: file_envoy_config_tap_v3_common_proto_depIdxs,
		EnumInfos:         file_envoy_config_tap_v3_common_proto_enumTypes,
		MessageInfos:      file_envoy_config_tap_v3_common_proto_msgTypes,
	}.Build()
	File_envoy_config_tap_v3_common_proto = out.File
	file_envoy_config_tap_v3_common_proto_goTypes = nil
	file_envoy_config_tap_v3_common_proto_depIdxs = nil
}
