// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.30.2
// source: custom_error.proto

package jsonerror

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Error code for `CustomError`.
type CustomError_CustomErrorCode int32

const (
	// Default error.
	CustomError_CUSTOM_ERROR_CODE_UNSPECIFIED CustomError_CustomErrorCode = 0
	// Too many foo.
	CustomError_TOO_MANY_FOO CustomError_CustomErrorCode = 1
	// Not enough foo.
	CustomError_NOT_ENOUGH_FOO CustomError_CustomErrorCode = 2
	// Catastrophic error.
	CustomError_UNIVERSE_WAS_DESTROYED CustomError_CustomErrorCode = 3
)

// Enum value maps for CustomError_CustomErrorCode.
var (
	CustomError_CustomErrorCode_name = map[int32]string{
		0: "CUSTOM_ERROR_CODE_UNSPECIFIED",
		1: "TOO_MANY_FOO",
		2: "NOT_ENOUGH_FOO",
		3: "UNIVERSE_WAS_DESTROYED",
	}
	CustomError_CustomErrorCode_value = map[string]int32{
		"CUSTOM_ERROR_CODE_UNSPECIFIED": 0,
		"TOO_MANY_FOO":                  1,
		"NOT_ENOUGH_FOO":                2,
		"UNIVERSE_WAS_DESTROYED":        3,
	}
)

func (x CustomError_CustomErrorCode) Enum() *CustomError_CustomErrorCode {
	p := new(CustomError_CustomErrorCode)
	*p = x
	return p
}

func (x CustomError_CustomErrorCode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (CustomError_CustomErrorCode) Descriptor() protoreflect.EnumDescriptor {
	return file_custom_error_proto_enumTypes[0].Descriptor()
}

func (CustomError_CustomErrorCode) Type() protoreflect.EnumType {
	return &file_custom_error_proto_enumTypes[0]
}

func (x CustomError_CustomErrorCode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use CustomError_CustomErrorCode.Descriptor instead.
func (CustomError_CustomErrorCode) EnumDescriptor() ([]byte, []int) {
	return file_custom_error_proto_rawDescGZIP(), []int{0, 0}
}

// CustomError is an example of a custom error message  which may be included
// in an rpc status. It is not meant to reflect a standard error.
type CustomError struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Error code specific to the custom API being invoked.
	Code CustomError_CustomErrorCode `protobuf:"varint,1,opt,name=code,proto3,enum=error.CustomError_CustomErrorCode" json:"code,omitempty"`
	// Name of the failed entity.
	Entity string `protobuf:"bytes,2,opt,name=entity,proto3" json:"entity,omitempty"`
	// Message that describes the error.
	ErrorMessage  string `protobuf:"bytes,3,opt,name=error_message,json=errorMessage,proto3" json:"error_message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CustomError) Reset() {
	*x = CustomError{}
	mi := &file_custom_error_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CustomError) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CustomError) ProtoMessage() {}

func (x *CustomError) ProtoReflect() protoreflect.Message {
	mi := &file_custom_error_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CustomError.ProtoReflect.Descriptor instead.
func (*CustomError) Descriptor() ([]byte, []int) {
	return file_custom_error_proto_rawDescGZIP(), []int{0}
}

func (x *CustomError) GetCode() CustomError_CustomErrorCode {
	if x != nil {
		return x.Code
	}
	return CustomError_CUSTOM_ERROR_CODE_UNSPECIFIED
}

func (x *CustomError) GetEntity() string {
	if x != nil {
		return x.Entity
	}
	return ""
}

func (x *CustomError) GetErrorMessage() string {
	if x != nil {
		return x.ErrorMessage
	}
	return ""
}

var File_custom_error_proto protoreflect.FileDescriptor

const file_custom_error_proto_rawDesc = "" +
	"\n" +
	"\x12custom_error.proto\x12\x05error\"\xfa\x01\n" +
	"\vCustomError\x126\n" +
	"\x04code\x18\x01 \x01(\x0e2\".error.CustomError.CustomErrorCodeR\x04code\x12\x16\n" +
	"\x06entity\x18\x02 \x01(\tR\x06entity\x12#\n" +
	"\rerror_message\x18\x03 \x01(\tR\ferrorMessage\"v\n" +
	"\x0fCustomErrorCode\x12!\n" +
	"\x1dCUSTOM_ERROR_CODE_UNSPECIFIED\x10\x00\x12\x10\n" +
	"\fTOO_MANY_FOO\x10\x01\x12\x12\n" +
	"\x0eNOT_ENOUGH_FOO\x10\x02\x12\x1a\n" +
	"\x16UNIVERSE_WAS_DESTROYED\x10\x03BCZAgithub.com/googleapis/gax-go/v2/apierror/internal/proto;jsonerrorb\x06proto3"

var (
	file_custom_error_proto_rawDescOnce sync.Once
	file_custom_error_proto_rawDescData []byte
)

func file_custom_error_proto_rawDescGZIP() []byte {
	file_custom_error_proto_rawDescOnce.Do(func() {
		file_custom_error_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_custom_error_proto_rawDesc), len(file_custom_error_proto_rawDesc)))
	})
	return file_custom_error_proto_rawDescData
}

var file_custom_error_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_custom_error_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_custom_error_proto_goTypes = []any{
	(CustomError_CustomErrorCode)(0), // 0: error.CustomError.CustomErrorCode
	(*CustomError)(nil),              // 1: error.CustomError
}
var file_custom_error_proto_depIdxs = []int32{
	0, // 0: error.CustomError.code:type_name -> error.CustomError.CustomErrorCode
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_custom_error_proto_init() }
func file_custom_error_proto_init() {
	if File_custom_error_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_custom_error_proto_rawDesc), len(file_custom_error_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_custom_error_proto_goTypes,
		DependencyIndexes: file_custom_error_proto_depIdxs,
		EnumInfos:         file_custom_error_proto_enumTypes,
		MessageInfos:      file_custom_error_proto_msgTypes,
	}.Build()
	File_custom_error_proto = out.File
	file_custom_error_proto_goTypes = nil
	file_custom_error_proto_depIdxs = nil
}
