// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/iam/v1/options.proto

package iampb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Encapsulates settings provided to GetIamPolicy.
type GetPolicyOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. The maximum policy version that will be used to format the
	// policy.
	//
	// Valid values are 0, 1, and 3. Requests specifying an invalid value will be
	// rejected.
	//
	// Requests for policies with any conditional role bindings must specify
	// version 3. Policies with no conditional role bindings may specify any valid
	// value or leave the field unset.
	//
	// The policy in the response might use the policy version that you specified,
	// or it might use a lower policy version. For example, if you specify version
	// 3, but the policy has no conditional role bindings, the response uses
	// version 1.
	//
	// To learn which resources support conditions in their IAM policies, see the
	// [IAM
	// documentation](https://cloud.google.com/iam/help/conditions/resource-policies).
	RequestedPolicyVersion int32 `protobuf:"varint,1,opt,name=requested_policy_version,json=requestedPolicyVersion,proto3" json:"requested_policy_version,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *GetPolicyOptions) Reset() {
	*x = GetPolicyOptions{}
	mi := &file_google_iam_v1_options_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetPolicyOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetPolicyOptions) ProtoMessage() {}

func (x *GetPolicyOptions) ProtoReflect() protoreflect.Message {
	mi := &file_google_iam_v1_options_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetPolicyOptions.ProtoReflect.Descriptor instead.
func (*GetPolicyOptions) Descriptor() ([]byte, []int) {
	return file_google_iam_v1_options_proto_rawDescGZIP(), []int{0}
}

func (x *GetPolicyOptions) GetRequestedPolicyVersion() int32 {
	if x != nil {
		return x.RequestedPolicyVersion
	}
	return 0
}

var File_google_iam_v1_options_proto protoreflect.FileDescriptor

const file_google_iam_v1_options_proto_rawDesc = "" +
	"\n" +
	"\x1bgoogle/iam/v1/options.proto\x12\rgoogle.iam.v1\"L\n" +
	"\x10GetPolicyOptions\x128\n" +
	"\x18requested_policy_version\x18\x01 \x01(\x05R\x16requestedPolicyVersionB}\n" +
	"\x11com.google.iam.v1B\fOptionsProtoP\x01Z)cloud.google.com/go/iam/apiv1/iampb;iampb\xf8\x01\x01\xaa\x02\x13Google.Cloud.Iam.V1\xca\x02\x13Google\\Cloud\\Iam\\V1b\x06proto3"

var (
	file_google_iam_v1_options_proto_rawDescOnce sync.Once
	file_google_iam_v1_options_proto_rawDescData []byte
)

func file_google_iam_v1_options_proto_rawDescGZIP() []byte {
	file_google_iam_v1_options_proto_rawDescOnce.Do(func() {
		file_google_iam_v1_options_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_iam_v1_options_proto_rawDesc), len(file_google_iam_v1_options_proto_rawDesc)))
	})
	return file_google_iam_v1_options_proto_rawDescData
}

var file_google_iam_v1_options_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_google_iam_v1_options_proto_goTypes = []any{
	(*GetPolicyOptions)(nil), // 0: google.iam.v1.GetPolicyOptions
}
var file_google_iam_v1_options_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_google_iam_v1_options_proto_init() }
func file_google_iam_v1_options_proto_init() {
	if File_google_iam_v1_options_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_iam_v1_options_proto_rawDesc), len(file_google_iam_v1_options_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_iam_v1_options_proto_goTypes,
		DependencyIndexes: file_google_iam_v1_options_proto_depIdxs,
		MessageInfos:      file_google_iam_v1_options_proto_msgTypes,
	}.Build()
	File_google_iam_v1_options_proto = out.File
	file_google_iam_v1_options_proto_goTypes = nil
	file_google_iam_v1_options_proto_depIdxs = nil
}
