// Copyright 2020 Huawei Technologies Co.,Ltd.
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package region

type Region struct {
	Id        string
	Endpoints []string
}

func NewRegion(id string, endpoints ...string) *Region {
	return &Region{
		Id:        id,
		Endpoints: endpoints,
	}
}

// Deprecated: As of 0.1.27, because of the support of the multi-endpoint feature, use WithEndpointsOverride instead
func (r *Region) WithEndpointOverride(endpoint string) *Region {
	return r.WithEndpointsOverride([]string{endpoint})
}

func (r *Region) WithEndpointsOverride(endpoints []string) *Region {
	r.Endpoints = endpoints
	return r
}
