// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/core/v3/udp_socket_config.proto

package corev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Generic UDP socket configuration.
type UdpSocketConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The maximum size of received UDP datagrams. Using a larger size will cause Envoy to allocate
	// more memory per socket. Received datagrams above this size will be dropped. If not set
	// defaults to 1500 bytes.
	MaxRxDatagramSize *wrapperspb.UInt64Value `protobuf:"bytes,1,opt,name=max_rx_datagram_size,json=maxRxDatagramSize,proto3" json:"max_rx_datagram_size,omitempty"`
	// Configures whether Generic Receive Offload (GRO)
	// <https://en.wikipedia.org/wiki/Large_receive_offload>_ is preferred when reading from the
	// UDP socket. The default is context dependent and is documented where UdpSocketConfig is used.
	// This option affects performance but not functionality. If GRO is not supported by the operating
	// system, non-GRO receive will be used.
	PreferGro     *wrapperspb.BoolValue `protobuf:"bytes,2,opt,name=prefer_gro,json=preferGro,proto3" json:"prefer_gro,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UdpSocketConfig) Reset() {
	*x = UdpSocketConfig{}
	mi := &file_envoy_config_core_v3_udp_socket_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UdpSocketConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UdpSocketConfig) ProtoMessage() {}

func (x *UdpSocketConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_core_v3_udp_socket_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UdpSocketConfig.ProtoReflect.Descriptor instead.
func (*UdpSocketConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_core_v3_udp_socket_config_proto_rawDescGZIP(), []int{0}
}

func (x *UdpSocketConfig) GetMaxRxDatagramSize() *wrapperspb.UInt64Value {
	if x != nil {
		return x.MaxRxDatagramSize
	}
	return nil
}

func (x *UdpSocketConfig) GetPreferGro() *wrapperspb.BoolValue {
	if x != nil {
		return x.PreferGro
	}
	return nil
}

var File_envoy_config_core_v3_udp_socket_config_proto protoreflect.FileDescriptor

const file_envoy_config_core_v3_udp_socket_config_proto_rawDesc = "" +
	"\n" +
	",envoy/config/core/v3/udp_socket_config.proto\x12\x14envoy.config.core.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa8\x01\n" +
	"\x0fUdpSocketConfig\x12Z\n" +
	"\x14max_rx_datagram_size\x18\x01 \x01(\v2\x1c.google.protobuf.UInt64ValueB\v\xfaB\b2\x06\x10\x80\x80\x04 \x00R\x11maxRxDatagramSize\x129\n" +
	"\n" +
	"prefer_gro\x18\x02 \x01(\v2\x1a.google.protobuf.BoolValueR\tpreferGroB\x88\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\"io.envoyproxy.envoy.config.core.v3B\x14UdpSocketConfigProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3b\x06proto3"

var (
	file_envoy_config_core_v3_udp_socket_config_proto_rawDescOnce sync.Once
	file_envoy_config_core_v3_udp_socket_config_proto_rawDescData []byte
)

func file_envoy_config_core_v3_udp_socket_config_proto_rawDescGZIP() []byte {
	file_envoy_config_core_v3_udp_socket_config_proto_rawDescOnce.Do(func() {
		file_envoy_config_core_v3_udp_socket_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_udp_socket_config_proto_rawDesc), len(file_envoy_config_core_v3_udp_socket_config_proto_rawDesc)))
	})
	return file_envoy_config_core_v3_udp_socket_config_proto_rawDescData
}

var file_envoy_config_core_v3_udp_socket_config_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_core_v3_udp_socket_config_proto_goTypes = []any{
	(*UdpSocketConfig)(nil),        // 0: envoy.config.core.v3.UdpSocketConfig
	(*wrapperspb.UInt64Value)(nil), // 1: google.protobuf.UInt64Value
	(*wrapperspb.BoolValue)(nil),   // 2: google.protobuf.BoolValue
}
var file_envoy_config_core_v3_udp_socket_config_proto_depIdxs = []int32{
	1, // 0: envoy.config.core.v3.UdpSocketConfig.max_rx_datagram_size:type_name -> google.protobuf.UInt64Value
	2, // 1: envoy.config.core.v3.UdpSocketConfig.prefer_gro:type_name -> google.protobuf.BoolValue
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_config_core_v3_udp_socket_config_proto_init() }
func file_envoy_config_core_v3_udp_socket_config_proto_init() {
	if File_envoy_config_core_v3_udp_socket_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_udp_socket_config_proto_rawDesc), len(file_envoy_config_core_v3_udp_socket_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_core_v3_udp_socket_config_proto_goTypes,
		DependencyIndexes: file_envoy_config_core_v3_udp_socket_config_proto_depIdxs,
		MessageInfos:      file_envoy_config_core_v3_udp_socket_config_proto_msgTypes,
	}.Build()
	File_envoy_config_core_v3_udp_socket_config_proto = out.File
	file_envoy_config_core_v3_udp_socket_config_proto_goTypes = nil
	file_envoy_config_core_v3_udp_socket_config_proto_depIdxs = nil
}
