// Code generated by "enumer -type=KernelModuleType -linecomment -text"; DO NOT EDIT.

package runtime

import (
	"fmt"
	"strings"
)

const _KernelModuleTypeName = "unknownbuilt-indynamic"

var _KernelModuleTypeIndex = [...]uint8{0, 7, 15, 22}

const _KernelModuleTypeLowerName = "unknownbuilt-indynamic"

func (i KernelModuleType) String() string {
	if i < 0 || i >= KernelModuleType(len(_KernelModuleTypeIndex)-1) {
		return fmt.Sprintf("KernelModuleType(%d)", i)
	}
	return _KernelModuleTypeName[_KernelModuleTypeIndex[i]:_KernelModuleTypeIndex[i+1]]
}

// An "invalid array index" compiler error signifies that the constant values have changed.
// Re-run the stringer command to generate them again.
func _KernelModuleTypeNoOp() {
	var x [1]struct{}
	_ = x[KernelModuleTypeUnknown-(0)]
	_ = x[KernelModuleTypeBuiltin-(1)]
	_ = x[KernelModuleTypeDynamic-(2)]
}

var _KernelModuleTypeValues = []KernelModuleType{KernelModuleTypeUnknown, KernelModuleTypeBuiltin, KernelModuleTypeDynamic}

var _KernelModuleTypeNameToValueMap = map[string]KernelModuleType{
	_KernelModuleTypeName[0:7]:        KernelModuleTypeUnknown,
	_KernelModuleTypeLowerName[0:7]:   KernelModuleTypeUnknown,
	_KernelModuleTypeName[7:15]:       KernelModuleTypeBuiltin,
	_KernelModuleTypeLowerName[7:15]:  KernelModuleTypeBuiltin,
	_KernelModuleTypeName[15:22]:      KernelModuleTypeDynamic,
	_KernelModuleTypeLowerName[15:22]: KernelModuleTypeDynamic,
}

var _KernelModuleTypeNames = []string{
	_KernelModuleTypeName[0:7],
	_KernelModuleTypeName[7:15],
	_KernelModuleTypeName[15:22],
}

// KernelModuleTypeString retrieves an enum value from the enum constants string name.
// Throws an error if the param is not part of the enum.
func KernelModuleTypeString(s string) (KernelModuleType, error) {
	if val, ok := _KernelModuleTypeNameToValueMap[s]; ok {
		return val, nil
	}

	if val, ok := _KernelModuleTypeNameToValueMap[strings.ToLower(s)]; ok {
		return val, nil
	}
	return 0, fmt.Errorf("%s does not belong to KernelModuleType values", s)
}

// KernelModuleTypeValues returns all values of the enum
func KernelModuleTypeValues() []KernelModuleType {
	return _KernelModuleTypeValues
}

// KernelModuleTypeStrings returns a slice of all String values of the enum
func KernelModuleTypeStrings() []string {
	strs := make([]string, len(_KernelModuleTypeNames))
	copy(strs, _KernelModuleTypeNames)
	return strs
}

// IsAKernelModuleType returns "true" if the value is listed in the enum definition. "false" otherwise
func (i KernelModuleType) IsAKernelModuleType() bool {
	for _, v := range _KernelModuleTypeValues {
		if i == v {
			return true
		}
	}
	return false
}

// MarshalText implements the encoding.TextMarshaler interface for KernelModuleType
func (i KernelModuleType) MarshalText() ([]byte, error) {
	return []byte(i.String()), nil
}

// UnmarshalText implements the encoding.TextUnmarshaler interface for KernelModuleType
func (i *KernelModuleType) UnmarshalText(text []byte) error {
	var err error
	*i, err = KernelModuleTypeString(string(text))
	return err
}
