// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/data/orca/v3/orca_load_report.proto

package v3

import (
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type OrcaLoadReport struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	CpuUtilization float64                `protobuf:"fixed64,1,opt,name=cpu_utilization,json=cpuUtilization,proto3" json:"cpu_utilization,omitempty"`
	MemUtilization float64                `protobuf:"fixed64,2,opt,name=mem_utilization,json=memUtilization,proto3" json:"mem_utilization,omitempty"`
	// Deprecated: Marked as deprecated in xds/data/orca/v3/orca_load_report.proto.
	Rps                    uint64             `protobuf:"varint,3,opt,name=rps,proto3" json:"rps,omitempty"`
	RequestCost            map[string]float64 `protobuf:"bytes,4,rep,name=request_cost,json=requestCost,proto3" json:"request_cost,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"fixed64,2,opt,name=value"`
	Utilization            map[string]float64 `protobuf:"bytes,5,rep,name=utilization,proto3" json:"utilization,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"fixed64,2,opt,name=value"`
	RpsFractional          float64            `protobuf:"fixed64,6,opt,name=rps_fractional,json=rpsFractional,proto3" json:"rps_fractional,omitempty"`
	Eps                    float64            `protobuf:"fixed64,7,opt,name=eps,proto3" json:"eps,omitempty"`
	NamedMetrics           map[string]float64 `protobuf:"bytes,8,rep,name=named_metrics,json=namedMetrics,proto3" json:"named_metrics,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"fixed64,2,opt,name=value"`
	ApplicationUtilization float64            `protobuf:"fixed64,9,opt,name=application_utilization,json=applicationUtilization,proto3" json:"application_utilization,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *OrcaLoadReport) Reset() {
	*x = OrcaLoadReport{}
	mi := &file_xds_data_orca_v3_orca_load_report_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OrcaLoadReport) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OrcaLoadReport) ProtoMessage() {}

func (x *OrcaLoadReport) ProtoReflect() protoreflect.Message {
	mi := &file_xds_data_orca_v3_orca_load_report_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OrcaLoadReport.ProtoReflect.Descriptor instead.
func (*OrcaLoadReport) Descriptor() ([]byte, []int) {
	return file_xds_data_orca_v3_orca_load_report_proto_rawDescGZIP(), []int{0}
}

func (x *OrcaLoadReport) GetCpuUtilization() float64 {
	if x != nil {
		return x.CpuUtilization
	}
	return 0
}

func (x *OrcaLoadReport) GetMemUtilization() float64 {
	if x != nil {
		return x.MemUtilization
	}
	return 0
}

// Deprecated: Marked as deprecated in xds/data/orca/v3/orca_load_report.proto.
func (x *OrcaLoadReport) GetRps() uint64 {
	if x != nil {
		return x.Rps
	}
	return 0
}

func (x *OrcaLoadReport) GetRequestCost() map[string]float64 {
	if x != nil {
		return x.RequestCost
	}
	return nil
}

func (x *OrcaLoadReport) GetUtilization() map[string]float64 {
	if x != nil {
		return x.Utilization
	}
	return nil
}

func (x *OrcaLoadReport) GetRpsFractional() float64 {
	if x != nil {
		return x.RpsFractional
	}
	return 0
}

func (x *OrcaLoadReport) GetEps() float64 {
	if x != nil {
		return x.Eps
	}
	return 0
}

func (x *OrcaLoadReport) GetNamedMetrics() map[string]float64 {
	if x != nil {
		return x.NamedMetrics
	}
	return nil
}

func (x *OrcaLoadReport) GetApplicationUtilization() float64 {
	if x != nil {
		return x.ApplicationUtilization
	}
	return 0
}

var File_xds_data_orca_v3_orca_load_report_proto protoreflect.FileDescriptor

const file_xds_data_orca_v3_orca_load_report_proto_rawDesc = "" +
	"\n" +
	"'xds/data/orca/v3/orca_load_report.proto\x12\x10xds.data.orca.v3\x1a\x17validate/validate.proto\"\xa6\x06\n" +
	"\x0eOrcaLoadReport\x127\n" +
	"\x0fcpu_utilization\x18\x01 \x01(\x01B\x0e\xfaB\v\x12\t)\x00\x00\x00\x00\x00\x00\x00\x00R\x0ecpuUtilization\x12@\n" +
	"\x0fmem_utilization\x18\x02 \x01(\x01B\x17\xfaB\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00\xf0?)\x00\x00\x00\x00\x00\x00\x00\x00R\x0ememUtilization\x12\x14\n" +
	"\x03rps\x18\x03 \x01(\x04B\x02\x18\x01R\x03rps\x12T\n" +
	"\frequest_cost\x18\x04 \x03(\v21.xds.data.orca.v3.OrcaLoadReport.RequestCostEntryR\vrequestCost\x12q\n" +
	"\vutilization\x18\x05 \x03(\v21.xds.data.orca.v3.OrcaLoadReport.UtilizationEntryB\x1c\xfaB\x19\x9a\x01\x16*\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00\xf0?)\x00\x00\x00\x00\x00\x00\x00\x00R\vutilization\x125\n" +
	"\x0erps_fractional\x18\x06 \x01(\x01B\x0e\xfaB\v\x12\t)\x00\x00\x00\x00\x00\x00\x00\x00R\rrpsFractional\x12 \n" +
	"\x03eps\x18\a \x01(\x01B\x0e\xfaB\v\x12\t)\x00\x00\x00\x00\x00\x00\x00\x00R\x03eps\x12W\n" +
	"\rnamed_metrics\x18\b \x03(\v22.xds.data.orca.v3.OrcaLoadReport.NamedMetricsEntryR\fnamedMetrics\x12G\n" +
	"\x17application_utilization\x18\t \x01(\x01B\x0e\xfaB\v\x12\t)\x00\x00\x00\x00\x00\x00\x00\x00R\x16applicationUtilization\x1a>\n" +
	"\x10RequestCostEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\x01R\x05value:\x028\x01\x1a>\n" +
	"\x10UtilizationEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\x01R\x05value:\x028\x01\x1a?\n" +
	"\x11NamedMetricsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\x01R\x05value:\x028\x01B]\n" +
	"\x1bcom.github.xds.data.orca.v3B\x13OrcaLoadReportProtoP\x01Z'github.com/cncf/xds/go/xds/data/orca/v3b\x06proto3"

var (
	file_xds_data_orca_v3_orca_load_report_proto_rawDescOnce sync.Once
	file_xds_data_orca_v3_orca_load_report_proto_rawDescData []byte
)

func file_xds_data_orca_v3_orca_load_report_proto_rawDescGZIP() []byte {
	file_xds_data_orca_v3_orca_load_report_proto_rawDescOnce.Do(func() {
		file_xds_data_orca_v3_orca_load_report_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_data_orca_v3_orca_load_report_proto_rawDesc), len(file_xds_data_orca_v3_orca_load_report_proto_rawDesc)))
	})
	return file_xds_data_orca_v3_orca_load_report_proto_rawDescData
}

var file_xds_data_orca_v3_orca_load_report_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_xds_data_orca_v3_orca_load_report_proto_goTypes = []any{
	(*OrcaLoadReport)(nil), // 0: xds.data.orca.v3.OrcaLoadReport
	nil,                    // 1: xds.data.orca.v3.OrcaLoadReport.RequestCostEntry
	nil,                    // 2: xds.data.orca.v3.OrcaLoadReport.UtilizationEntry
	nil,                    // 3: xds.data.orca.v3.OrcaLoadReport.NamedMetricsEntry
}
var file_xds_data_orca_v3_orca_load_report_proto_depIdxs = []int32{
	1, // 0: xds.data.orca.v3.OrcaLoadReport.request_cost:type_name -> xds.data.orca.v3.OrcaLoadReport.RequestCostEntry
	2, // 1: xds.data.orca.v3.OrcaLoadReport.utilization:type_name -> xds.data.orca.v3.OrcaLoadReport.UtilizationEntry
	3, // 2: xds.data.orca.v3.OrcaLoadReport.named_metrics:type_name -> xds.data.orca.v3.OrcaLoadReport.NamedMetricsEntry
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_xds_data_orca_v3_orca_load_report_proto_init() }
func file_xds_data_orca_v3_orca_load_report_proto_init() {
	if File_xds_data_orca_v3_orca_load_report_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_data_orca_v3_orca_load_report_proto_rawDesc), len(file_xds_data_orca_v3_orca_load_report_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_data_orca_v3_orca_load_report_proto_goTypes,
		DependencyIndexes: file_xds_data_orca_v3_orca_load_report_proto_depIdxs,
		MessageInfos:      file_xds_data_orca_v3_orca_load_report_proto_msgTypes,
	}.Build()
	File_xds_data_orca_v3_orca_load_report_proto = out.File
	file_xds_data_orca_v3_orca_load_report_proto_goTypes = nil
	file_xds_data_orca_v3_orca_load_report_proto_depIdxs = nil
}
