// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/core/v3/event_service_config.proto

package corev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#not-implemented-hide:]
// Configuration of the event reporting service endpoint.
type EventServiceConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to ConfigSourceSpecifier:
	//
	//	*EventServiceConfig_GrpcService
	ConfigSourceSpecifier isEventServiceConfig_ConfigSourceSpecifier `protobuf_oneof:"config_source_specifier"`
	unknownFields         protoimpl.UnknownFields
	sizeCache             protoimpl.SizeCache
}

func (x *EventServiceConfig) Reset() {
	*x = EventServiceConfig{}
	mi := &file_envoy_config_core_v3_event_service_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EventServiceConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EventServiceConfig) ProtoMessage() {}

func (x *EventServiceConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_core_v3_event_service_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EventServiceConfig.ProtoReflect.Descriptor instead.
func (*EventServiceConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_core_v3_event_service_config_proto_rawDescGZIP(), []int{0}
}

func (x *EventServiceConfig) GetConfigSourceSpecifier() isEventServiceConfig_ConfigSourceSpecifier {
	if x != nil {
		return x.ConfigSourceSpecifier
	}
	return nil
}

func (x *EventServiceConfig) GetGrpcService() *GrpcService {
	if x != nil {
		if x, ok := x.ConfigSourceSpecifier.(*EventServiceConfig_GrpcService); ok {
			return x.GrpcService
		}
	}
	return nil
}

type isEventServiceConfig_ConfigSourceSpecifier interface {
	isEventServiceConfig_ConfigSourceSpecifier()
}

type EventServiceConfig_GrpcService struct {
	// Specifies the gRPC service that hosts the event reporting service.
	GrpcService *GrpcService `protobuf:"bytes,1,opt,name=grpc_service,json=grpcService,proto3,oneof"`
}

func (*EventServiceConfig_GrpcService) isEventServiceConfig_ConfigSourceSpecifier() {}

var File_envoy_config_core_v3_event_service_config_proto protoreflect.FileDescriptor

const file_envoy_config_core_v3_event_service_config_proto_rawDesc = "" +
	"\n" +
	"/envoy/config/core/v3/event_service_config.proto\x12\x14envoy.config.core.v3\x1a'envoy/config/core/v3/grpc_service.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xa9\x01\n" +
	"\x12EventServiceConfig\x12F\n" +
	"\fgrpc_service\x18\x01 \x01(\v2!.envoy.config.core.v3.GrpcServiceH\x00R\vgrpcService:+\x9aň\x1e&\n" +
	"$envoy.api.v2.core.EventServiceConfigB\x1e\n" +
	"\x17config_source_specifier\x12\x03\xf8B\x01B\x8b\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\"io.envoyproxy.envoy.config.core.v3B\x17EventServiceConfigProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3b\x06proto3"

var (
	file_envoy_config_core_v3_event_service_config_proto_rawDescOnce sync.Once
	file_envoy_config_core_v3_event_service_config_proto_rawDescData []byte
)

func file_envoy_config_core_v3_event_service_config_proto_rawDescGZIP() []byte {
	file_envoy_config_core_v3_event_service_config_proto_rawDescOnce.Do(func() {
		file_envoy_config_core_v3_event_service_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_event_service_config_proto_rawDesc), len(file_envoy_config_core_v3_event_service_config_proto_rawDesc)))
	})
	return file_envoy_config_core_v3_event_service_config_proto_rawDescData
}

var file_envoy_config_core_v3_event_service_config_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_core_v3_event_service_config_proto_goTypes = []any{
	(*EventServiceConfig)(nil), // 0: envoy.config.core.v3.EventServiceConfig
	(*GrpcService)(nil),        // 1: envoy.config.core.v3.GrpcService
}
var file_envoy_config_core_v3_event_service_config_proto_depIdxs = []int32{
	1, // 0: envoy.config.core.v3.EventServiceConfig.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_config_core_v3_event_service_config_proto_init() }
func file_envoy_config_core_v3_event_service_config_proto_init() {
	if File_envoy_config_core_v3_event_service_config_proto != nil {
		return
	}
	file_envoy_config_core_v3_grpc_service_proto_init()
	file_envoy_config_core_v3_event_service_config_proto_msgTypes[0].OneofWrappers = []any{
		(*EventServiceConfig_GrpcService)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_core_v3_event_service_config_proto_rawDesc), len(file_envoy_config_core_v3_event_service_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_core_v3_event_service_config_proto_goTypes,
		DependencyIndexes: file_envoy_config_core_v3_event_service_config_proto_depIdxs,
		MessageInfos:      file_envoy_config_core_v3_event_service_config_proto_msgTypes,
	}.Build()
	File_envoy_config_core_v3_event_service_config_proto = out.File
	file_envoy_config_core_v3_event_service_config_proto_goTypes = nil
	file_envoy_config_core_v3_event_service_config_proto_depIdxs = nil
}
