// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Code generated by "deep-copy -type PCIDriverRebindConfigV1Alpha1 -pointer-receiver -header-file ../../../../../hack/boilerplate.txt -o deep_copy.generated.go ."; DO NOT EDIT.

package hardware

// DeepCopy generates a deep copy of *PCIDriverRebindConfigV1Alpha1.
func (o *PCIDriverRebindConfigV1Alpha1) DeepCopy() *PCIDriverRebindConfigV1Alpha1 {
	var cp PCIDriverRebindConfigV1Alpha1 = *o
	return &cp
}
