// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/annotations/v3/sensitive.proto

package v3

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var file_xds_annotations_v3_sensitive_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*bool)(nil),
		Field:         61008053,
		Name:          "xds.annotations.v3.sensitive",
		Tag:           "varint,61008053,opt,name=sensitive",
		Filename:      "xds/annotations/v3/sensitive.proto",
	},
}

// Extension fields to descriptorpb.FieldOptions.
var (
	// optional bool sensitive = 61008053;
	E_Sensitive = &file_xds_annotations_v3_sensitive_proto_extTypes[0]
)

var File_xds_annotations_v3_sensitive_proto protoreflect.FileDescriptor

const file_xds_annotations_v3_sensitive_proto_rawDesc = "" +
	"\n" +
	"\"xds/annotations/v3/sensitive.proto\x12\x12xds.annotations.v3\x1a google/protobuf/descriptor.proto:>\n" +
	"\tsensitive\x12\x1d.google.protobuf.FieldOptions\x18\xb5ы\x1d \x01(\bR\tsensitiveB+Z)github.com/cncf/xds/go/xds/annotations/v3b\x06proto3"

var file_xds_annotations_v3_sensitive_proto_goTypes = []any{
	(*descriptorpb.FieldOptions)(nil), // 0: google.protobuf.FieldOptions
}
var file_xds_annotations_v3_sensitive_proto_depIdxs = []int32{
	0, // 0: xds.annotations.v3.sensitive:extendee -> google.protobuf.FieldOptions
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	0, // [0:1] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_xds_annotations_v3_sensitive_proto_init() }
func file_xds_annotations_v3_sensitive_proto_init() {
	if File_xds_annotations_v3_sensitive_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_annotations_v3_sensitive_proto_rawDesc), len(file_xds_annotations_v3_sensitive_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 1,
			NumServices:   0,
		},
		GoTypes:           file_xds_annotations_v3_sensitive_proto_goTypes,
		DependencyIndexes: file_xds_annotations_v3_sensitive_proto_depIdxs,
		ExtensionInfos:    file_xds_annotations_v3_sensitive_proto_extTypes,
	}.Build()
	File_xds_annotations_v3_sensitive_proto = out.File
	file_xds_annotations_v3_sensitive_proto_goTypes = nil
	file_xds_annotations_v3_sensitive_proto_depIdxs = nil
}
