// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/admin/v3/tap.proto

package adminv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/tap/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The /tap admin request body that is used to configure an active tap session.
type TapRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The opaque configuration ID used to match the configuration to a loaded extension.
	// A tap extension configures a similar opaque ID that is used to match.
	ConfigId string `protobuf:"bytes,1,opt,name=config_id,json=configId,proto3" json:"config_id,omitempty"`
	// The tap configuration to load.
	TapConfig     *v3.TapConfig `protobuf:"bytes,2,opt,name=tap_config,json=tapConfig,proto3" json:"tap_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TapRequest) Reset() {
	*x = TapRequest{}
	mi := &file_envoy_admin_v3_tap_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TapRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TapRequest) ProtoMessage() {}

func (x *TapRequest) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_tap_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TapRequest.ProtoReflect.Descriptor instead.
func (*TapRequest) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_tap_proto_rawDescGZIP(), []int{0}
}

func (x *TapRequest) GetConfigId() string {
	if x != nil {
		return x.ConfigId
	}
	return ""
}

func (x *TapRequest) GetTapConfig() *v3.TapConfig {
	if x != nil {
		return x.TapConfig
	}
	return nil
}

var File_envoy_admin_v3_tap_proto protoreflect.FileDescriptor

const file_envoy_admin_v3_tap_proto_rawDesc = "" +
	"\n" +
	"\x18envoy/admin/v3/tap.proto\x12\x0eenvoy.admin.v3\x1a envoy/config/tap/v3/common.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xa2\x01\n" +
	"\n" +
	"TapRequest\x12$\n" +
	"\tconfig_id\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\bconfigId\x12G\n" +
	"\n" +
	"tap_config\x18\x02 \x01(\v2\x1e.envoy.config.tap.v3.TapConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\ttapConfig:%\x9aň\x1e \n" +
	"\x1eenvoy.admin.v2alpha.TapRequestBq\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1cio.envoyproxy.envoy.admin.v3B\bTapProtoP\x01Z=github.com/envoyproxy/go-control-plane/envoy/admin/v3;adminv3b\x06proto3"

var (
	file_envoy_admin_v3_tap_proto_rawDescOnce sync.Once
	file_envoy_admin_v3_tap_proto_rawDescData []byte
)

func file_envoy_admin_v3_tap_proto_rawDescGZIP() []byte {
	file_envoy_admin_v3_tap_proto_rawDescOnce.Do(func() {
		file_envoy_admin_v3_tap_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_tap_proto_rawDesc), len(file_envoy_admin_v3_tap_proto_rawDesc)))
	})
	return file_envoy_admin_v3_tap_proto_rawDescData
}

var file_envoy_admin_v3_tap_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_admin_v3_tap_proto_goTypes = []any{
	(*TapRequest)(nil),   // 0: envoy.admin.v3.TapRequest
	(*v3.TapConfig)(nil), // 1: envoy.config.tap.v3.TapConfig
}
var file_envoy_admin_v3_tap_proto_depIdxs = []int32{
	1, // 0: envoy.admin.v3.TapRequest.tap_config:type_name -> envoy.config.tap.v3.TapConfig
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_admin_v3_tap_proto_init() }
func file_envoy_admin_v3_tap_proto_init() {
	if File_envoy_admin_v3_tap_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_tap_proto_rawDesc), len(file_envoy_admin_v3_tap_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_admin_v3_tap_proto_goTypes,
		DependencyIndexes: file_envoy_admin_v3_tap_proto_depIdxs,
		MessageInfos:      file_envoy_admin_v3_tap_proto_msgTypes,
	}.Build()
	File_envoy_admin_v3_tap_proto = out.File
	file_envoy_admin_v3_tap_proto_goTypes = nil
	file_envoy_admin_v3_tap_proto_depIdxs = nil
}
