// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/monitoring/v3/query_service.proto

package monitoringpb

import (
	reflect "reflect"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var File_google_monitoring_v3_query_service_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_query_service_proto_rawDesc = "" +
	"\n" +
	"(google/monitoring/v3/query_service.proto\x12\x14google.monitoring.v3\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a)google/monitoring/v3/metric_service.proto2\xe1\x02\n" +
	"\fQueryService\x12\xa4\x01\n" +
	"\x0fQueryTimeSeries\x12,.google.monitoring.v3.QueryTimeSeriesRequest\x1a-.google.monitoring.v3.QueryTimeSeriesResponse\"4\x82\xd3\xe4\x93\x02+:\x01*\"&/v3/{name=projects/*}/timeSeries:query\x88\x02\x01\x1a\xa9\x01\xcaA\x19monitoring.googleapis.com\xd2A\x89\x01https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/monitoring,https://www.googleapis.com/auth/monitoring.readB\xcc\x01\n" +
	"\x18com.google.monitoring.v3B\x11QueryServiceProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var file_google_monitoring_v3_query_service_proto_goTypes = []any{
	(*QueryTimeSeriesRequest)(nil),  // 0: google.monitoring.v3.QueryTimeSeriesRequest
	(*QueryTimeSeriesResponse)(nil), // 1: google.monitoring.v3.QueryTimeSeriesResponse
}
var file_google_monitoring_v3_query_service_proto_depIdxs = []int32{
	0, // 0: google.monitoring.v3.QueryService.QueryTimeSeries:input_type -> google.monitoring.v3.QueryTimeSeriesRequest
	1, // 1: google.monitoring.v3.QueryService.QueryTimeSeries:output_type -> google.monitoring.v3.QueryTimeSeriesResponse
	1, // [1:2] is the sub-list for method output_type
	0, // [0:1] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_query_service_proto_init() }
func file_google_monitoring_v3_query_service_proto_init() {
	if File_google_monitoring_v3_query_service_proto != nil {
		return
	}
	file_google_monitoring_v3_metric_service_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_query_service_proto_rawDesc), len(file_google_monitoring_v3_query_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_monitoring_v3_query_service_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_query_service_proto_depIdxs,
	}.Build()
	File_google_monitoring_v3_query_service_proto = out.File
	file_google_monitoring_v3_query_service_proto_goTypes = nil
	file_google_monitoring_v3_query_service_proto_depIdxs = nil
}
