// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Code generated by "deep-copy -type AcquireConfigSpecSpec -type AcquireConfigStatusSpec -type ServiceSpec -header-file ../../../../hack/boilerplate.txt -o deep_copy.generated.go ."; DO NOT EDIT.

package v1alpha1

// DeepCopy generates a deep copy of AcquireConfigSpecSpec.
func (o AcquireConfigSpecSpec) DeepCopy() AcquireConfigSpecSpec {
	var cp AcquireConfigSpecSpec = o
	return cp
}

// DeepCopy generates a deep copy of AcquireConfigStatusSpec.
func (o AcquireConfigStatusSpec) DeepCopy() AcquireConfigStatusSpec {
	var cp AcquireConfigStatusSpec = o
	return cp
}

// DeepCopy generates a deep copy of ServiceSpec.
func (o ServiceSpec) DeepCopy() ServiceSpec {
	var cp ServiceSpec = o
	return cp
}
