// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/matcher/v3/filter_state.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// FilterStateMatcher provides a general interface for matching the filter state objects.
type FilterStateMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The filter state key to retrieve the object.
	Key string `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	// Types that are valid to be assigned to Matcher:
	//
	//	*FilterStateMatcher_StringMatch
	//	*FilterStateMatcher_AddressMatch
	Matcher       isFilterStateMatcher_Matcher `protobuf_oneof:"matcher"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FilterStateMatcher) Reset() {
	*x = FilterStateMatcher{}
	mi := &file_envoy_type_matcher_v3_filter_state_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FilterStateMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FilterStateMatcher) ProtoMessage() {}

func (x *FilterStateMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_filter_state_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FilterStateMatcher.ProtoReflect.Descriptor instead.
func (*FilterStateMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_filter_state_proto_rawDescGZIP(), []int{0}
}

func (x *FilterStateMatcher) GetKey() string {
	if x != nil {
		return x.Key
	}
	return ""
}

func (x *FilterStateMatcher) GetMatcher() isFilterStateMatcher_Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

func (x *FilterStateMatcher) GetStringMatch() *StringMatcher {
	if x != nil {
		if x, ok := x.Matcher.(*FilterStateMatcher_StringMatch); ok {
			return x.StringMatch
		}
	}
	return nil
}

func (x *FilterStateMatcher) GetAddressMatch() *AddressMatcher {
	if x != nil {
		if x, ok := x.Matcher.(*FilterStateMatcher_AddressMatch); ok {
			return x.AddressMatch
		}
	}
	return nil
}

type isFilterStateMatcher_Matcher interface {
	isFilterStateMatcher_Matcher()
}

type FilterStateMatcher_StringMatch struct {
	// Matches the filter state object as a string value.
	StringMatch *StringMatcher `protobuf:"bytes,2,opt,name=string_match,json=stringMatch,proto3,oneof"`
}

type FilterStateMatcher_AddressMatch struct {
	// Matches the filter state object as a ip Instance.
	AddressMatch *AddressMatcher `protobuf:"bytes,3,opt,name=address_match,json=addressMatch,proto3,oneof"`
}

func (*FilterStateMatcher_StringMatch) isFilterStateMatcher_Matcher() {}

func (*FilterStateMatcher_AddressMatch) isFilterStateMatcher_Matcher() {}

var File_envoy_type_matcher_v3_filter_state_proto protoreflect.FileDescriptor

const file_envoy_type_matcher_v3_filter_state_proto_rawDesc = "" +
	"\n" +
	"(envoy/type/matcher/v3/filter_state.proto\x12\x15envoy.type.matcher.v3\x1a#envoy/type/matcher/v3/address.proto\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd8\x01\n" +
	"\x12FilterStateMatcher\x12\x19\n" +
	"\x03key\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x03key\x12I\n" +
	"\fstring_match\x18\x02 \x01(\v2$.envoy.type.matcher.v3.StringMatcherH\x00R\vstringMatch\x12L\n" +
	"\raddress_match\x18\x03 \x01(\v2%.envoy.type.matcher.v3.AddressMatcherH\x00R\faddressMatchB\x0e\n" +
	"\amatcher\x12\x03\xf8B\x01B\x89\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.type.matcher.v3B\x10FilterStateProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_type_matcher_v3_filter_state_proto_rawDescOnce sync.Once
	file_envoy_type_matcher_v3_filter_state_proto_rawDescData []byte
)

func file_envoy_type_matcher_v3_filter_state_proto_rawDescGZIP() []byte {
	file_envoy_type_matcher_v3_filter_state_proto_rawDescOnce.Do(func() {
		file_envoy_type_matcher_v3_filter_state_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_filter_state_proto_rawDesc), len(file_envoy_type_matcher_v3_filter_state_proto_rawDesc)))
	})
	return file_envoy_type_matcher_v3_filter_state_proto_rawDescData
}

var file_envoy_type_matcher_v3_filter_state_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_type_matcher_v3_filter_state_proto_goTypes = []any{
	(*FilterStateMatcher)(nil), // 0: envoy.type.matcher.v3.FilterStateMatcher
	(*StringMatcher)(nil),      // 1: envoy.type.matcher.v3.StringMatcher
	(*AddressMatcher)(nil),     // 2: envoy.type.matcher.v3.AddressMatcher
}
var file_envoy_type_matcher_v3_filter_state_proto_depIdxs = []int32{
	1, // 0: envoy.type.matcher.v3.FilterStateMatcher.string_match:type_name -> envoy.type.matcher.v3.StringMatcher
	2, // 1: envoy.type.matcher.v3.FilterStateMatcher.address_match:type_name -> envoy.type.matcher.v3.AddressMatcher
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_type_matcher_v3_filter_state_proto_init() }
func file_envoy_type_matcher_v3_filter_state_proto_init() {
	if File_envoy_type_matcher_v3_filter_state_proto != nil {
		return
	}
	file_envoy_type_matcher_v3_address_proto_init()
	file_envoy_type_matcher_v3_string_proto_init()
	file_envoy_type_matcher_v3_filter_state_proto_msgTypes[0].OneofWrappers = []any{
		(*FilterStateMatcher_StringMatch)(nil),
		(*FilterStateMatcher_AddressMatch)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_filter_state_proto_rawDesc), len(file_envoy_type_matcher_v3_filter_state_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_matcher_v3_filter_state_proto_goTypes,
		DependencyIndexes: file_envoy_type_matcher_v3_filter_state_proto_depIdxs,
		MessageInfos:      file_envoy_type_matcher_v3_filter_state_proto_msgTypes,
	}.Build()
	File_envoy_type_matcher_v3_filter_state_proto = out.File
	file_envoy_type_matcher_v3_filter_state_proto_goTypes = nil
	file_envoy_type_matcher_v3_filter_state_proto_depIdxs = nil
}
