// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/matcher/v3/metadata.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#next-major-version: MetadataMatcher should use StructMatcher]
type MetadataMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The filter name to retrieve the “Struct“ from the “Metadata“.
	Filter string `protobuf:"bytes,1,opt,name=filter,proto3" json:"filter,omitempty"`
	// The path to retrieve the “Value“ from the “Struct“.
	Path []*MetadataMatcher_PathSegment `protobuf:"bytes,2,rep,name=path,proto3" json:"path,omitempty"`
	// The “MetadataMatcher“ is matched if the value retrieved by path is matched to this value.
	Value *ValueMatcher `protobuf:"bytes,3,opt,name=value,proto3" json:"value,omitempty"`
	// If true, the match result will be inverted.
	Invert        bool `protobuf:"varint,4,opt,name=invert,proto3" json:"invert,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetadataMatcher) Reset() {
	*x = MetadataMatcher{}
	mi := &file_envoy_type_matcher_v3_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetadataMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetadataMatcher) ProtoMessage() {}

func (x *MetadataMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetadataMatcher.ProtoReflect.Descriptor instead.
func (*MetadataMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *MetadataMatcher) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *MetadataMatcher) GetPath() []*MetadataMatcher_PathSegment {
	if x != nil {
		return x.Path
	}
	return nil
}

func (x *MetadataMatcher) GetValue() *ValueMatcher {
	if x != nil {
		return x.Value
	}
	return nil
}

func (x *MetadataMatcher) GetInvert() bool {
	if x != nil {
		return x.Invert
	}
	return false
}

// Specifies the segment in a path to retrieve value from “Metadata“.
//
// .. note::
//
//	Currently it's not supported to retrieve a value from a list in ``Metadata``. This means that
//	if the segment key refers to a list, it has to be the last segment in a path.
type MetadataMatcher_PathSegment struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Segment:
	//
	//	*MetadataMatcher_PathSegment_Key
	Segment       isMetadataMatcher_PathSegment_Segment `protobuf_oneof:"segment"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetadataMatcher_PathSegment) Reset() {
	*x = MetadataMatcher_PathSegment{}
	mi := &file_envoy_type_matcher_v3_metadata_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetadataMatcher_PathSegment) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetadataMatcher_PathSegment) ProtoMessage() {}

func (x *MetadataMatcher_PathSegment) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_metadata_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetadataMatcher_PathSegment.ProtoReflect.Descriptor instead.
func (*MetadataMatcher_PathSegment) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_metadata_proto_rawDescGZIP(), []int{0, 0}
}

func (x *MetadataMatcher_PathSegment) GetSegment() isMetadataMatcher_PathSegment_Segment {
	if x != nil {
		return x.Segment
	}
	return nil
}

func (x *MetadataMatcher_PathSegment) GetKey() string {
	if x != nil {
		if x, ok := x.Segment.(*MetadataMatcher_PathSegment_Key); ok {
			return x.Key
		}
	}
	return ""
}

type isMetadataMatcher_PathSegment_Segment interface {
	isMetadataMatcher_PathSegment_Segment()
}

type MetadataMatcher_PathSegment_Key struct {
	// If specified, use the key to retrieve the value in a “Struct“.
	Key string `protobuf:"bytes,1,opt,name=key,proto3,oneof"`
}

func (*MetadataMatcher_PathSegment_Key) isMetadataMatcher_PathSegment_Segment() {}

var File_envoy_type_matcher_v3_metadata_proto protoreflect.FileDescriptor

const file_envoy_type_matcher_v3_metadata_proto_rawDesc = "" +
	"\n" +
	"$envoy/type/matcher/v3/metadata.proto\x12\x15envoy.type.matcher.v3\x1a!envoy/type/matcher/v3/value.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xff\x02\n" +
	"\x0fMetadataMatcher\x12\x1f\n" +
	"\x06filter\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x06filter\x12P\n" +
	"\x04path\x18\x02 \x03(\v22.envoy.type.matcher.v3.MetadataMatcher.PathSegmentB\b\xfaB\x05\x92\x01\x02\b\x01R\x04path\x12C\n" +
	"\x05value\x18\x03 \x01(\v2#.envoy.type.matcher.v3.ValueMatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05value\x12\x16\n" +
	"\x06invert\x18\x04 \x01(\bR\x06invert\x1aq\n" +
	"\vPathSegment\x12\x1b\n" +
	"\x03key\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\x03key:5\x9aň\x1e0\n" +
	".envoy.type.matcher.MetadataMatcher.PathSegmentB\x0e\n" +
	"\asegment\x12\x03\xf8B\x01:)\x9aň\x1e$\n" +
	"\"envoy.type.matcher.MetadataMatcherB\x86\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.type.matcher.v3B\rMetadataProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_type_matcher_v3_metadata_proto_rawDescOnce sync.Once
	file_envoy_type_matcher_v3_metadata_proto_rawDescData []byte
)

func file_envoy_type_matcher_v3_metadata_proto_rawDescGZIP() []byte {
	file_envoy_type_matcher_v3_metadata_proto_rawDescOnce.Do(func() {
		file_envoy_type_matcher_v3_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_metadata_proto_rawDesc), len(file_envoy_type_matcher_v3_metadata_proto_rawDesc)))
	})
	return file_envoy_type_matcher_v3_metadata_proto_rawDescData
}

var file_envoy_type_matcher_v3_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_type_matcher_v3_metadata_proto_goTypes = []any{
	(*MetadataMatcher)(nil),             // 0: envoy.type.matcher.v3.MetadataMatcher
	(*MetadataMatcher_PathSegment)(nil), // 1: envoy.type.matcher.v3.MetadataMatcher.PathSegment
	(*ValueMatcher)(nil),                // 2: envoy.type.matcher.v3.ValueMatcher
}
var file_envoy_type_matcher_v3_metadata_proto_depIdxs = []int32{
	1, // 0: envoy.type.matcher.v3.MetadataMatcher.path:type_name -> envoy.type.matcher.v3.MetadataMatcher.PathSegment
	2, // 1: envoy.type.matcher.v3.MetadataMatcher.value:type_name -> envoy.type.matcher.v3.ValueMatcher
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_type_matcher_v3_metadata_proto_init() }
func file_envoy_type_matcher_v3_metadata_proto_init() {
	if File_envoy_type_matcher_v3_metadata_proto != nil {
		return
	}
	file_envoy_type_matcher_v3_value_proto_init()
	file_envoy_type_matcher_v3_metadata_proto_msgTypes[1].OneofWrappers = []any{
		(*MetadataMatcher_PathSegment_Key)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_metadata_proto_rawDesc), len(file_envoy_type_matcher_v3_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_matcher_v3_metadata_proto_goTypes,
		DependencyIndexes: file_envoy_type_matcher_v3_metadata_proto_depIdxs,
		MessageInfos:      file_envoy_type_matcher_v3_metadata_proto_msgTypes,
	}.Build()
	File_envoy_type_matcher_v3_metadata_proto = out.File
	file_envoy_type_matcher_v3_metadata_proto_goTypes = nil
	file_envoy_type_matcher_v3_metadata_proto_depIdxs = nil
}
