// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/monitoring/v3/uptime_service.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The protocol for the `ListUptimeCheckConfigs` request.
type ListUptimeCheckConfigsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The
	// [project](https://cloud.google.com/monitoring/api/v3#project_name) whose
	// Uptime check configurations are listed. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// If provided, this field specifies the criteria that must be met by
	// uptime checks to be included in the response.
	//
	// For more details, see [Filtering
	// syntax](https://cloud.google.com/monitoring/api/v3/sorting-and-filtering#filter_syntax).
	Filter string `protobuf:"bytes,2,opt,name=filter,proto3" json:"filter,omitempty"`
	// The maximum number of results to return in a single response. The server
	// may further constrain the maximum number of results returned in a single
	// page. If the page_size is <=0, the server will decide the number of results
	// to be returned.
	PageSize int32 `protobuf:"varint,3,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// If this field is not empty then it must contain the `nextPageToken` value
	// returned by a previous call to this method.  Using this field causes the
	// method to return more results from the previous method call.
	PageToken     string `protobuf:"bytes,4,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListUptimeCheckConfigsRequest) Reset() {
	*x = ListUptimeCheckConfigsRequest{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListUptimeCheckConfigsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListUptimeCheckConfigsRequest) ProtoMessage() {}

func (x *ListUptimeCheckConfigsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListUptimeCheckConfigsRequest.ProtoReflect.Descriptor instead.
func (*ListUptimeCheckConfigsRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{0}
}

func (x *ListUptimeCheckConfigsRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *ListUptimeCheckConfigsRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *ListUptimeCheckConfigsRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListUptimeCheckConfigsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

// The protocol for the `ListUptimeCheckConfigs` response.
type ListUptimeCheckConfigsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The returned Uptime check configurations.
	UptimeCheckConfigs []*UptimeCheckConfig `protobuf:"bytes,1,rep,name=uptime_check_configs,json=uptimeCheckConfigs,proto3" json:"uptime_check_configs,omitempty"`
	// This field represents the pagination token to retrieve the next page of
	// results. If the value is empty, it means no further results for the
	// request. To retrieve the next page of results, the value of the
	// next_page_token is passed to the subsequent List method call (in the
	// request message's page_token field).
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	// The total number of Uptime check configurations for the project,
	// irrespective of any pagination.
	TotalSize     int32 `protobuf:"varint,3,opt,name=total_size,json=totalSize,proto3" json:"total_size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListUptimeCheckConfigsResponse) Reset() {
	*x = ListUptimeCheckConfigsResponse{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListUptimeCheckConfigsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListUptimeCheckConfigsResponse) ProtoMessage() {}

func (x *ListUptimeCheckConfigsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListUptimeCheckConfigsResponse.ProtoReflect.Descriptor instead.
func (*ListUptimeCheckConfigsResponse) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListUptimeCheckConfigsResponse) GetUptimeCheckConfigs() []*UptimeCheckConfig {
	if x != nil {
		return x.UptimeCheckConfigs
	}
	return nil
}

func (x *ListUptimeCheckConfigsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

func (x *ListUptimeCheckConfigsResponse) GetTotalSize() int32 {
	if x != nil {
		return x.TotalSize
	}
	return 0
}

// The protocol for the `GetUptimeCheckConfig` request.
type GetUptimeCheckConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The Uptime check configuration to retrieve. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/uptimeCheckConfigs/[UPTIME_CHECK_ID]
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetUptimeCheckConfigRequest) Reset() {
	*x = GetUptimeCheckConfigRequest{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetUptimeCheckConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetUptimeCheckConfigRequest) ProtoMessage() {}

func (x *GetUptimeCheckConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetUptimeCheckConfigRequest.ProtoReflect.Descriptor instead.
func (*GetUptimeCheckConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{2}
}

func (x *GetUptimeCheckConfigRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// The protocol for the `CreateUptimeCheckConfig` request.
type CreateUptimeCheckConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The
	// [project](https://cloud.google.com/monitoring/api/v3#project_name) in which
	// to create the Uptime check. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Required. The new Uptime check configuration.
	UptimeCheckConfig *UptimeCheckConfig `protobuf:"bytes,2,opt,name=uptime_check_config,json=uptimeCheckConfig,proto3" json:"uptime_check_config,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *CreateUptimeCheckConfigRequest) Reset() {
	*x = CreateUptimeCheckConfigRequest{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateUptimeCheckConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateUptimeCheckConfigRequest) ProtoMessage() {}

func (x *CreateUptimeCheckConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateUptimeCheckConfigRequest.ProtoReflect.Descriptor instead.
func (*CreateUptimeCheckConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{3}
}

func (x *CreateUptimeCheckConfigRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateUptimeCheckConfigRequest) GetUptimeCheckConfig() *UptimeCheckConfig {
	if x != nil {
		return x.UptimeCheckConfig
	}
	return nil
}

// The protocol for the `UpdateUptimeCheckConfig` request.
type UpdateUptimeCheckConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. If present, only the listed fields in the current Uptime check
	// configuration are updated with values from the new configuration. If this
	// field is empty, then the current configuration is completely replaced with
	// the new configuration.
	UpdateMask *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	// Required. If an `updateMask` has been specified, this field gives
	// the values for the set of fields mentioned in the `updateMask`. If an
	// `updateMask` has not been given, this Uptime check configuration replaces
	// the current configuration. If a field is mentioned in `updateMask` but
	// the corresponding field is omitted in this partial Uptime check
	// configuration, it has the effect of deleting/clearing the field from the
	// configuration on the server.
	//
	// The following fields can be updated: `display_name`,
	// `http_check`, `tcp_check`, `timeout`, `content_matchers`, and
	// `selected_regions`.
	UptimeCheckConfig *UptimeCheckConfig `protobuf:"bytes,3,opt,name=uptime_check_config,json=uptimeCheckConfig,proto3" json:"uptime_check_config,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *UpdateUptimeCheckConfigRequest) Reset() {
	*x = UpdateUptimeCheckConfigRequest{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateUptimeCheckConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateUptimeCheckConfigRequest) ProtoMessage() {}

func (x *UpdateUptimeCheckConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateUptimeCheckConfigRequest.ProtoReflect.Descriptor instead.
func (*UpdateUptimeCheckConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{4}
}

func (x *UpdateUptimeCheckConfigRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

func (x *UpdateUptimeCheckConfigRequest) GetUptimeCheckConfig() *UptimeCheckConfig {
	if x != nil {
		return x.UptimeCheckConfig
	}
	return nil
}

// The protocol for the `DeleteUptimeCheckConfig` request.
type DeleteUptimeCheckConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The Uptime check configuration to delete. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/uptimeCheckConfigs/[UPTIME_CHECK_ID]
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteUptimeCheckConfigRequest) Reset() {
	*x = DeleteUptimeCheckConfigRequest{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteUptimeCheckConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteUptimeCheckConfigRequest) ProtoMessage() {}

func (x *DeleteUptimeCheckConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteUptimeCheckConfigRequest.ProtoReflect.Descriptor instead.
func (*DeleteUptimeCheckConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{5}
}

func (x *DeleteUptimeCheckConfigRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// The protocol for the `ListUptimeCheckIps` request.
type ListUptimeCheckIpsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The maximum number of results to return in a single response. The server
	// may further constrain the maximum number of results returned in a single
	// page. If the page_size is <=0, the server will decide the number of results
	// to be returned.
	// NOTE: this field is not yet implemented
	PageSize int32 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// If this field is not empty then it must contain the `nextPageToken` value
	// returned by a previous call to this method.  Using this field causes the
	// method to return more results from the previous method call.
	// NOTE: this field is not yet implemented
	PageToken     string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListUptimeCheckIpsRequest) Reset() {
	*x = ListUptimeCheckIpsRequest{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListUptimeCheckIpsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListUptimeCheckIpsRequest) ProtoMessage() {}

func (x *ListUptimeCheckIpsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListUptimeCheckIpsRequest.ProtoReflect.Descriptor instead.
func (*ListUptimeCheckIpsRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{6}
}

func (x *ListUptimeCheckIpsRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListUptimeCheckIpsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

// The protocol for the `ListUptimeCheckIps` response.
type ListUptimeCheckIpsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The returned list of IP addresses (including region and location) that the
	// checkers run from.
	UptimeCheckIps []*UptimeCheckIp `protobuf:"bytes,1,rep,name=uptime_check_ips,json=uptimeCheckIps,proto3" json:"uptime_check_ips,omitempty"`
	// This field represents the pagination token to retrieve the next page of
	// results. If the value is empty, it means no further results for the
	// request. To retrieve the next page of results, the value of the
	// next_page_token is passed to the subsequent List method call (in the
	// request message's page_token field).
	// NOTE: this field is not yet implemented
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListUptimeCheckIpsResponse) Reset() {
	*x = ListUptimeCheckIpsResponse{}
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListUptimeCheckIpsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListUptimeCheckIpsResponse) ProtoMessage() {}

func (x *ListUptimeCheckIpsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_uptime_service_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListUptimeCheckIpsResponse.ProtoReflect.Descriptor instead.
func (*ListUptimeCheckIpsResponse) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_uptime_service_proto_rawDescGZIP(), []int{7}
}

func (x *ListUptimeCheckIpsResponse) GetUptimeCheckIps() []*UptimeCheckIp {
	if x != nil {
		return x.UptimeCheckIps
	}
	return nil
}

func (x *ListUptimeCheckIpsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

var File_google_monitoring_v3_uptime_service_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_uptime_service_proto_rawDesc = "" +
	"\n" +
	")google/monitoring/v3/uptime_service.proto\x12\x14google.monitoring.v3\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a!google/monitoring/v3/uptime.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a google/protobuf/field_mask.proto\"\xc0\x01\n" +
	"\x1dListUptimeCheckConfigsRequest\x12K\n" +
	"\x06parent\x18\x01 \x01(\tB3\xe0A\x02\xfaA-\x12+monitoring.googleapis.com/UptimeCheckConfigR\x06parent\x12\x16\n" +
	"\x06filter\x18\x02 \x01(\tR\x06filter\x12\x1b\n" +
	"\tpage_size\x18\x03 \x01(\x05R\bpageSize\x12\x1d\n" +
	"\n" +
	"page_token\x18\x04 \x01(\tR\tpageToken\"\xc2\x01\n" +
	"\x1eListUptimeCheckConfigsResponse\x12Y\n" +
	"\x14uptime_check_configs\x18\x01 \x03(\v2'.google.monitoring.v3.UptimeCheckConfigR\x12uptimeCheckConfigs\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\x12\x1d\n" +
	"\n" +
	"total_size\x18\x03 \x01(\x05R\ttotalSize\"f\n" +
	"\x1bGetUptimeCheckConfigRequest\x12G\n" +
	"\x04name\x18\x01 \x01(\tB3\xe0A\x02\xfaA-\n" +
	"+monitoring.googleapis.com/UptimeCheckConfigR\x04name\"\xcb\x01\n" +
	"\x1eCreateUptimeCheckConfigRequest\x12K\n" +
	"\x06parent\x18\x01 \x01(\tB3\xe0A\x02\xfaA-\x12+monitoring.googleapis.com/UptimeCheckConfigR\x06parent\x12\\\n" +
	"\x13uptime_check_config\x18\x02 \x01(\v2'.google.monitoring.v3.UptimeCheckConfigB\x03\xe0A\x02R\x11uptimeCheckConfig\"\xbb\x01\n" +
	"\x1eUpdateUptimeCheckConfigRequest\x12;\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskR\n" +
	"updateMask\x12\\\n" +
	"\x13uptime_check_config\x18\x03 \x01(\v2'.google.monitoring.v3.UptimeCheckConfigB\x03\xe0A\x02R\x11uptimeCheckConfig\"i\n" +
	"\x1eDeleteUptimeCheckConfigRequest\x12G\n" +
	"\x04name\x18\x01 \x01(\tB3\xe0A\x02\xfaA-\n" +
	"+monitoring.googleapis.com/UptimeCheckConfigR\x04name\"W\n" +
	"\x19ListUptimeCheckIpsRequest\x12\x1b\n" +
	"\tpage_size\x18\x02 \x01(\x05R\bpageSize\x12\x1d\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tR\tpageToken\"\x93\x01\n" +
	"\x1aListUptimeCheckIpsResponse\x12M\n" +
	"\x10uptime_check_ips\x18\x01 \x03(\v2#.google.monitoring.v3.UptimeCheckIpR\x0euptimeCheckIps\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xbd\n" +
	"\n" +
	"\x12UptimeCheckService\x12\xc0\x01\n" +
	"\x16ListUptimeCheckConfigs\x123.google.monitoring.v3.ListUptimeCheckConfigsRequest\x1a4.google.monitoring.v3.ListUptimeCheckConfigsResponse\";\xdaA\x06parent\x82\xd3\xe4\x93\x02,\x12*/v3/{parent=projects/*}/uptimeCheckConfigs\x12\xad\x01\n" +
	"\x14GetUptimeCheckConfig\x121.google.monitoring.v3.GetUptimeCheckConfigRequest\x1a'.google.monitoring.v3.UptimeCheckConfig\"9\xdaA\x04name\x82\xd3\xe4\x93\x02,\x12*/v3/{name=projects/*/uptimeCheckConfigs/*}\x12\xde\x01\n" +
	"\x17CreateUptimeCheckConfig\x124.google.monitoring.v3.CreateUptimeCheckConfigRequest\x1a'.google.monitoring.v3.UptimeCheckConfig\"d\xdaA\x1aparent,uptime_check_config\x82\xd3\xe4\x93\x02A:\x13uptime_check_config\"*/v3/{parent=projects/*}/uptimeCheckConfigs\x12\xeb\x01\n" +
	"\x17UpdateUptimeCheckConfig\x124.google.monitoring.v3.UpdateUptimeCheckConfigRequest\x1a'.google.monitoring.v3.UptimeCheckConfig\"q\xdaA\x13uptime_check_config\x82\xd3\xe4\x93\x02U:\x13uptime_check_config2>/v3/{uptime_check_config.name=projects/*/uptimeCheckConfigs/*}\x12\xa2\x01\n" +
	"\x17DeleteUptimeCheckConfig\x124.google.monitoring.v3.DeleteUptimeCheckConfigRequest\x1a\x16.google.protobuf.Empty\"9\xdaA\x04name\x82\xd3\xe4\x93\x02,**/v3/{name=projects/*/uptimeCheckConfigs/*}\x12\x93\x01\n" +
	"\x12ListUptimeCheckIps\x12/.google.monitoring.v3.ListUptimeCheckIpsRequest\x1a0.google.monitoring.v3.ListUptimeCheckIpsResponse\"\x1a\x82\xd3\xe4\x93\x02\x14\x12\x12/v3/uptimeCheckIps\x1a\xa9\x01\xcaA\x19monitoring.googleapis.com\xd2A\x89\x01https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/monitoring,https://www.googleapis.com/auth/monitoring.readB\xcd\x01\n" +
	"\x18com.google.monitoring.v3B\x12UptimeServiceProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_uptime_service_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_uptime_service_proto_rawDescData []byte
)

func file_google_monitoring_v3_uptime_service_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_uptime_service_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_uptime_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_uptime_service_proto_rawDesc), len(file_google_monitoring_v3_uptime_service_proto_rawDesc)))
	})
	return file_google_monitoring_v3_uptime_service_proto_rawDescData
}

var file_google_monitoring_v3_uptime_service_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_google_monitoring_v3_uptime_service_proto_goTypes = []any{
	(*ListUptimeCheckConfigsRequest)(nil),  // 0: google.monitoring.v3.ListUptimeCheckConfigsRequest
	(*ListUptimeCheckConfigsResponse)(nil), // 1: google.monitoring.v3.ListUptimeCheckConfigsResponse
	(*GetUptimeCheckConfigRequest)(nil),    // 2: google.monitoring.v3.GetUptimeCheckConfigRequest
	(*CreateUptimeCheckConfigRequest)(nil), // 3: google.monitoring.v3.CreateUptimeCheckConfigRequest
	(*UpdateUptimeCheckConfigRequest)(nil), // 4: google.monitoring.v3.UpdateUptimeCheckConfigRequest
	(*DeleteUptimeCheckConfigRequest)(nil), // 5: google.monitoring.v3.DeleteUptimeCheckConfigRequest
	(*ListUptimeCheckIpsRequest)(nil),      // 6: google.monitoring.v3.ListUptimeCheckIpsRequest
	(*ListUptimeCheckIpsResponse)(nil),     // 7: google.monitoring.v3.ListUptimeCheckIpsResponse
	(*UptimeCheckConfig)(nil),              // 8: google.monitoring.v3.UptimeCheckConfig
	(*fieldmaskpb.FieldMask)(nil),          // 9: google.protobuf.FieldMask
	(*UptimeCheckIp)(nil),                  // 10: google.monitoring.v3.UptimeCheckIp
	(*emptypb.Empty)(nil),                  // 11: google.protobuf.Empty
}
var file_google_monitoring_v3_uptime_service_proto_depIdxs = []int32{
	8,  // 0: google.monitoring.v3.ListUptimeCheckConfigsResponse.uptime_check_configs:type_name -> google.monitoring.v3.UptimeCheckConfig
	8,  // 1: google.monitoring.v3.CreateUptimeCheckConfigRequest.uptime_check_config:type_name -> google.monitoring.v3.UptimeCheckConfig
	9,  // 2: google.monitoring.v3.UpdateUptimeCheckConfigRequest.update_mask:type_name -> google.protobuf.FieldMask
	8,  // 3: google.monitoring.v3.UpdateUptimeCheckConfigRequest.uptime_check_config:type_name -> google.monitoring.v3.UptimeCheckConfig
	10, // 4: google.monitoring.v3.ListUptimeCheckIpsResponse.uptime_check_ips:type_name -> google.monitoring.v3.UptimeCheckIp
	0,  // 5: google.monitoring.v3.UptimeCheckService.ListUptimeCheckConfigs:input_type -> google.monitoring.v3.ListUptimeCheckConfigsRequest
	2,  // 6: google.monitoring.v3.UptimeCheckService.GetUptimeCheckConfig:input_type -> google.monitoring.v3.GetUptimeCheckConfigRequest
	3,  // 7: google.monitoring.v3.UptimeCheckService.CreateUptimeCheckConfig:input_type -> google.monitoring.v3.CreateUptimeCheckConfigRequest
	4,  // 8: google.monitoring.v3.UptimeCheckService.UpdateUptimeCheckConfig:input_type -> google.monitoring.v3.UpdateUptimeCheckConfigRequest
	5,  // 9: google.monitoring.v3.UptimeCheckService.DeleteUptimeCheckConfig:input_type -> google.monitoring.v3.DeleteUptimeCheckConfigRequest
	6,  // 10: google.monitoring.v3.UptimeCheckService.ListUptimeCheckIps:input_type -> google.monitoring.v3.ListUptimeCheckIpsRequest
	1,  // 11: google.monitoring.v3.UptimeCheckService.ListUptimeCheckConfigs:output_type -> google.monitoring.v3.ListUptimeCheckConfigsResponse
	8,  // 12: google.monitoring.v3.UptimeCheckService.GetUptimeCheckConfig:output_type -> google.monitoring.v3.UptimeCheckConfig
	8,  // 13: google.monitoring.v3.UptimeCheckService.CreateUptimeCheckConfig:output_type -> google.monitoring.v3.UptimeCheckConfig
	8,  // 14: google.monitoring.v3.UptimeCheckService.UpdateUptimeCheckConfig:output_type -> google.monitoring.v3.UptimeCheckConfig
	11, // 15: google.monitoring.v3.UptimeCheckService.DeleteUptimeCheckConfig:output_type -> google.protobuf.Empty
	7,  // 16: google.monitoring.v3.UptimeCheckService.ListUptimeCheckIps:output_type -> google.monitoring.v3.ListUptimeCheckIpsResponse
	11, // [11:17] is the sub-list for method output_type
	5,  // [5:11] is the sub-list for method input_type
	5,  // [5:5] is the sub-list for extension type_name
	5,  // [5:5] is the sub-list for extension extendee
	0,  // [0:5] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_uptime_service_proto_init() }
func file_google_monitoring_v3_uptime_service_proto_init() {
	if File_google_monitoring_v3_uptime_service_proto != nil {
		return
	}
	file_google_monitoring_v3_uptime_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_uptime_service_proto_rawDesc), len(file_google_monitoring_v3_uptime_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_monitoring_v3_uptime_service_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_uptime_service_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_uptime_service_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_uptime_service_proto = out.File
	file_google_monitoring_v3_uptime_service_proto_goTypes = nil
	file_google_monitoring_v3_uptime_service_proto_depIdxs = nil
}
