// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/type/matcher/v3/range.proto

package v3

import (
	v3 "github.com/cncf/xds/go/xds/type/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Int64RangeMatcher struct {
	state         protoimpl.MessageState            `protogen:"open.v1"`
	RangeMatchers []*Int64RangeMatcher_RangeMatcher `protobuf:"bytes,1,rep,name=range_matchers,json=rangeMatchers,proto3" json:"range_matchers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Int64RangeMatcher) Reset() {
	*x = Int64RangeMatcher{}
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Int64RangeMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Int64RangeMatcher) ProtoMessage() {}

func (x *Int64RangeMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Int64RangeMatcher.ProtoReflect.Descriptor instead.
func (*Int64RangeMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_range_proto_rawDescGZIP(), []int{0}
}

func (x *Int64RangeMatcher) GetRangeMatchers() []*Int64RangeMatcher_RangeMatcher {
	if x != nil {
		return x.RangeMatchers
	}
	return nil
}

type Int32RangeMatcher struct {
	state         protoimpl.MessageState            `protogen:"open.v1"`
	RangeMatchers []*Int32RangeMatcher_RangeMatcher `protobuf:"bytes,1,rep,name=range_matchers,json=rangeMatchers,proto3" json:"range_matchers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Int32RangeMatcher) Reset() {
	*x = Int32RangeMatcher{}
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Int32RangeMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Int32RangeMatcher) ProtoMessage() {}

func (x *Int32RangeMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Int32RangeMatcher.ProtoReflect.Descriptor instead.
func (*Int32RangeMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_range_proto_rawDescGZIP(), []int{1}
}

func (x *Int32RangeMatcher) GetRangeMatchers() []*Int32RangeMatcher_RangeMatcher {
	if x != nil {
		return x.RangeMatchers
	}
	return nil
}

type DoubleRangeMatcher struct {
	state         protoimpl.MessageState             `protogen:"open.v1"`
	RangeMatchers []*DoubleRangeMatcher_RangeMatcher `protobuf:"bytes,1,rep,name=range_matchers,json=rangeMatchers,proto3" json:"range_matchers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DoubleRangeMatcher) Reset() {
	*x = DoubleRangeMatcher{}
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DoubleRangeMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DoubleRangeMatcher) ProtoMessage() {}

func (x *DoubleRangeMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DoubleRangeMatcher.ProtoReflect.Descriptor instead.
func (*DoubleRangeMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_range_proto_rawDescGZIP(), []int{2}
}

func (x *DoubleRangeMatcher) GetRangeMatchers() []*DoubleRangeMatcher_RangeMatcher {
	if x != nil {
		return x.RangeMatchers
	}
	return nil
}

type Int64RangeMatcher_RangeMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ranges        []*v3.Int64Range       `protobuf:"bytes,1,rep,name=ranges,proto3" json:"ranges,omitempty"`
	OnMatch       *Matcher_OnMatch       `protobuf:"bytes,2,opt,name=on_match,json=onMatch,proto3" json:"on_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Int64RangeMatcher_RangeMatcher) Reset() {
	*x = Int64RangeMatcher_RangeMatcher{}
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Int64RangeMatcher_RangeMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Int64RangeMatcher_RangeMatcher) ProtoMessage() {}

func (x *Int64RangeMatcher_RangeMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Int64RangeMatcher_RangeMatcher.ProtoReflect.Descriptor instead.
func (*Int64RangeMatcher_RangeMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_range_proto_rawDescGZIP(), []int{0, 0}
}

func (x *Int64RangeMatcher_RangeMatcher) GetRanges() []*v3.Int64Range {
	if x != nil {
		return x.Ranges
	}
	return nil
}

func (x *Int64RangeMatcher_RangeMatcher) GetOnMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

type Int32RangeMatcher_RangeMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ranges        []*v3.Int32Range       `protobuf:"bytes,1,rep,name=ranges,proto3" json:"ranges,omitempty"`
	OnMatch       *Matcher_OnMatch       `protobuf:"bytes,2,opt,name=on_match,json=onMatch,proto3" json:"on_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Int32RangeMatcher_RangeMatcher) Reset() {
	*x = Int32RangeMatcher_RangeMatcher{}
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Int32RangeMatcher_RangeMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Int32RangeMatcher_RangeMatcher) ProtoMessage() {}

func (x *Int32RangeMatcher_RangeMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Int32RangeMatcher_RangeMatcher.ProtoReflect.Descriptor instead.
func (*Int32RangeMatcher_RangeMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_range_proto_rawDescGZIP(), []int{1, 0}
}

func (x *Int32RangeMatcher_RangeMatcher) GetRanges() []*v3.Int32Range {
	if x != nil {
		return x.Ranges
	}
	return nil
}

func (x *Int32RangeMatcher_RangeMatcher) GetOnMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

type DoubleRangeMatcher_RangeMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ranges        []*v3.DoubleRange      `protobuf:"bytes,1,rep,name=ranges,proto3" json:"ranges,omitempty"`
	OnMatch       *Matcher_OnMatch       `protobuf:"bytes,2,opt,name=on_match,json=onMatch,proto3" json:"on_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DoubleRangeMatcher_RangeMatcher) Reset() {
	*x = DoubleRangeMatcher_RangeMatcher{}
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DoubleRangeMatcher_RangeMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DoubleRangeMatcher_RangeMatcher) ProtoMessage() {}

func (x *DoubleRangeMatcher_RangeMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_range_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DoubleRangeMatcher_RangeMatcher.ProtoReflect.Descriptor instead.
func (*DoubleRangeMatcher_RangeMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_range_proto_rawDescGZIP(), []int{2, 0}
}

func (x *DoubleRangeMatcher_RangeMatcher) GetRanges() []*v3.DoubleRange {
	if x != nil {
		return x.Ranges
	}
	return nil
}

func (x *DoubleRangeMatcher_RangeMatcher) GetOnMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

var File_xds_type_matcher_v3_range_proto protoreflect.FileDescriptor

const file_xds_type_matcher_v3_range_proto_rawDesc = "" +
	"\n" +
	"\x1fxds/type/matcher/v3/range.proto\x12\x13xds.type.matcher.v3\x1a\x17xds/type/v3/range.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x17validate/validate.proto\"\xfc\x01\n" +
	"\x11Int64RangeMatcher\x12Z\n" +
	"\x0erange_matchers\x18\x01 \x03(\v23.xds.type.matcher.v3.Int64RangeMatcher.RangeMatcherR\rrangeMatchers\x1a\x8a\x01\n" +
	"\fRangeMatcher\x129\n" +
	"\x06ranges\x18\x01 \x03(\v2\x17.xds.type.v3.Int64RangeB\b\xfaB\x05\x92\x01\x02\b\x01R\x06ranges\x12?\n" +
	"\bon_match\x18\x02 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchR\aonMatch\"\xfc\x01\n" +
	"\x11Int32RangeMatcher\x12Z\n" +
	"\x0erange_matchers\x18\x01 \x03(\v23.xds.type.matcher.v3.Int32RangeMatcher.RangeMatcherR\rrangeMatchers\x1a\x8a\x01\n" +
	"\fRangeMatcher\x129\n" +
	"\x06ranges\x18\x01 \x03(\v2\x17.xds.type.v3.Int32RangeB\b\xfaB\x05\x92\x01\x02\b\x01R\x06ranges\x12?\n" +
	"\bon_match\x18\x02 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchR\aonMatch\"\xff\x01\n" +
	"\x12DoubleRangeMatcher\x12[\n" +
	"\x0erange_matchers\x18\x01 \x03(\v24.xds.type.matcher.v3.DoubleRangeMatcher.RangeMatcherR\rrangeMatchers\x1a\x8b\x01\n" +
	"\fRangeMatcher\x12:\n" +
	"\x06ranges\x18\x01 \x03(\v2\x18.xds.type.v3.DoubleRangeB\b\xfaB\x05\x92\x01\x02\b\x01R\x06ranges\x12?\n" +
	"\bon_match\x18\x02 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchR\aonMatchBZ\n" +
	"\x1ecom.github.xds.type.matcher.v3B\n" +
	"RangeProtoP\x01Z*github.com/cncf/xds/go/xds/type/matcher/v3b\x06proto3"

var (
	file_xds_type_matcher_v3_range_proto_rawDescOnce sync.Once
	file_xds_type_matcher_v3_range_proto_rawDescData []byte
)

func file_xds_type_matcher_v3_range_proto_rawDescGZIP() []byte {
	file_xds_type_matcher_v3_range_proto_rawDescOnce.Do(func() {
		file_xds_type_matcher_v3_range_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_range_proto_rawDesc), len(file_xds_type_matcher_v3_range_proto_rawDesc)))
	})
	return file_xds_type_matcher_v3_range_proto_rawDescData
}

var file_xds_type_matcher_v3_range_proto_msgTypes = make([]protoimpl.MessageInfo, 6)
var file_xds_type_matcher_v3_range_proto_goTypes = []any{
	(*Int64RangeMatcher)(nil),               // 0: xds.type.matcher.v3.Int64RangeMatcher
	(*Int32RangeMatcher)(nil),               // 1: xds.type.matcher.v3.Int32RangeMatcher
	(*DoubleRangeMatcher)(nil),              // 2: xds.type.matcher.v3.DoubleRangeMatcher
	(*Int64RangeMatcher_RangeMatcher)(nil),  // 3: xds.type.matcher.v3.Int64RangeMatcher.RangeMatcher
	(*Int32RangeMatcher_RangeMatcher)(nil),  // 4: xds.type.matcher.v3.Int32RangeMatcher.RangeMatcher
	(*DoubleRangeMatcher_RangeMatcher)(nil), // 5: xds.type.matcher.v3.DoubleRangeMatcher.RangeMatcher
	(*v3.Int64Range)(nil),                   // 6: xds.type.v3.Int64Range
	(*Matcher_OnMatch)(nil),                 // 7: xds.type.matcher.v3.Matcher.OnMatch
	(*v3.Int32Range)(nil),                   // 8: xds.type.v3.Int32Range
	(*v3.DoubleRange)(nil),                  // 9: xds.type.v3.DoubleRange
}
var file_xds_type_matcher_v3_range_proto_depIdxs = []int32{
	3, // 0: xds.type.matcher.v3.Int64RangeMatcher.range_matchers:type_name -> xds.type.matcher.v3.Int64RangeMatcher.RangeMatcher
	4, // 1: xds.type.matcher.v3.Int32RangeMatcher.range_matchers:type_name -> xds.type.matcher.v3.Int32RangeMatcher.RangeMatcher
	5, // 2: xds.type.matcher.v3.DoubleRangeMatcher.range_matchers:type_name -> xds.type.matcher.v3.DoubleRangeMatcher.RangeMatcher
	6, // 3: xds.type.matcher.v3.Int64RangeMatcher.RangeMatcher.ranges:type_name -> xds.type.v3.Int64Range
	7, // 4: xds.type.matcher.v3.Int64RangeMatcher.RangeMatcher.on_match:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	8, // 5: xds.type.matcher.v3.Int32RangeMatcher.RangeMatcher.ranges:type_name -> xds.type.v3.Int32Range
	7, // 6: xds.type.matcher.v3.Int32RangeMatcher.RangeMatcher.on_match:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	9, // 7: xds.type.matcher.v3.DoubleRangeMatcher.RangeMatcher.ranges:type_name -> xds.type.v3.DoubleRange
	7, // 8: xds.type.matcher.v3.DoubleRangeMatcher.RangeMatcher.on_match:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	9, // [9:9] is the sub-list for method output_type
	9, // [9:9] is the sub-list for method input_type
	9, // [9:9] is the sub-list for extension type_name
	9, // [9:9] is the sub-list for extension extendee
	0, // [0:9] is the sub-list for field type_name
}

func init() { file_xds_type_matcher_v3_range_proto_init() }
func file_xds_type_matcher_v3_range_proto_init() {
	if File_xds_type_matcher_v3_range_proto != nil {
		return
	}
	file_xds_type_matcher_v3_matcher_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_range_proto_rawDesc), len(file_xds_type_matcher_v3_range_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   6,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_type_matcher_v3_range_proto_goTypes,
		DependencyIndexes: file_xds_type_matcher_v3_range_proto_depIdxs,
		MessageInfos:      file_xds_type_matcher_v3_range_proto_msgTypes,
	}.Build()
	File_xds_type_matcher_v3_range_proto = out.File
	file_xds_type_matcher_v3_range_proto_goTypes = nil
	file_xds_type_matcher_v3_range_proto_depIdxs = nil
}
