// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Code generated by "deep-copy -type DiscoveryServiceConfigV1Alpha1 -pointer-receiver -header-file ../../../../../hack/boilerplate.txt -o deep_copy.generated.go ."; DO NOT EDIT.

package cluster

import (
	"net/url"
)

// DeepCopy generates a deep copy of *DiscoveryServiceConfigV1Alpha1.
func (o *DiscoveryServiceConfigV1Alpha1) DeepCopy() *DiscoveryServiceConfigV1Alpha1 {
	var cp DiscoveryServiceConfigV1Alpha1 = *o
	if o.EndpointURL.URL != nil {
		cp.EndpointURL.URL = new(url.URL)
		*cp.EndpointURL.URL = *o.EndpointURL.URL
		if o.EndpointURL.URL.User != nil {
			cp.EndpointURL.URL.User = new(url.Userinfo)
			*cp.EndpointURL.URL.User = *o.EndpointURL.URL.User
		}
	}
	return &cp
}
