// Code generated by smithy-go-codegen DO NOT EDIT.

package signin

import (
	"context"
	"fmt"
	"github.com/aws/smithy-go/middleware"
	"github.com/aws/smithy-go/ptr"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Create a permission statement in the account's SignIn resource-based policy
func (c *Client) PutResourcePermissionStatement(ctx context.Context, params *PutResourcePermissionStatementInput, optFns ...func(*Options)) (*PutResourcePermissionStatementOutput, error) {
	if params == nil {
		params = &PutResourcePermissionStatementInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "PutResourcePermissionStatement", params, optFns, c.addOperationPutResourcePermissionStatementMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*PutResourcePermissionStatementOutput)
	out.ResultMetadata = metadata
	return out, nil
}

// Input for PutResourcePermissionStatement operation
type PutResourcePermissionStatementInput struct {

	// Idempotency token for the request
	ClientToken *string

	// Console VPC endpoint identifier
	ConsoleSourceVpce *string

	// Principal to exclude from the permission statement
	ExcludedPrincipal *string

	// AWS region where the VPC and VPC endpoint reside Required when sourceVpc or
	// signinSourceVpce/consoleSourceVpce is provided
	RequestedRegion *string

	// SignIn VPC endpoint identifier
	SigninSourceVpce *string

	// Source IP address
	SourceIp *string

	// VPC identifier to restrict console access
	SourceVpc *string

	// Source IP address within VPC
	VpcSourceIp *string

	noSmithyDocumentSerde
}

func (in *PutResourcePermissionStatementInput) bindEndpointParams(p *EndpointParameters) {

	p.IsControlPlane = ptr.Bool(true)
}

// Output for PutResourcePermissionStatement operation
type PutResourcePermissionStatementOutput struct {

	// Unique identifier for the created permission statement
	//
	// This member is required.
	StatementId *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationPutResourcePermissionStatementMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsRestjson1_serializeOpPutResourcePermissionStatement{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpPutResourcePermissionStatement{}, middleware.After)
	if err != nil {
		return err
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addCredentialSource(stack, options); err != nil {
		return err
	}
	if err = addIdempotencyToken_opPutResourcePermissionStatementMiddleware(stack, options); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware(options.Region, "PutResourcePermissionStatement"), middleware.Before); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addInterceptors(stack, options); err != nil {
		return err
	}
	return nil
}

type idempotencyToken_initializeOpPutResourcePermissionStatement struct {
	tokenProvider IdempotencyTokenProvider
}

func (*idempotencyToken_initializeOpPutResourcePermissionStatement) ID() string {
	return "OperationIdempotencyTokenAutoFill"
}

func (m *idempotencyToken_initializeOpPutResourcePermissionStatement) HandleInitialize(ctx context.Context, in middleware.InitializeInput, next middleware.InitializeHandler) (
	out middleware.InitializeOutput, metadata middleware.Metadata, err error,
) {
	if m.tokenProvider == nil {
		return next.HandleInitialize(ctx, in)
	}

	input, ok := in.Parameters.(*PutResourcePermissionStatementInput)
	if !ok {
		return out, metadata, fmt.Errorf("expected middleware input to be of type *PutResourcePermissionStatementInput ")
	}

	if input.ClientToken == nil {
		t, err := m.tokenProvider.GetIdempotencyToken()
		if err != nil {
			return out, metadata, err
		}
		input.ClientToken = &t
	}
	return next.HandleInitialize(ctx, in)
}
func addIdempotencyToken_opPutResourcePermissionStatementMiddleware(stack *middleware.Stack, cfg Options) error {
	return stack.Initialize.Add(&idempotencyToken_initializeOpPutResourcePermissionStatement{tokenProvider: cfg.IdempotencyTokenProvider}, middleware.Before)
}
