// Copyright 2026 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/cloud/kms/v1/autokey_admin.proto

package kmspb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The states AutokeyConfig can be in.
type AutokeyConfig_State int32

const (
	// The state of the AutokeyConfig is unspecified.
	AutokeyConfig_STATE_UNSPECIFIED AutokeyConfig_State = 0
	// The AutokeyConfig is currently active.
	AutokeyConfig_ACTIVE AutokeyConfig_State = 1
	// A previously configured key project has been deleted and the current
	// AutokeyConfig is unusable.
	AutokeyConfig_KEY_PROJECT_DELETED AutokeyConfig_State = 2
	// The AutokeyConfig is not yet initialized or has been reset to its default
	// uninitialized state.
	AutokeyConfig_UNINITIALIZED AutokeyConfig_State = 3
	// The service account lacks the necessary permissions in the key project to
	// configure Autokey.
	AutokeyConfig_KEY_PROJECT_PERMISSION_DENIED AutokeyConfig_State = 4
)

// Enum value maps for AutokeyConfig_State.
var (
	AutokeyConfig_State_name = map[int32]string{
		0: "STATE_UNSPECIFIED",
		1: "ACTIVE",
		2: "KEY_PROJECT_DELETED",
		3: "UNINITIALIZED",
		4: "KEY_PROJECT_PERMISSION_DENIED",
	}
	AutokeyConfig_State_value = map[string]int32{
		"STATE_UNSPECIFIED":             0,
		"ACTIVE":                        1,
		"KEY_PROJECT_DELETED":           2,
		"UNINITIALIZED":                 3,
		"KEY_PROJECT_PERMISSION_DENIED": 4,
	}
)

func (x AutokeyConfig_State) Enum() *AutokeyConfig_State {
	p := new(AutokeyConfig_State)
	*p = x
	return p
}

func (x AutokeyConfig_State) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (AutokeyConfig_State) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_kms_v1_autokey_admin_proto_enumTypes[0].Descriptor()
}

func (AutokeyConfig_State) Type() protoreflect.EnumType {
	return &file_google_cloud_kms_v1_autokey_admin_proto_enumTypes[0]
}

func (x AutokeyConfig_State) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use AutokeyConfig_State.Descriptor instead.
func (AutokeyConfig_State) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{2, 0}
}

// Defines the resolution mode enum for the key project.
// The
// [KeyProjectResolutionMode][google.cloud.kms.v1.AutokeyConfig.KeyProjectResolutionMode]
// determines the mechanism by which
// [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig] identifies a
// [key_project][google.cloud.kms.v1.AutokeyConfig.key_project] at its
// specific configuration node. This parameter also determines if Autokey can
// be used within this project or folder.
type AutokeyConfig_KeyProjectResolutionMode int32

const (
	// Default value. KeyProjectResolutionMode when not specified will act as
	// `DEDICATED_KEY_PROJECT`.
	AutokeyConfig_KEY_PROJECT_RESOLUTION_MODE_UNSPECIFIED AutokeyConfig_KeyProjectResolutionMode = 0
	// Keys are created in a dedicated project specified by `key_project`.
	AutokeyConfig_DEDICATED_KEY_PROJECT AutokeyConfig_KeyProjectResolutionMode = 1
	// Keys are created in the same project as the resource requesting the key.
	// The `key_project` must not be set when this mode is used.
	AutokeyConfig_RESOURCE_PROJECT AutokeyConfig_KeyProjectResolutionMode = 2
	// Disables the AutokeyConfig. When this mode is set, any AutokeyConfig
	// from higher levels in the resource hierarchy are ignored for this
	// resource and its descendants. This setting can be overridden
	// by a more specific configuration at a lower level. For example,
	// if Autokey is disabled on a folder, it can be re-enabled on a sub-folder
	// or project within that folder by setting a different mode (e.g.,
	// DEDICATED_KEY_PROJECT or RESOURCE_PROJECT).
	AutokeyConfig_DISABLED AutokeyConfig_KeyProjectResolutionMode = 3
)

// Enum value maps for AutokeyConfig_KeyProjectResolutionMode.
var (
	AutokeyConfig_KeyProjectResolutionMode_name = map[int32]string{
		0: "KEY_PROJECT_RESOLUTION_MODE_UNSPECIFIED",
		1: "DEDICATED_KEY_PROJECT",
		2: "RESOURCE_PROJECT",
		3: "DISABLED",
	}
	AutokeyConfig_KeyProjectResolutionMode_value = map[string]int32{
		"KEY_PROJECT_RESOLUTION_MODE_UNSPECIFIED": 0,
		"DEDICATED_KEY_PROJECT":                   1,
		"RESOURCE_PROJECT":                        2,
		"DISABLED":                                3,
	}
)

func (x AutokeyConfig_KeyProjectResolutionMode) Enum() *AutokeyConfig_KeyProjectResolutionMode {
	p := new(AutokeyConfig_KeyProjectResolutionMode)
	*p = x
	return p
}

func (x AutokeyConfig_KeyProjectResolutionMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (AutokeyConfig_KeyProjectResolutionMode) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_kms_v1_autokey_admin_proto_enumTypes[1].Descriptor()
}

func (AutokeyConfig_KeyProjectResolutionMode) Type() protoreflect.EnumType {
	return &file_google_cloud_kms_v1_autokey_admin_proto_enumTypes[1]
}

func (x AutokeyConfig_KeyProjectResolutionMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use AutokeyConfig_KeyProjectResolutionMode.Descriptor instead.
func (AutokeyConfig_KeyProjectResolutionMode) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{2, 1}
}

// Request message for
// [UpdateAutokeyConfig][google.cloud.kms.v1.AutokeyAdmin.UpdateAutokeyConfig].
type UpdateAutokeyConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig] with values to
	// update.
	AutokeyConfig *AutokeyConfig `protobuf:"bytes,1,opt,name=autokey_config,json=autokeyConfig,proto3" json:"autokey_config,omitempty"`
	// Required. Masks which fields of the
	// [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig] to update, e.g.
	// `keyProject`.
	UpdateMask    *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateAutokeyConfigRequest) Reset() {
	*x = UpdateAutokeyConfigRequest{}
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAutokeyConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAutokeyConfigRequest) ProtoMessage() {}

func (x *UpdateAutokeyConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAutokeyConfigRequest.ProtoReflect.Descriptor instead.
func (*UpdateAutokeyConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{0}
}

func (x *UpdateAutokeyConfigRequest) GetAutokeyConfig() *AutokeyConfig {
	if x != nil {
		return x.AutokeyConfig
	}
	return nil
}

func (x *UpdateAutokeyConfigRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

// Request message for
// [GetAutokeyConfig][google.cloud.kms.v1.AutokeyAdmin.GetAutokeyConfig].
type GetAutokeyConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig]
	// resource, e.g. `folders/{FOLDER_NUMBER}/autokeyConfig` or
	// `projects/{PROJECT_NUMBER}/autokeyConfig`.
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetAutokeyConfigRequest) Reset() {
	*x = GetAutokeyConfigRequest{}
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetAutokeyConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetAutokeyConfigRequest) ProtoMessage() {}

func (x *GetAutokeyConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetAutokeyConfigRequest.ProtoReflect.Descriptor instead.
func (*GetAutokeyConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{1}
}

func (x *GetAutokeyConfigRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// Cloud KMS Autokey configuration for a folder.
type AutokeyConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Identifier. Name of the [AutokeyConfig][google.cloud.kms.v1.AutokeyConfig]
	// resource, e.g. `folders/{FOLDER_NUMBER}/autokeyConfig` or
	// `projects/{PROJECT_NUMBER}/autokeyConfig`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Optional. Name of the key project, e.g. `projects/{PROJECT_ID}` or
	// `projects/{PROJECT_NUMBER}`, where Cloud KMS Autokey will provision a new
	// [CryptoKey][google.cloud.kms.v1.CryptoKey] when a
	// [KeyHandle][google.cloud.kms.v1.KeyHandle] is created. On
	// [UpdateAutokeyConfig][google.cloud.kms.v1.AutokeyAdmin.UpdateAutokeyConfig],
	// the caller will require `cloudkms.cryptoKeys.setIamPolicy` permission on
	// this key project. Once configured, for Cloud KMS Autokey to function
	// properly, this key project must have the Cloud KMS API activated and the
	// Cloud KMS Service Agent for this key project must be granted the
	// `cloudkms.admin` role (or pertinent permissions). A request with an empty
	// key project field will clear the configuration.
	KeyProject string `protobuf:"bytes,2,opt,name=key_project,json=keyProject,proto3" json:"key_project,omitempty"`
	// Output only. The state for the AutokeyConfig.
	State AutokeyConfig_State `protobuf:"varint,4,opt,name=state,proto3,enum=google.cloud.kms.v1.AutokeyConfig_State" json:"state,omitempty"`
	// Optional. A checksum computed by the server based on the value of other
	// fields. This may be sent on update requests to ensure that the client has
	// an up-to-date value before proceeding. The request will be rejected with an
	// ABORTED error on a mismatched etag.
	Etag string `protobuf:"bytes,6,opt,name=etag,proto3" json:"etag,omitempty"`
	// Optional. KeyProjectResolutionMode for the AutokeyConfig.
	// Valid values are `DEDICATED_KEY_PROJECT`, `RESOURCE_PROJECT`, or
	// `DISABLED`.
	KeyProjectResolutionMode AutokeyConfig_KeyProjectResolutionMode `protobuf:"varint,8,opt,name=key_project_resolution_mode,json=keyProjectResolutionMode,proto3,enum=google.cloud.kms.v1.AutokeyConfig_KeyProjectResolutionMode" json:"key_project_resolution_mode,omitempty"`
	unknownFields            protoimpl.UnknownFields
	sizeCache                protoimpl.SizeCache
}

func (x *AutokeyConfig) Reset() {
	*x = AutokeyConfig{}
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AutokeyConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AutokeyConfig) ProtoMessage() {}

func (x *AutokeyConfig) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AutokeyConfig.ProtoReflect.Descriptor instead.
func (*AutokeyConfig) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{2}
}

func (x *AutokeyConfig) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *AutokeyConfig) GetKeyProject() string {
	if x != nil {
		return x.KeyProject
	}
	return ""
}

func (x *AutokeyConfig) GetState() AutokeyConfig_State {
	if x != nil {
		return x.State
	}
	return AutokeyConfig_STATE_UNSPECIFIED
}

func (x *AutokeyConfig) GetEtag() string {
	if x != nil {
		return x.Etag
	}
	return ""
}

func (x *AutokeyConfig) GetKeyProjectResolutionMode() AutokeyConfig_KeyProjectResolutionMode {
	if x != nil {
		return x.KeyProjectResolutionMode
	}
	return AutokeyConfig_KEY_PROJECT_RESOLUTION_MODE_UNSPECIFIED
}

// Request message for
// [ShowEffectiveAutokeyConfig][google.cloud.kms.v1.AutokeyAdmin.ShowEffectiveAutokeyConfig].
type ShowEffectiveAutokeyConfigRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the resource project to the show effective Cloud KMS
	// Autokey configuration for. This may be helpful for interrogating the effect
	// of nested folder configurations on a given resource project.
	Parent        string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ShowEffectiveAutokeyConfigRequest) Reset() {
	*x = ShowEffectiveAutokeyConfigRequest{}
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ShowEffectiveAutokeyConfigRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ShowEffectiveAutokeyConfigRequest) ProtoMessage() {}

func (x *ShowEffectiveAutokeyConfigRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ShowEffectiveAutokeyConfigRequest.ProtoReflect.Descriptor instead.
func (*ShowEffectiveAutokeyConfigRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{3}
}

func (x *ShowEffectiveAutokeyConfigRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

// Response message for
// [ShowEffectiveAutokeyConfig][google.cloud.kms.v1.AutokeyAdmin.ShowEffectiveAutokeyConfig].
type ShowEffectiveAutokeyConfigResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of the key project configured in the resource project's folder
	// ancestry.
	KeyProject    string `protobuf:"bytes,1,opt,name=key_project,json=keyProject,proto3" json:"key_project,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ShowEffectiveAutokeyConfigResponse) Reset() {
	*x = ShowEffectiveAutokeyConfigResponse{}
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ShowEffectiveAutokeyConfigResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ShowEffectiveAutokeyConfigResponse) ProtoMessage() {}

func (x *ShowEffectiveAutokeyConfigResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_kms_v1_autokey_admin_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ShowEffectiveAutokeyConfigResponse.ProtoReflect.Descriptor instead.
func (*ShowEffectiveAutokeyConfigResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP(), []int{4}
}

func (x *ShowEffectiveAutokeyConfigResponse) GetKeyProject() string {
	if x != nil {
		return x.KeyProject
	}
	return ""
}

var File_google_cloud_kms_v1_autokey_admin_proto protoreflect.FileDescriptor

const file_google_cloud_kms_v1_autokey_admin_proto_rawDesc = "" +
	"\n" +
	"'google/cloud/kms/v1/autokey_admin.proto\x12\x13google.cloud.kms.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a google/protobuf/field_mask.proto\"\xae\x01\n" +
	"\x1aUpdateAutokeyConfigRequest\x12N\n" +
	"\x0eautokey_config\x18\x01 \x01(\v2\".google.cloud.kms.v1.AutokeyConfigB\x03\xe0A\x02R\rautokeyConfig\x12@\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskB\x03\xe0A\x02R\n" +
	"updateMask\"\\\n" +
	"\x17GetAutokeyConfigRequest\x12A\n" +
	"\x04name\x18\x01 \x01(\tB-\xe0A\x02\xfaA'\n" +
	"%cloudkms.googleapis.com/AutokeyConfigR\x04name\"\xc0\x05\n" +
	"\rAutokeyConfig\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\bR\x04name\x12$\n" +
	"\vkey_project\x18\x02 \x01(\tB\x03\xe0A\x01R\n" +
	"keyProject\x12C\n" +
	"\x05state\x18\x04 \x01(\x0e2(.google.cloud.kms.v1.AutokeyConfig.StateB\x03\xe0A\x03R\x05state\x12\x17\n" +
	"\x04etag\x18\x06 \x01(\tB\x03\xe0A\x01R\x04etag\x12\x7f\n" +
	"\x1bkey_project_resolution_mode\x18\b \x01(\x0e2;.google.cloud.kms.v1.AutokeyConfig.KeyProjectResolutionModeB\x03\xe0A\x01R\x18keyProjectResolutionMode\"y\n" +
	"\x05State\x12\x15\n" +
	"\x11STATE_UNSPECIFIED\x10\x00\x12\n" +
	"\n" +
	"\x06ACTIVE\x10\x01\x12\x17\n" +
	"\x13KEY_PROJECT_DELETED\x10\x02\x12\x11\n" +
	"\rUNINITIALIZED\x10\x03\x12!\n" +
	"\x1dKEY_PROJECT_PERMISSION_DENIED\x10\x04\"\x86\x01\n" +
	"\x18KeyProjectResolutionMode\x12+\n" +
	"'KEY_PROJECT_RESOLUTION_MODE_UNSPECIFIED\x10\x00\x12\x19\n" +
	"\x15DEDICATED_KEY_PROJECT\x10\x01\x12\x14\n" +
	"\x10RESOURCE_PROJECT\x10\x02\x12\f\n" +
	"\bDISABLED\x10\x03:\x8c\x01\xeaA\x88\x01\n" +
	"%cloudkms.googleapis.com/AutokeyConfig\x12\x1efolders/{folder}/autokeyConfig\x12 projects/{project}/autokeyConfig*\x0eautokeyConfigs2\rautokeyConfig\"p\n" +
	"!ShowEffectiveAutokeyConfigRequest\x12K\n" +
	"\x06parent\x18\x01 \x01(\tB3\xe0A\x02\xfaA-\n" +
	"+cloudresourcemanager.googleapis.com/ProjectR\x06parent\"E\n" +
	"\"ShowEffectiveAutokeyConfigResponse\x12\x1f\n" +
	"\vkey_project\x18\x01 \x01(\tR\n" +
	"keyProject2\xb7\x06\n" +
	"\fAutokeyAdmin\x12\x9a\x02\n" +
	"\x13UpdateAutokeyConfig\x12/.google.cloud.kms.v1.UpdateAutokeyConfigRequest\x1a\".google.cloud.kms.v1.AutokeyConfig\"\xad\x01\xdaA\x1aautokey_config,update_mask\x82\xd3\xe4\x93\x02\x89\x01:\x0eautokey_configZD:\x0eautokey_config22/v1/{autokey_config.name=projects/*/autokeyConfig}21/v1/{autokey_config.name=folders/*/autokeyConfig}\x12\xbe\x01\n" +
	"\x10GetAutokeyConfig\x12,.google.cloud.kms.v1.GetAutokeyConfigRequest\x1a\".google.cloud.kms.v1.AutokeyConfig\"X\xdaA\x04name\x82\xd3\xe4\x93\x02KZ%\x12#/v1/{name=projects/*/autokeyConfig}\x12\"/v1/{name=folders/*/autokeyConfig}\x12\xd2\x01\n" +
	"\x1aShowEffectiveAutokeyConfig\x126.google.cloud.kms.v1.ShowEffectiveAutokeyConfigRequest\x1a7.google.cloud.kms.v1.ShowEffectiveAutokeyConfigResponse\"C\xdaA\x06parent\x82\xd3\xe4\x93\x024\x122/v1/{parent=projects/*}:showEffectiveAutokeyConfig\x1at\xcaA\x17cloudkms.googleapis.com\xd2AWhttps://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/cloudkmsBY\n" +
	"\x17com.google.cloud.kms.v1B\x11AutokeyAdminProtoP\x01Z)cloud.google.com/go/kms/apiv1/kmspb;kmspbb\x06proto3"

var (
	file_google_cloud_kms_v1_autokey_admin_proto_rawDescOnce sync.Once
	file_google_cloud_kms_v1_autokey_admin_proto_rawDescData []byte
)

func file_google_cloud_kms_v1_autokey_admin_proto_rawDescGZIP() []byte {
	file_google_cloud_kms_v1_autokey_admin_proto_rawDescOnce.Do(func() {
		file_google_cloud_kms_v1_autokey_admin_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_kms_v1_autokey_admin_proto_rawDesc), len(file_google_cloud_kms_v1_autokey_admin_proto_rawDesc)))
	})
	return file_google_cloud_kms_v1_autokey_admin_proto_rawDescData
}

var file_google_cloud_kms_v1_autokey_admin_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_google_cloud_kms_v1_autokey_admin_proto_msgTypes = make([]protoimpl.MessageInfo, 5)
var file_google_cloud_kms_v1_autokey_admin_proto_goTypes = []any{
	(AutokeyConfig_State)(0),                    // 0: google.cloud.kms.v1.AutokeyConfig.State
	(AutokeyConfig_KeyProjectResolutionMode)(0), // 1: google.cloud.kms.v1.AutokeyConfig.KeyProjectResolutionMode
	(*UpdateAutokeyConfigRequest)(nil),          // 2: google.cloud.kms.v1.UpdateAutokeyConfigRequest
	(*GetAutokeyConfigRequest)(nil),             // 3: google.cloud.kms.v1.GetAutokeyConfigRequest
	(*AutokeyConfig)(nil),                       // 4: google.cloud.kms.v1.AutokeyConfig
	(*ShowEffectiveAutokeyConfigRequest)(nil),   // 5: google.cloud.kms.v1.ShowEffectiveAutokeyConfigRequest
	(*ShowEffectiveAutokeyConfigResponse)(nil),  // 6: google.cloud.kms.v1.ShowEffectiveAutokeyConfigResponse
	(*fieldmaskpb.FieldMask)(nil),               // 7: google.protobuf.FieldMask
}
var file_google_cloud_kms_v1_autokey_admin_proto_depIdxs = []int32{
	4, // 0: google.cloud.kms.v1.UpdateAutokeyConfigRequest.autokey_config:type_name -> google.cloud.kms.v1.AutokeyConfig
	7, // 1: google.cloud.kms.v1.UpdateAutokeyConfigRequest.update_mask:type_name -> google.protobuf.FieldMask
	0, // 2: google.cloud.kms.v1.AutokeyConfig.state:type_name -> google.cloud.kms.v1.AutokeyConfig.State
	1, // 3: google.cloud.kms.v1.AutokeyConfig.key_project_resolution_mode:type_name -> google.cloud.kms.v1.AutokeyConfig.KeyProjectResolutionMode
	2, // 4: google.cloud.kms.v1.AutokeyAdmin.UpdateAutokeyConfig:input_type -> google.cloud.kms.v1.UpdateAutokeyConfigRequest
	3, // 5: google.cloud.kms.v1.AutokeyAdmin.GetAutokeyConfig:input_type -> google.cloud.kms.v1.GetAutokeyConfigRequest
	5, // 6: google.cloud.kms.v1.AutokeyAdmin.ShowEffectiveAutokeyConfig:input_type -> google.cloud.kms.v1.ShowEffectiveAutokeyConfigRequest
	4, // 7: google.cloud.kms.v1.AutokeyAdmin.UpdateAutokeyConfig:output_type -> google.cloud.kms.v1.AutokeyConfig
	4, // 8: google.cloud.kms.v1.AutokeyAdmin.GetAutokeyConfig:output_type -> google.cloud.kms.v1.AutokeyConfig
	6, // 9: google.cloud.kms.v1.AutokeyAdmin.ShowEffectiveAutokeyConfig:output_type -> google.cloud.kms.v1.ShowEffectiveAutokeyConfigResponse
	7, // [7:10] is the sub-list for method output_type
	4, // [4:7] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_google_cloud_kms_v1_autokey_admin_proto_init() }
func file_google_cloud_kms_v1_autokey_admin_proto_init() {
	if File_google_cloud_kms_v1_autokey_admin_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_kms_v1_autokey_admin_proto_rawDesc), len(file_google_cloud_kms_v1_autokey_admin_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   5,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_cloud_kms_v1_autokey_admin_proto_goTypes,
		DependencyIndexes: file_google_cloud_kms_v1_autokey_admin_proto_depIdxs,
		EnumInfos:         file_google_cloud_kms_v1_autokey_admin_proto_enumTypes,
		MessageInfos:      file_google_cloud_kms_v1_autokey_admin_proto_msgTypes,
	}.Build()
	File_google_cloud_kms_v1_autokey_admin_proto = out.File
	file_google_cloud_kms_v1_autokey_admin_proto_goTypes = nil
	file_google_cloud_kms_v1_autokey_admin_proto_depIdxs = nil
}
