// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/cluster/v3/filter.proto

package clusterv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Filter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the filter configuration.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Filter specific configuration which depends on the filter being
	// instantiated. See the supported filters for further documentation.
	// Note that Envoy's :ref:`downstream network
	// filters <config_network_filters>` are not valid upstream network filters.
	// Only one of typed_config or config_discovery can be used.
	TypedConfig *anypb.Any `protobuf:"bytes,2,opt,name=typed_config,json=typedConfig,proto3" json:"typed_config,omitempty"`
	// Configuration source specifier for an extension configuration discovery
	// service. In case of a failure and without the default configuration, the
	// listener closes the connections.
	// Only one of typed_config or config_discovery can be used.
	ConfigDiscovery *v3.ExtensionConfigSource `protobuf:"bytes,3,opt,name=config_discovery,json=configDiscovery,proto3" json:"config_discovery,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Filter) Reset() {
	*x = Filter{}
	mi := &file_envoy_config_cluster_v3_filter_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Filter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Filter) ProtoMessage() {}

func (x *Filter) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_cluster_v3_filter_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Filter.ProtoReflect.Descriptor instead.
func (*Filter) Descriptor() ([]byte, []int) {
	return file_envoy_config_cluster_v3_filter_proto_rawDescGZIP(), []int{0}
}

func (x *Filter) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Filter) GetTypedConfig() *anypb.Any {
	if x != nil {
		return x.TypedConfig
	}
	return nil
}

func (x *Filter) GetConfigDiscovery() *v3.ExtensionConfigSource {
	if x != nil {
		return x.ConfigDiscovery
	}
	return nil
}

var File_envoy_config_cluster_v3_filter_proto protoreflect.FileDescriptor

const file_envoy_config_cluster_v3_filter_proto_rawDesc = "" +
	"\n" +
	"$envoy/config/cluster/v3/filter.proto\x12\x17envoy.config.cluster.v3\x1a(envoy/config/core/v3/config_source.proto\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xda\x01\n" +
	"\x06Filter\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x127\n" +
	"\ftyped_config\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\vtypedConfig\x12V\n" +
	"\x10config_discovery\x18\x03 \x01(\v2+.envoy.config.core.v3.ExtensionConfigSourceR\x0fconfigDiscovery:\"\x9aň\x1e\x1d\n" +
	"\x1benvoy.api.v2.cluster.FilterB\x88\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"%io.envoyproxy.envoy.config.cluster.v3B\vFilterProtoP\x01ZHgithub.com/envoyproxy/go-control-plane/envoy/config/cluster/v3;clusterv3b\x06proto3"

var (
	file_envoy_config_cluster_v3_filter_proto_rawDescOnce sync.Once
	file_envoy_config_cluster_v3_filter_proto_rawDescData []byte
)

func file_envoy_config_cluster_v3_filter_proto_rawDescGZIP() []byte {
	file_envoy_config_cluster_v3_filter_proto_rawDescOnce.Do(func() {
		file_envoy_config_cluster_v3_filter_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_cluster_v3_filter_proto_rawDesc), len(file_envoy_config_cluster_v3_filter_proto_rawDesc)))
	})
	return file_envoy_config_cluster_v3_filter_proto_rawDescData
}

var file_envoy_config_cluster_v3_filter_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_cluster_v3_filter_proto_goTypes = []any{
	(*Filter)(nil),                   // 0: envoy.config.cluster.v3.Filter
	(*anypb.Any)(nil),                // 1: google.protobuf.Any
	(*v3.ExtensionConfigSource)(nil), // 2: envoy.config.core.v3.ExtensionConfigSource
}
var file_envoy_config_cluster_v3_filter_proto_depIdxs = []int32{
	1, // 0: envoy.config.cluster.v3.Filter.typed_config:type_name -> google.protobuf.Any
	2, // 1: envoy.config.cluster.v3.Filter.config_discovery:type_name -> envoy.config.core.v3.ExtensionConfigSource
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_config_cluster_v3_filter_proto_init() }
func file_envoy_config_cluster_v3_filter_proto_init() {
	if File_envoy_config_cluster_v3_filter_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_cluster_v3_filter_proto_rawDesc), len(file_envoy_config_cluster_v3_filter_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_cluster_v3_filter_proto_goTypes,
		DependencyIndexes: file_envoy_config_cluster_v3_filter_proto_depIdxs,
		MessageInfos:      file_envoy_config_cluster_v3_filter_proto_msgTypes,
	}.Build()
	File_envoy_config_cluster_v3_filter_proto = out.File
	file_envoy_config_cluster_v3_filter_proto_goTypes = nil
	file_envoy_config_cluster_v3_filter_proto_depIdxs = nil
}
