// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: xds/type/v3/typed_struct.proto

package v3

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type TypedStruct struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	TypeUrl       string                 `protobuf:"bytes,1,opt,name=type_url,json=typeUrl,proto3" json:"type_url,omitempty"`
	Value         *structpb.Struct       `protobuf:"bytes,2,opt,name=value,proto3" json:"value,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TypedStruct) Reset() {
	*x = TypedStruct{}
	mi := &file_xds_type_v3_typed_struct_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TypedStruct) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TypedStruct) ProtoMessage() {}

func (x *TypedStruct) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_v3_typed_struct_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TypedStruct.ProtoReflect.Descriptor instead.
func (*TypedStruct) Descriptor() ([]byte, []int) {
	return file_xds_type_v3_typed_struct_proto_rawDescGZIP(), []int{0}
}

func (x *TypedStruct) GetTypeUrl() string {
	if x != nil {
		return x.TypeUrl
	}
	return ""
}

func (x *TypedStruct) GetValue() *structpb.Struct {
	if x != nil {
		return x.Value
	}
	return nil
}

var File_xds_type_v3_typed_struct_proto protoreflect.FileDescriptor

const file_xds_type_v3_typed_struct_proto_rawDesc = "" +
	"\n" +
	"\x1exds/type/v3/typed_struct.proto\x12\vxds.type.v3\x1a\x1cgoogle/protobuf/struct.proto\"W\n" +
	"\vTypedStruct\x12\x19\n" +
	"\btype_url\x18\x01 \x01(\tR\atypeUrl\x12-\n" +
	"\x05value\x18\x02 \x01(\v2\x17.google.protobuf.StructR\x05valueBP\n" +
	"\x16com.github.xds.type.v3B\x10TypedStructProtoP\x01Z\"github.com/cncf/xds/go/xds/type/v3b\x06proto3"

var (
	file_xds_type_v3_typed_struct_proto_rawDescOnce sync.Once
	file_xds_type_v3_typed_struct_proto_rawDescData []byte
)

func file_xds_type_v3_typed_struct_proto_rawDescGZIP() []byte {
	file_xds_type_v3_typed_struct_proto_rawDescOnce.Do(func() {
		file_xds_type_v3_typed_struct_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_type_v3_typed_struct_proto_rawDesc), len(file_xds_type_v3_typed_struct_proto_rawDesc)))
	})
	return file_xds_type_v3_typed_struct_proto_rawDescData
}

var file_xds_type_v3_typed_struct_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_xds_type_v3_typed_struct_proto_goTypes = []any{
	(*TypedStruct)(nil),     // 0: xds.type.v3.TypedStruct
	(*structpb.Struct)(nil), // 1: google.protobuf.Struct
}
var file_xds_type_v3_typed_struct_proto_depIdxs = []int32{
	1, // 0: xds.type.v3.TypedStruct.value:type_name -> google.protobuf.Struct
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_xds_type_v3_typed_struct_proto_init() }
func file_xds_type_v3_typed_struct_proto_init() {
	if File_xds_type_v3_typed_struct_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_type_v3_typed_struct_proto_rawDesc), len(file_xds_type_v3_typed_struct_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_type_v3_typed_struct_proto_goTypes,
		DependencyIndexes: file_xds_type_v3_typed_struct_proto_depIdxs,
		MessageInfos:      file_xds_type_v3_typed_struct_proto_msgTypes,
	}.Build()
	File_xds_type_v3_typed_struct_proto = out.File
	file_xds_type_v3_typed_struct_proto_goTypes = nil
	file_xds_type_v3_typed_struct_proto_depIdxs = nil
}
