// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/monitoring/v3/snooze_service.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The message definition for creating a `Snooze`. Users must provide the body
// of the `Snooze` to be created but must omit the `Snooze` field, `name`.
type CreateSnoozeRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The
	// [project](https://cloud.google.com/monitoring/api/v3#project_name) in which
	// a `Snooze` should be created. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Required. The `Snooze` to create. Omit the `name` field, as it will be
	// filled in by the API.
	Snooze        *Snooze `protobuf:"bytes,2,opt,name=snooze,proto3" json:"snooze,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateSnoozeRequest) Reset() {
	*x = CreateSnoozeRequest{}
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateSnoozeRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateSnoozeRequest) ProtoMessage() {}

func (x *CreateSnoozeRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateSnoozeRequest.ProtoReflect.Descriptor instead.
func (*CreateSnoozeRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_service_proto_rawDescGZIP(), []int{0}
}

func (x *CreateSnoozeRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateSnoozeRequest) GetSnooze() *Snooze {
	if x != nil {
		return x.Snooze
	}
	return nil
}

// The message definition for listing `Snooze`s associated with the given
// `parent`, satisfying the optional `filter`.
type ListSnoozesRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The
	// [project](https://cloud.google.com/monitoring/api/v3#project_name) whose
	// `Snooze`s should be listed. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Optional. Optional filter to restrict results to the given criteria. The
	// following fields are supported.
	//
	//   - `interval.start_time`
	//   - `interval.end_time`
	//
	// For example:
	//
	//	interval.start_time > "2022-03-11T00:00:00-08:00" AND
	//	    interval.end_time < "2022-03-12T00:00:00-08:00"
	Filter string `protobuf:"bytes,2,opt,name=filter,proto3" json:"filter,omitempty"`
	// Optional. The maximum number of results to return for a single query. The
	// server may further constrain the maximum number of results returned in a
	// single page. The value should be in the range [1, 1000]. If the value given
	// is outside this range, the server will decide the number of results to be
	// returned.
	PageSize int32 `protobuf:"varint,4,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Optional. The `next_page_token` from a previous call to
	// `ListSnoozesRequest` to get the next page of results.
	PageToken     string `protobuf:"bytes,5,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListSnoozesRequest) Reset() {
	*x = ListSnoozesRequest{}
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListSnoozesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListSnoozesRequest) ProtoMessage() {}

func (x *ListSnoozesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListSnoozesRequest.ProtoReflect.Descriptor instead.
func (*ListSnoozesRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListSnoozesRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *ListSnoozesRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *ListSnoozesRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListSnoozesRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

// The results of a successful `ListSnoozes` call, containing the matching
// `Snooze`s.
type ListSnoozesResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// `Snooze`s matching this list call.
	Snoozes []*Snooze `protobuf:"bytes,1,rep,name=snoozes,proto3" json:"snoozes,omitempty"`
	// Page token for repeated calls to `ListSnoozes`, to fetch additional pages
	// of results. If this is empty or missing, there are no more pages.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListSnoozesResponse) Reset() {
	*x = ListSnoozesResponse{}
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListSnoozesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListSnoozesResponse) ProtoMessage() {}

func (x *ListSnoozesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListSnoozesResponse.ProtoReflect.Descriptor instead.
func (*ListSnoozesResponse) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListSnoozesResponse) GetSnoozes() []*Snooze {
	if x != nil {
		return x.Snoozes
	}
	return nil
}

func (x *ListSnoozesResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

// The message definition for retrieving a `Snooze`. Users must specify the
// field, `name`, which identifies the `Snooze`.
type GetSnoozeRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The ID of the `Snooze` to retrieve. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/snoozes/[SNOOZE_ID]
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetSnoozeRequest) Reset() {
	*x = GetSnoozeRequest{}
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetSnoozeRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetSnoozeRequest) ProtoMessage() {}

func (x *GetSnoozeRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetSnoozeRequest.ProtoReflect.Descriptor instead.
func (*GetSnoozeRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_service_proto_rawDescGZIP(), []int{3}
}

func (x *GetSnoozeRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// The message definition for updating a `Snooze`. The field, `snooze.name`
// identifies the `Snooze` to be updated. The remainder of `snooze` gives the
// content the `Snooze` in question will be assigned.
//
// What fields can be updated depends on the start time and end time of the
// `Snooze`.
//
//   - end time is in the past: These `Snooze`s are considered
//     read-only and cannot be updated.
//   - start time is in the past and end time is in the future: `display_name`
//     and `interval.end_time` can be updated.
//   - start time is in the future: `display_name`, `interval.start_time` and
//     `interval.end_time` can be updated.
type UpdateSnoozeRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The `Snooze` to update. Must have the name field present.
	Snooze *Snooze `protobuf:"bytes,1,opt,name=snooze,proto3" json:"snooze,omitempty"`
	// Required. The fields to update.
	//
	// For each field listed in `update_mask`:
	//
	//   - If the `Snooze` object supplied in the `UpdateSnoozeRequest` has a
	//     value for that field, the value of the field in the existing `Snooze`
	//     will be set to the value of the field in the supplied `Snooze`.
	//   - If the field does not have a value in the supplied `Snooze`, the field
	//     in the existing `Snooze` is set to its default value.
	//
	// Fields not listed retain their existing value.
	//
	// The following are the field names that are accepted in `update_mask`:
	//
	//   - `display_name`
	//   - `interval.start_time`
	//   - `interval.end_time`
	//
	// That said, the start time and end time of the `Snooze` determines which
	// fields can legally be updated. Before attempting an update, users should
	// consult the documentation for `UpdateSnoozeRequest`, which talks about
	// which fields can be updated.
	UpdateMask    *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateSnoozeRequest) Reset() {
	*x = UpdateSnoozeRequest{}
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateSnoozeRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateSnoozeRequest) ProtoMessage() {}

func (x *UpdateSnoozeRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_snooze_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateSnoozeRequest.ProtoReflect.Descriptor instead.
func (*UpdateSnoozeRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_snooze_service_proto_rawDescGZIP(), []int{4}
}

func (x *UpdateSnoozeRequest) GetSnooze() *Snooze {
	if x != nil {
		return x.Snooze
	}
	return nil
}

func (x *UpdateSnoozeRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

var File_google_monitoring_v3_snooze_service_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_snooze_service_proto_rawDesc = "" +
	"\n" +
	")google/monitoring/v3/snooze_service.proto\x12\x14google.monitoring.v3\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a!google/monitoring/v3/snooze.proto\x1a google/protobuf/field_mask.proto\"\x92\x01\n" +
	"\x13CreateSnoozeRequest\x12@\n" +
	"\x06parent\x18\x01 \x01(\tB(\xe0A\x02\xfaA\"\x12 monitoring.googleapis.com/SnoozeR\x06parent\x129\n" +
	"\x06snooze\x18\x02 \x01(\v2\x1c.google.monitoring.v3.SnoozeB\x03\xe0A\x02R\x06snooze\"\xb9\x01\n" +
	"\x12ListSnoozesRequest\x12@\n" +
	"\x06parent\x18\x01 \x01(\tB(\xe0A\x02\xfaA\"\x12 monitoring.googleapis.com/SnoozeR\x06parent\x12\x1b\n" +
	"\x06filter\x18\x02 \x01(\tB\x03\xe0A\x01R\x06filter\x12 \n" +
	"\tpage_size\x18\x04 \x01(\x05B\x03\xe0A\x01R\bpageSize\x12\"\n" +
	"\n" +
	"page_token\x18\x05 \x01(\tB\x03\xe0A\x01R\tpageToken\"u\n" +
	"\x13ListSnoozesResponse\x126\n" +
	"\asnoozes\x18\x01 \x03(\v2\x1c.google.monitoring.v3.SnoozeR\asnoozes\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"P\n" +
	"\x10GetSnoozeRequest\x12<\n" +
	"\x04name\x18\x01 \x01(\tB(\xe0A\x02\xfaA\"\n" +
	" monitoring.googleapis.com/SnoozeR\x04name\"\x92\x01\n" +
	"\x13UpdateSnoozeRequest\x129\n" +
	"\x06snooze\x18\x01 \x01(\v2\x1c.google.monitoring.v3.SnoozeB\x03\xe0A\x02R\x06snooze\x12@\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskB\x03\xe0A\x02R\n" +
	"updateMask2\x98\x06\n" +
	"\rSnoozeService\x12\x98\x01\n" +
	"\fCreateSnooze\x12).google.monitoring.v3.CreateSnoozeRequest\x1a\x1c.google.monitoring.v3.Snooze\"?\xdaA\rparent,snooze\x82\xd3\xe4\x93\x02):\x06snooze\"\x1f/v3/{parent=projects/*}/snoozes\x12\x94\x01\n" +
	"\vListSnoozes\x12(.google.monitoring.v3.ListSnoozesRequest\x1a).google.monitoring.v3.ListSnoozesResponse\"0\xdaA\x06parent\x82\xd3\xe4\x93\x02!\x12\x1f/v3/{parent=projects/*}/snoozes\x12\x81\x01\n" +
	"\tGetSnooze\x12&.google.monitoring.v3.GetSnoozeRequest\x1a\x1c.google.monitoring.v3.Snooze\".\xdaA\x04name\x82\xd3\xe4\x93\x02!\x12\x1f/v3/{name=projects/*/snoozes/*}\x12\xa4\x01\n" +
	"\fUpdateSnooze\x12).google.monitoring.v3.UpdateSnoozeRequest\x1a\x1c.google.monitoring.v3.Snooze\"K\xdaA\x12snooze,update_mask\x82\xd3\xe4\x93\x020:\x06snooze2&/v3/{snooze.name=projects/*/snoozes/*}\x1a\xa9\x01\xcaA\x19monitoring.googleapis.com\xd2A\x89\x01https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/monitoring,https://www.googleapis.com/auth/monitoring.readB\xcd\x01\n" +
	"\x18com.google.monitoring.v3B\x12SnoozeServiceProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_snooze_service_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_snooze_service_proto_rawDescData []byte
)

func file_google_monitoring_v3_snooze_service_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_snooze_service_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_snooze_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_snooze_service_proto_rawDesc), len(file_google_monitoring_v3_snooze_service_proto_rawDesc)))
	})
	return file_google_monitoring_v3_snooze_service_proto_rawDescData
}

var file_google_monitoring_v3_snooze_service_proto_msgTypes = make([]protoimpl.MessageInfo, 5)
var file_google_monitoring_v3_snooze_service_proto_goTypes = []any{
	(*CreateSnoozeRequest)(nil),   // 0: google.monitoring.v3.CreateSnoozeRequest
	(*ListSnoozesRequest)(nil),    // 1: google.monitoring.v3.ListSnoozesRequest
	(*ListSnoozesResponse)(nil),   // 2: google.monitoring.v3.ListSnoozesResponse
	(*GetSnoozeRequest)(nil),      // 3: google.monitoring.v3.GetSnoozeRequest
	(*UpdateSnoozeRequest)(nil),   // 4: google.monitoring.v3.UpdateSnoozeRequest
	(*Snooze)(nil),                // 5: google.monitoring.v3.Snooze
	(*fieldmaskpb.FieldMask)(nil), // 6: google.protobuf.FieldMask
}
var file_google_monitoring_v3_snooze_service_proto_depIdxs = []int32{
	5, // 0: google.monitoring.v3.CreateSnoozeRequest.snooze:type_name -> google.monitoring.v3.Snooze
	5, // 1: google.monitoring.v3.ListSnoozesResponse.snoozes:type_name -> google.monitoring.v3.Snooze
	5, // 2: google.monitoring.v3.UpdateSnoozeRequest.snooze:type_name -> google.monitoring.v3.Snooze
	6, // 3: google.monitoring.v3.UpdateSnoozeRequest.update_mask:type_name -> google.protobuf.FieldMask
	0, // 4: google.monitoring.v3.SnoozeService.CreateSnooze:input_type -> google.monitoring.v3.CreateSnoozeRequest
	1, // 5: google.monitoring.v3.SnoozeService.ListSnoozes:input_type -> google.monitoring.v3.ListSnoozesRequest
	3, // 6: google.monitoring.v3.SnoozeService.GetSnooze:input_type -> google.monitoring.v3.GetSnoozeRequest
	4, // 7: google.monitoring.v3.SnoozeService.UpdateSnooze:input_type -> google.monitoring.v3.UpdateSnoozeRequest
	5, // 8: google.monitoring.v3.SnoozeService.CreateSnooze:output_type -> google.monitoring.v3.Snooze
	2, // 9: google.monitoring.v3.SnoozeService.ListSnoozes:output_type -> google.monitoring.v3.ListSnoozesResponse
	5, // 10: google.monitoring.v3.SnoozeService.GetSnooze:output_type -> google.monitoring.v3.Snooze
	5, // 11: google.monitoring.v3.SnoozeService.UpdateSnooze:output_type -> google.monitoring.v3.Snooze
	8, // [8:12] is the sub-list for method output_type
	4, // [4:8] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_snooze_service_proto_init() }
func file_google_monitoring_v3_snooze_service_proto_init() {
	if File_google_monitoring_v3_snooze_service_proto != nil {
		return
	}
	file_google_monitoring_v3_snooze_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_snooze_service_proto_rawDesc), len(file_google_monitoring_v3_snooze_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   5,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_monitoring_v3_snooze_service_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_snooze_service_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_snooze_service_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_snooze_service_proto = out.File
	file_google_monitoring_v3_snooze_service_proto_goTypes = nil
	file_google_monitoring_v3_snooze_service_proto_depIdxs = nil
}
