// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/trace.proto

package tracev3

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var File_envoy_config_trace_v3_trace_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_trace_proto_rawDesc = "" +
	"\n" +
	"!envoy/config/trace/v3/trace.proto\x12\x15envoy.config.trace.v3\x1a#envoy/config/trace/v3/datadog.proto\x1a&envoy/config/trace/v3/dynamic_ot.proto\x1a'envoy/config/trace/v3/http_tracer.proto\x1a%envoy/config/trace/v3/lightstep.proto\x1a)envoy/config/trace/v3/opentelemetry.proto\x1a#envoy/config/trace/v3/service.proto\x1a\"envoy/config/trace/v3/zipkin.protoBy\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\n" +
	"TraceProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3P\x00P\x01P\x02P\x03P\x04P\x05P\x06b\x06proto3"

var file_envoy_config_trace_v3_trace_proto_goTypes = []any{}
var file_envoy_config_trace_v3_trace_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_trace_proto_init() }
func file_envoy_config_trace_v3_trace_proto_init() {
	if File_envoy_config_trace_v3_trace_proto != nil {
		return
	}
	file_envoy_config_trace_v3_datadog_proto_init()
	file_envoy_config_trace_v3_dynamic_ot_proto_init()
	file_envoy_config_trace_v3_http_tracer_proto_init()
	file_envoy_config_trace_v3_lightstep_proto_init()
	file_envoy_config_trace_v3_opentelemetry_proto_init()
	file_envoy_config_trace_v3_service_proto_init()
	file_envoy_config_trace_v3_zipkin_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_trace_proto_rawDesc), len(file_envoy_config_trace_v3_trace_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_trace_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_trace_proto_depIdxs,
	}.Build()
	File_envoy_config_trace_v3_trace_proto = out.File
	file_envoy_config_trace_v3_trace_proto_goTypes = nil
	file_envoy_config_trace_v3_trace_proto_depIdxs = nil
}
