// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/monitoring/v3/service_service.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The `CreateService` request.
type CreateServiceRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource
	// [name](https://cloud.google.com/monitoring/api/v3#project_name) of the
	// parent Metrics Scope. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Optional. The Service id to use for this Service. If omitted, an id will be
	// generated instead. Must match the pattern `[a-z0-9\-]+`
	ServiceId string `protobuf:"bytes,3,opt,name=service_id,json=serviceId,proto3" json:"service_id,omitempty"`
	// Required. The `Service` to create.
	Service       *Service `protobuf:"bytes,2,opt,name=service,proto3" json:"service,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateServiceRequest) Reset() {
	*x = CreateServiceRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateServiceRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateServiceRequest) ProtoMessage() {}

func (x *CreateServiceRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateServiceRequest.ProtoReflect.Descriptor instead.
func (*CreateServiceRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{0}
}

func (x *CreateServiceRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateServiceRequest) GetServiceId() string {
	if x != nil {
		return x.ServiceId
	}
	return ""
}

func (x *CreateServiceRequest) GetService() *Service {
	if x != nil {
		return x.Service
	}
	return nil
}

// The `GetService` request.
type GetServiceRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the `Service`. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetServiceRequest) Reset() {
	*x = GetServiceRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetServiceRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetServiceRequest) ProtoMessage() {}

func (x *GetServiceRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetServiceRequest.ProtoReflect.Descriptor instead.
func (*GetServiceRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{1}
}

func (x *GetServiceRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// The `ListServices` request.
type ListServicesRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the parent containing the listed services,
	// either a [project](https://cloud.google.com/monitoring/api/v3#project_name)
	// or a Monitoring Metrics Scope. The formats are:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	//	workspaces/[HOST_PROJECT_ID_OR_NUMBER]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// A filter specifying what `Service`s to return. The filter supports
	// filtering on a particular service-identifier type or one of its attributes.
	//
	// To filter on a particular service-identifier type, the `identifier_case`
	// refers to which option in the `identifier` field is populated. For example,
	// the filter `identifier_case = "CUSTOM"` would match all services with a
	// value for the `custom` field. Valid options include "CUSTOM", "APP_ENGINE",
	// "MESH_ISTIO", and the other options listed at
	// https://cloud.google.com/monitoring/api/ref_v3/rest/v3/services#Service
	//
	// To filter on an attribute of a service-identifier type, apply the filter
	// name by using the snake case of the service-identifier type and the
	// attribute of that service-identifier type, and join the two with a period.
	// For example, to filter by the `meshUid` field of the `MeshIstio`
	// service-identifier type, you must filter on `mesh_istio.mesh_uid =
	// "123"` to match all services with mesh UID "123". Service-identifier types
	// and their attributes are described at
	// https://cloud.google.com/monitoring/api/ref_v3/rest/v3/services#Service
	Filter string `protobuf:"bytes,2,opt,name=filter,proto3" json:"filter,omitempty"`
	// A non-negative number that is the maximum number of results to return.
	// When 0, use default page size.
	PageSize int32 `protobuf:"varint,3,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// If this field is not empty then it must contain the `nextPageToken` value
	// returned by a previous call to this method.  Using this field causes the
	// method to return additional results from the previous method call.
	PageToken     string `protobuf:"bytes,4,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServicesRequest) Reset() {
	*x = ListServicesRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServicesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServicesRequest) ProtoMessage() {}

func (x *ListServicesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServicesRequest.ProtoReflect.Descriptor instead.
func (*ListServicesRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListServicesRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *ListServicesRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *ListServicesRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListServicesRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

// The `ListServices` response.
type ListServicesResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The `Service`s matching the specified filter.
	Services []*Service `protobuf:"bytes,1,rep,name=services,proto3" json:"services,omitempty"`
	// If there are more results than have been returned, then this field is set
	// to a non-empty value.  To see the additional results,
	// use that value as `page_token` in the next call to this method.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServicesResponse) Reset() {
	*x = ListServicesResponse{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServicesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServicesResponse) ProtoMessage() {}

func (x *ListServicesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServicesResponse.ProtoReflect.Descriptor instead.
func (*ListServicesResponse) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{3}
}

func (x *ListServicesResponse) GetServices() []*Service {
	if x != nil {
		return x.Services
	}
	return nil
}

func (x *ListServicesResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

// The `UpdateService` request.
type UpdateServiceRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The `Service` to draw updates from.
	// The given `name` specifies the resource to update.
	Service *Service `protobuf:"bytes,1,opt,name=service,proto3" json:"service,omitempty"`
	// A set of field paths defining which fields to use for the update.
	UpdateMask    *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateServiceRequest) Reset() {
	*x = UpdateServiceRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateServiceRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateServiceRequest) ProtoMessage() {}

func (x *UpdateServiceRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateServiceRequest.ProtoReflect.Descriptor instead.
func (*UpdateServiceRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{4}
}

func (x *UpdateServiceRequest) GetService() *Service {
	if x != nil {
		return x.Service
	}
	return nil
}

func (x *UpdateServiceRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

// The `DeleteService` request.
type DeleteServiceRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the `Service` to delete. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteServiceRequest) Reset() {
	*x = DeleteServiceRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteServiceRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteServiceRequest) ProtoMessage() {}

func (x *DeleteServiceRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteServiceRequest.ProtoReflect.Descriptor instead.
func (*DeleteServiceRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{5}
}

func (x *DeleteServiceRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// The `CreateServiceLevelObjective` request.
type CreateServiceLevelObjectiveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the parent `Service`. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Optional. The ServiceLevelObjective id to use for this
	// ServiceLevelObjective. If omitted, an id will be generated instead. Must
	// match the pattern `^[a-zA-Z0-9-_:.]+$`
	ServiceLevelObjectiveId string `protobuf:"bytes,3,opt,name=service_level_objective_id,json=serviceLevelObjectiveId,proto3" json:"service_level_objective_id,omitempty"`
	// Required. The `ServiceLevelObjective` to create.
	// The provided `name` will be respected if no `ServiceLevelObjective` exists
	// with this name.
	ServiceLevelObjective *ServiceLevelObjective `protobuf:"bytes,2,opt,name=service_level_objective,json=serviceLevelObjective,proto3" json:"service_level_objective,omitempty"`
	unknownFields         protoimpl.UnknownFields
	sizeCache             protoimpl.SizeCache
}

func (x *CreateServiceLevelObjectiveRequest) Reset() {
	*x = CreateServiceLevelObjectiveRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateServiceLevelObjectiveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateServiceLevelObjectiveRequest) ProtoMessage() {}

func (x *CreateServiceLevelObjectiveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateServiceLevelObjectiveRequest.ProtoReflect.Descriptor instead.
func (*CreateServiceLevelObjectiveRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{6}
}

func (x *CreateServiceLevelObjectiveRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateServiceLevelObjectiveRequest) GetServiceLevelObjectiveId() string {
	if x != nil {
		return x.ServiceLevelObjectiveId
	}
	return ""
}

func (x *CreateServiceLevelObjectiveRequest) GetServiceLevelObjective() *ServiceLevelObjective {
	if x != nil {
		return x.ServiceLevelObjective
	}
	return nil
}

// The `GetServiceLevelObjective` request.
type GetServiceLevelObjectiveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the `ServiceLevelObjective` to get. The format
	// is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]/serviceLevelObjectives/[SLO_NAME]
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// View of the `ServiceLevelObjective` to return. If `DEFAULT`, return the
	// `ServiceLevelObjective` as originally defined. If `EXPLICIT` and the
	// `ServiceLevelObjective` is defined in terms of a `BasicSli`, replace the
	// `BasicSli` with a `RequestBasedSli` spelling out how the SLI is computed.
	View          ServiceLevelObjective_View `protobuf:"varint,2,opt,name=view,proto3,enum=google.monitoring.v3.ServiceLevelObjective_View" json:"view,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetServiceLevelObjectiveRequest) Reset() {
	*x = GetServiceLevelObjectiveRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetServiceLevelObjectiveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetServiceLevelObjectiveRequest) ProtoMessage() {}

func (x *GetServiceLevelObjectiveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetServiceLevelObjectiveRequest.ProtoReflect.Descriptor instead.
func (*GetServiceLevelObjectiveRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{7}
}

func (x *GetServiceLevelObjectiveRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *GetServiceLevelObjectiveRequest) GetView() ServiceLevelObjective_View {
	if x != nil {
		return x.View
	}
	return ServiceLevelObjective_VIEW_UNSPECIFIED
}

// The `ListServiceLevelObjectives` request.
type ListServiceLevelObjectivesRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the parent containing the listed SLOs, either a
	// project or a Monitoring Metrics Scope. The formats are:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]
	//	workspaces/[HOST_PROJECT_ID_OR_NUMBER]/services/-
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// A filter specifying what `ServiceLevelObjective`s to return.
	Filter string `protobuf:"bytes,2,opt,name=filter,proto3" json:"filter,omitempty"`
	// A non-negative number that is the maximum number of results to return.
	// When 0, use default page size.
	PageSize int32 `protobuf:"varint,3,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// If this field is not empty then it must contain the `nextPageToken` value
	// returned by a previous call to this method.  Using this field causes the
	// method to return additional results from the previous method call.
	PageToken string `protobuf:"bytes,4,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// View of the `ServiceLevelObjective`s to return. If `DEFAULT`, return each
	// `ServiceLevelObjective` as originally defined. If `EXPLICIT` and the
	// `ServiceLevelObjective` is defined in terms of a `BasicSli`, replace the
	// `BasicSli` with a `RequestBasedSli` spelling out how the SLI is computed.
	View          ServiceLevelObjective_View `protobuf:"varint,5,opt,name=view,proto3,enum=google.monitoring.v3.ServiceLevelObjective_View" json:"view,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServiceLevelObjectivesRequest) Reset() {
	*x = ListServiceLevelObjectivesRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServiceLevelObjectivesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServiceLevelObjectivesRequest) ProtoMessage() {}

func (x *ListServiceLevelObjectivesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServiceLevelObjectivesRequest.ProtoReflect.Descriptor instead.
func (*ListServiceLevelObjectivesRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{8}
}

func (x *ListServiceLevelObjectivesRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *ListServiceLevelObjectivesRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *ListServiceLevelObjectivesRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListServiceLevelObjectivesRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

func (x *ListServiceLevelObjectivesRequest) GetView() ServiceLevelObjective_View {
	if x != nil {
		return x.View
	}
	return ServiceLevelObjective_VIEW_UNSPECIFIED
}

// The `ListServiceLevelObjectives` response.
type ListServiceLevelObjectivesResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The `ServiceLevelObjective`s matching the specified filter.
	ServiceLevelObjectives []*ServiceLevelObjective `protobuf:"bytes,1,rep,name=service_level_objectives,json=serviceLevelObjectives,proto3" json:"service_level_objectives,omitempty"`
	// If there are more results than have been returned, then this field is set
	// to a non-empty value.  To see the additional results,
	// use that value as `page_token` in the next call to this method.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListServiceLevelObjectivesResponse) Reset() {
	*x = ListServiceLevelObjectivesResponse{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListServiceLevelObjectivesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListServiceLevelObjectivesResponse) ProtoMessage() {}

func (x *ListServiceLevelObjectivesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListServiceLevelObjectivesResponse.ProtoReflect.Descriptor instead.
func (*ListServiceLevelObjectivesResponse) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{9}
}

func (x *ListServiceLevelObjectivesResponse) GetServiceLevelObjectives() []*ServiceLevelObjective {
	if x != nil {
		return x.ServiceLevelObjectives
	}
	return nil
}

func (x *ListServiceLevelObjectivesResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

// The `UpdateServiceLevelObjective` request.
type UpdateServiceLevelObjectiveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The `ServiceLevelObjective` to draw updates from.
	// The given `name` specifies the resource to update.
	ServiceLevelObjective *ServiceLevelObjective `protobuf:"bytes,1,opt,name=service_level_objective,json=serviceLevelObjective,proto3" json:"service_level_objective,omitempty"`
	// A set of field paths defining which fields to use for the update.
	UpdateMask    *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateServiceLevelObjectiveRequest) Reset() {
	*x = UpdateServiceLevelObjectiveRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateServiceLevelObjectiveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateServiceLevelObjectiveRequest) ProtoMessage() {}

func (x *UpdateServiceLevelObjectiveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateServiceLevelObjectiveRequest.ProtoReflect.Descriptor instead.
func (*UpdateServiceLevelObjectiveRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{10}
}

func (x *UpdateServiceLevelObjectiveRequest) GetServiceLevelObjective() *ServiceLevelObjective {
	if x != nil {
		return x.ServiceLevelObjective
	}
	return nil
}

func (x *UpdateServiceLevelObjectiveRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

// The `DeleteServiceLevelObjective` request.
type DeleteServiceLevelObjectiveRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Resource name of the `ServiceLevelObjective` to delete. The
	// format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]/serviceLevelObjectives/[SLO_NAME]
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteServiceLevelObjectiveRequest) Reset() {
	*x = DeleteServiceLevelObjectiveRequest{}
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteServiceLevelObjectiveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteServiceLevelObjectiveRequest) ProtoMessage() {}

func (x *DeleteServiceLevelObjectiveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_service_service_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteServiceLevelObjectiveRequest.ProtoReflect.Descriptor instead.
func (*DeleteServiceLevelObjectiveRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_service_service_proto_rawDescGZIP(), []int{11}
}

func (x *DeleteServiceLevelObjectiveRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

var File_google_monitoring_v3_service_service_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_service_service_proto_rawDesc = "" +
	"\n" +
	"*google/monitoring/v3/service_service.proto\x12\x14google.monitoring.v3\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\"google/monitoring/v3/service.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a google/protobuf/field_mask.proto\"\xb6\x01\n" +
	"\x14CreateServiceRequest\x12A\n" +
	"\x06parent\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\x12!monitoring.googleapis.com/ServiceR\x06parent\x12\x1d\n" +
	"\n" +
	"service_id\x18\x03 \x01(\tR\tserviceId\x12<\n" +
	"\aservice\x18\x02 \x01(\v2\x1d.google.monitoring.v3.ServiceB\x03\xe0A\x02R\aservice\"R\n" +
	"\x11GetServiceRequest\x12=\n" +
	"\x04name\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!monitoring.googleapis.com/ServiceR\x04name\"\xac\x01\n" +
	"\x13ListServicesRequest\x12A\n" +
	"\x06parent\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\x12!monitoring.googleapis.com/ServiceR\x06parent\x12\x16\n" +
	"\x06filter\x18\x02 \x01(\tR\x06filter\x12\x1b\n" +
	"\tpage_size\x18\x03 \x01(\x05R\bpageSize\x12\x1d\n" +
	"\n" +
	"page_token\x18\x04 \x01(\tR\tpageToken\"y\n" +
	"\x14ListServicesResponse\x129\n" +
	"\bservices\x18\x01 \x03(\v2\x1d.google.monitoring.v3.ServiceR\bservices\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\x91\x01\n" +
	"\x14UpdateServiceRequest\x12<\n" +
	"\aservice\x18\x01 \x01(\v2\x1d.google.monitoring.v3.ServiceB\x03\xe0A\x02R\aservice\x12;\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskR\n" +
	"updateMask\"U\n" +
	"\x14DeleteServiceRequest\x12=\n" +
	"\x04name\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!monitoring.googleapis.com/ServiceR\x04name\"\x8e\x02\n" +
	"\"CreateServiceLevelObjectiveRequest\x12A\n" +
	"\x06parent\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!monitoring.googleapis.com/ServiceR\x06parent\x12;\n" +
	"\x1aservice_level_objective_id\x18\x03 \x01(\tR\x17serviceLevelObjectiveId\x12h\n" +
	"\x17service_level_objective\x18\x02 \x01(\v2+.google.monitoring.v3.ServiceLevelObjectiveB\x03\xe0A\x02R\x15serviceLevelObjective\"\xb4\x01\n" +
	"\x1fGetServiceLevelObjectiveRequest\x12K\n" +
	"\x04name\x18\x01 \x01(\tB7\xe0A\x02\xfaA1\n" +
	"/monitoring.googleapis.com/ServiceLevelObjectiveR\x04name\x12D\n" +
	"\x04view\x18\x02 \x01(\x0e20.google.monitoring.v3.ServiceLevelObjective.ViewR\x04view\"\x80\x02\n" +
	"!ListServiceLevelObjectivesRequest\x12A\n" +
	"\x06parent\x18\x01 \x01(\tB)\xe0A\x02\xfaA#\n" +
	"!monitoring.googleapis.com/ServiceR\x06parent\x12\x16\n" +
	"\x06filter\x18\x02 \x01(\tR\x06filter\x12\x1b\n" +
	"\tpage_size\x18\x03 \x01(\x05R\bpageSize\x12\x1d\n" +
	"\n" +
	"page_token\x18\x04 \x01(\tR\tpageToken\x12D\n" +
	"\x04view\x18\x05 \x01(\x0e20.google.monitoring.v3.ServiceLevelObjective.ViewR\x04view\"\xb3\x01\n" +
	"\"ListServiceLevelObjectivesResponse\x12e\n" +
	"\x18service_level_objectives\x18\x01 \x03(\v2+.google.monitoring.v3.ServiceLevelObjectiveR\x16serviceLevelObjectives\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xcb\x01\n" +
	"\"UpdateServiceLevelObjectiveRequest\x12h\n" +
	"\x17service_level_objective\x18\x01 \x01(\v2+.google.monitoring.v3.ServiceLevelObjectiveB\x03\xe0A\x02R\x15serviceLevelObjective\x12;\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskR\n" +
	"updateMask\"q\n" +
	"\"DeleteServiceLevelObjectiveRequest\x12K\n" +
	"\x04name\x18\x01 \x01(\tB7\xe0A\x02\xfaA1\n" +
	"/monitoring.googleapis.com/ServiceLevelObjectiveR\x04name2\xea\x0f\n" +
	"\x18ServiceMonitoringService\x12\x97\x01\n" +
	"\rCreateService\x12*.google.monitoring.v3.CreateServiceRequest\x1a\x1d.google.monitoring.v3.Service\";\xdaA\x0eparent,service\x82\xd3\xe4\x93\x02$:\aservice\"\x19/v3/{parent=*/*}/services\x12~\n" +
	"\n" +
	"GetService\x12'.google.monitoring.v3.GetServiceRequest\x1a\x1d.google.monitoring.v3.Service\"(\xdaA\x04name\x82\xd3\xe4\x93\x02\x1b\x12\x19/v3/{name=*/*/services/*}\x12\x91\x01\n" +
	"\fListServices\x12).google.monitoring.v3.ListServicesRequest\x1a*.google.monitoring.v3.ListServicesResponse\"*\xdaA\x06parent\x82\xd3\xe4\x93\x02\x1b\x12\x19/v3/{parent=*/*}/services\x12\x98\x01\n" +
	"\rUpdateService\x12*.google.monitoring.v3.UpdateServiceRequest\x1a\x1d.google.monitoring.v3.Service\"<\xdaA\aservice\x82\xd3\xe4\x93\x02,:\aservice2!/v3/{service.name=*/*/services/*}\x12}\n" +
	"\rDeleteService\x12*.google.monitoring.v3.DeleteServiceRequest\x1a\x16.google.protobuf.Empty\"(\xdaA\x04name\x82\xd3\xe4\x93\x02\x1b*\x19/v3/{name=*/*/services/*}\x12\xfa\x01\n" +
	"\x1bCreateServiceLevelObjective\x128.google.monitoring.v3.CreateServiceLevelObjectiveRequest\x1a+.google.monitoring.v3.ServiceLevelObjective\"t\xdaA\x1eparent,service_level_objective\x82\xd3\xe4\x93\x02M:\x17service_level_objective\"2/v3/{parent=*/*/services/*}/serviceLevelObjectives\x12\xc1\x01\n" +
	"\x18GetServiceLevelObjective\x125.google.monitoring.v3.GetServiceLevelObjectiveRequest\x1a+.google.monitoring.v3.ServiceLevelObjective\"A\xdaA\x04name\x82\xd3\xe4\x93\x024\x122/v3/{name=*/*/services/*/serviceLevelObjectives/*}\x12\xd4\x01\n" +
	"\x1aListServiceLevelObjectives\x127.google.monitoring.v3.ListServiceLevelObjectivesRequest\x1a8.google.monitoring.v3.ListServiceLevelObjectivesResponse\"C\xdaA\x06parent\x82\xd3\xe4\x93\x024\x122/v3/{parent=*/*/services/*}/serviceLevelObjectives\x12\x8c\x02\n" +
	"\x1bUpdateServiceLevelObjective\x128.google.monitoring.v3.UpdateServiceLevelObjectiveRequest\x1a+.google.monitoring.v3.ServiceLevelObjective\"\x85\x01\xdaA\x17service_level_objective\x82\xd3\xe4\x93\x02e:\x17service_level_objective2J/v3/{service_level_objective.name=*/*/services/*/serviceLevelObjectives/*}\x12\xb2\x01\n" +
	"\x1bDeleteServiceLevelObjective\x128.google.monitoring.v3.DeleteServiceLevelObjectiveRequest\x1a\x16.google.protobuf.Empty\"A\xdaA\x04name\x82\xd3\xe4\x93\x024*2/v3/{name=*/*/services/*/serviceLevelObjectives/*}\x1a\xa9\x01\xcaA\x19monitoring.googleapis.com\xd2A\x89\x01https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/monitoring,https://www.googleapis.com/auth/monitoring.readB\xd8\x01\n" +
	"\x18com.google.monitoring.v3B\x1dServiceMonitoringServiceProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_service_service_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_service_service_proto_rawDescData []byte
)

func file_google_monitoring_v3_service_service_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_service_service_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_service_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_service_service_proto_rawDesc), len(file_google_monitoring_v3_service_service_proto_rawDesc)))
	})
	return file_google_monitoring_v3_service_service_proto_rawDescData
}

var file_google_monitoring_v3_service_service_proto_msgTypes = make([]protoimpl.MessageInfo, 12)
var file_google_monitoring_v3_service_service_proto_goTypes = []any{
	(*CreateServiceRequest)(nil),               // 0: google.monitoring.v3.CreateServiceRequest
	(*GetServiceRequest)(nil),                  // 1: google.monitoring.v3.GetServiceRequest
	(*ListServicesRequest)(nil),                // 2: google.monitoring.v3.ListServicesRequest
	(*ListServicesResponse)(nil),               // 3: google.monitoring.v3.ListServicesResponse
	(*UpdateServiceRequest)(nil),               // 4: google.monitoring.v3.UpdateServiceRequest
	(*DeleteServiceRequest)(nil),               // 5: google.monitoring.v3.DeleteServiceRequest
	(*CreateServiceLevelObjectiveRequest)(nil), // 6: google.monitoring.v3.CreateServiceLevelObjectiveRequest
	(*GetServiceLevelObjectiveRequest)(nil),    // 7: google.monitoring.v3.GetServiceLevelObjectiveRequest
	(*ListServiceLevelObjectivesRequest)(nil),  // 8: google.monitoring.v3.ListServiceLevelObjectivesRequest
	(*ListServiceLevelObjectivesResponse)(nil), // 9: google.monitoring.v3.ListServiceLevelObjectivesResponse
	(*UpdateServiceLevelObjectiveRequest)(nil), // 10: google.monitoring.v3.UpdateServiceLevelObjectiveRequest
	(*DeleteServiceLevelObjectiveRequest)(nil), // 11: google.monitoring.v3.DeleteServiceLevelObjectiveRequest
	(*Service)(nil),                            // 12: google.monitoring.v3.Service
	(*fieldmaskpb.FieldMask)(nil),              // 13: google.protobuf.FieldMask
	(*ServiceLevelObjective)(nil),              // 14: google.monitoring.v3.ServiceLevelObjective
	(ServiceLevelObjective_View)(0),            // 15: google.monitoring.v3.ServiceLevelObjective.View
	(*emptypb.Empty)(nil),                      // 16: google.protobuf.Empty
}
var file_google_monitoring_v3_service_service_proto_depIdxs = []int32{
	12, // 0: google.monitoring.v3.CreateServiceRequest.service:type_name -> google.monitoring.v3.Service
	12, // 1: google.monitoring.v3.ListServicesResponse.services:type_name -> google.monitoring.v3.Service
	12, // 2: google.monitoring.v3.UpdateServiceRequest.service:type_name -> google.monitoring.v3.Service
	13, // 3: google.monitoring.v3.UpdateServiceRequest.update_mask:type_name -> google.protobuf.FieldMask
	14, // 4: google.monitoring.v3.CreateServiceLevelObjectiveRequest.service_level_objective:type_name -> google.monitoring.v3.ServiceLevelObjective
	15, // 5: google.monitoring.v3.GetServiceLevelObjectiveRequest.view:type_name -> google.monitoring.v3.ServiceLevelObjective.View
	15, // 6: google.monitoring.v3.ListServiceLevelObjectivesRequest.view:type_name -> google.monitoring.v3.ServiceLevelObjective.View
	14, // 7: google.monitoring.v3.ListServiceLevelObjectivesResponse.service_level_objectives:type_name -> google.monitoring.v3.ServiceLevelObjective
	14, // 8: google.monitoring.v3.UpdateServiceLevelObjectiveRequest.service_level_objective:type_name -> google.monitoring.v3.ServiceLevelObjective
	13, // 9: google.monitoring.v3.UpdateServiceLevelObjectiveRequest.update_mask:type_name -> google.protobuf.FieldMask
	0,  // 10: google.monitoring.v3.ServiceMonitoringService.CreateService:input_type -> google.monitoring.v3.CreateServiceRequest
	1,  // 11: google.monitoring.v3.ServiceMonitoringService.GetService:input_type -> google.monitoring.v3.GetServiceRequest
	2,  // 12: google.monitoring.v3.ServiceMonitoringService.ListServices:input_type -> google.monitoring.v3.ListServicesRequest
	4,  // 13: google.monitoring.v3.ServiceMonitoringService.UpdateService:input_type -> google.monitoring.v3.UpdateServiceRequest
	5,  // 14: google.monitoring.v3.ServiceMonitoringService.DeleteService:input_type -> google.monitoring.v3.DeleteServiceRequest
	6,  // 15: google.monitoring.v3.ServiceMonitoringService.CreateServiceLevelObjective:input_type -> google.monitoring.v3.CreateServiceLevelObjectiveRequest
	7,  // 16: google.monitoring.v3.ServiceMonitoringService.GetServiceLevelObjective:input_type -> google.monitoring.v3.GetServiceLevelObjectiveRequest
	8,  // 17: google.monitoring.v3.ServiceMonitoringService.ListServiceLevelObjectives:input_type -> google.monitoring.v3.ListServiceLevelObjectivesRequest
	10, // 18: google.monitoring.v3.ServiceMonitoringService.UpdateServiceLevelObjective:input_type -> google.monitoring.v3.UpdateServiceLevelObjectiveRequest
	11, // 19: google.monitoring.v3.ServiceMonitoringService.DeleteServiceLevelObjective:input_type -> google.monitoring.v3.DeleteServiceLevelObjectiveRequest
	12, // 20: google.monitoring.v3.ServiceMonitoringService.CreateService:output_type -> google.monitoring.v3.Service
	12, // 21: google.monitoring.v3.ServiceMonitoringService.GetService:output_type -> google.monitoring.v3.Service
	3,  // 22: google.monitoring.v3.ServiceMonitoringService.ListServices:output_type -> google.monitoring.v3.ListServicesResponse
	12, // 23: google.monitoring.v3.ServiceMonitoringService.UpdateService:output_type -> google.monitoring.v3.Service
	16, // 24: google.monitoring.v3.ServiceMonitoringService.DeleteService:output_type -> google.protobuf.Empty
	14, // 25: google.monitoring.v3.ServiceMonitoringService.CreateServiceLevelObjective:output_type -> google.monitoring.v3.ServiceLevelObjective
	14, // 26: google.monitoring.v3.ServiceMonitoringService.GetServiceLevelObjective:output_type -> google.monitoring.v3.ServiceLevelObjective
	9,  // 27: google.monitoring.v3.ServiceMonitoringService.ListServiceLevelObjectives:output_type -> google.monitoring.v3.ListServiceLevelObjectivesResponse
	14, // 28: google.monitoring.v3.ServiceMonitoringService.UpdateServiceLevelObjective:output_type -> google.monitoring.v3.ServiceLevelObjective
	16, // 29: google.monitoring.v3.ServiceMonitoringService.DeleteServiceLevelObjective:output_type -> google.protobuf.Empty
	20, // [20:30] is the sub-list for method output_type
	10, // [10:20] is the sub-list for method input_type
	10, // [10:10] is the sub-list for extension type_name
	10, // [10:10] is the sub-list for extension extendee
	0,  // [0:10] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_service_service_proto_init() }
func file_google_monitoring_v3_service_service_proto_init() {
	if File_google_monitoring_v3_service_service_proto != nil {
		return
	}
	file_google_monitoring_v3_service_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_service_service_proto_rawDesc), len(file_google_monitoring_v3_service_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   12,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_monitoring_v3_service_service_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_service_service_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_service_service_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_service_service_proto = out.File
	file_google_monitoring_v3_service_service_proto_goTypes = nil
	file_google_monitoring_v3_service_service_proto_depIdxs = nil
}
