// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/matcher/v3/value.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Specifies the way to match a Protobuf::Value. Primitive values and ListValue are supported.
// StructValue is not supported and is always not matched.
// [#next-free-field: 8]
type ValueMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies how to match a value.
	//
	// Types that are valid to be assigned to MatchPattern:
	//
	//	*ValueMatcher_NullMatch_
	//	*ValueMatcher_DoubleMatch
	//	*ValueMatcher_StringMatch
	//	*ValueMatcher_BoolMatch
	//	*ValueMatcher_PresentMatch
	//	*ValueMatcher_ListMatch
	//	*ValueMatcher_OrMatch
	MatchPattern  isValueMatcher_MatchPattern `protobuf_oneof:"match_pattern"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ValueMatcher) Reset() {
	*x = ValueMatcher{}
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ValueMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ValueMatcher) ProtoMessage() {}

func (x *ValueMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ValueMatcher.ProtoReflect.Descriptor instead.
func (*ValueMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_value_proto_rawDescGZIP(), []int{0}
}

func (x *ValueMatcher) GetMatchPattern() isValueMatcher_MatchPattern {
	if x != nil {
		return x.MatchPattern
	}
	return nil
}

func (x *ValueMatcher) GetNullMatch() *ValueMatcher_NullMatch {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_NullMatch_); ok {
			return x.NullMatch
		}
	}
	return nil
}

func (x *ValueMatcher) GetDoubleMatch() *DoubleMatcher {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_DoubleMatch); ok {
			return x.DoubleMatch
		}
	}
	return nil
}

func (x *ValueMatcher) GetStringMatch() *StringMatcher {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_StringMatch); ok {
			return x.StringMatch
		}
	}
	return nil
}

func (x *ValueMatcher) GetBoolMatch() bool {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_BoolMatch); ok {
			return x.BoolMatch
		}
	}
	return false
}

func (x *ValueMatcher) GetPresentMatch() bool {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_PresentMatch); ok {
			return x.PresentMatch
		}
	}
	return false
}

func (x *ValueMatcher) GetListMatch() *ListMatcher {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_ListMatch); ok {
			return x.ListMatch
		}
	}
	return nil
}

func (x *ValueMatcher) GetOrMatch() *OrMatcher {
	if x != nil {
		if x, ok := x.MatchPattern.(*ValueMatcher_OrMatch); ok {
			return x.OrMatch
		}
	}
	return nil
}

type isValueMatcher_MatchPattern interface {
	isValueMatcher_MatchPattern()
}

type ValueMatcher_NullMatch_ struct {
	// If specified, a match occurs if and only if the target value is a NullValue.
	NullMatch *ValueMatcher_NullMatch `protobuf:"bytes,1,opt,name=null_match,json=nullMatch,proto3,oneof"`
}

type ValueMatcher_DoubleMatch struct {
	// If specified, a match occurs if and only if the target value is a double value and is
	// matched to this field.
	DoubleMatch *DoubleMatcher `protobuf:"bytes,2,opt,name=double_match,json=doubleMatch,proto3,oneof"`
}

type ValueMatcher_StringMatch struct {
	// If specified, a match occurs if and only if the target value is a string value and is
	// matched to this field.
	StringMatch *StringMatcher `protobuf:"bytes,3,opt,name=string_match,json=stringMatch,proto3,oneof"`
}

type ValueMatcher_BoolMatch struct {
	// If specified, a match occurs if and only if the target value is a bool value and is equal
	// to this field.
	BoolMatch bool `protobuf:"varint,4,opt,name=bool_match,json=boolMatch,proto3,oneof"`
}

type ValueMatcher_PresentMatch struct {
	// If specified, value match will be performed based on whether the path is referring to a
	// valid primitive value in the metadata. If the path is referring to a non-primitive value,
	// the result is always not matched.
	PresentMatch bool `protobuf:"varint,5,opt,name=present_match,json=presentMatch,proto3,oneof"`
}

type ValueMatcher_ListMatch struct {
	// If specified, a match occurs if and only if the target value is a list value and
	// is matched to this field.
	ListMatch *ListMatcher `protobuf:"bytes,6,opt,name=list_match,json=listMatch,proto3,oneof"`
}

type ValueMatcher_OrMatch struct {
	// If specified, a match occurs if and only if any of the alternatives in the match accept the value.
	OrMatch *OrMatcher `protobuf:"bytes,7,opt,name=or_match,json=orMatch,proto3,oneof"`
}

func (*ValueMatcher_NullMatch_) isValueMatcher_MatchPattern() {}

func (*ValueMatcher_DoubleMatch) isValueMatcher_MatchPattern() {}

func (*ValueMatcher_StringMatch) isValueMatcher_MatchPattern() {}

func (*ValueMatcher_BoolMatch) isValueMatcher_MatchPattern() {}

func (*ValueMatcher_PresentMatch) isValueMatcher_MatchPattern() {}

func (*ValueMatcher_ListMatch) isValueMatcher_MatchPattern() {}

func (*ValueMatcher_OrMatch) isValueMatcher_MatchPattern() {}

// Specifies the way to match a list value.
type ListMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatchPattern:
	//
	//	*ListMatcher_OneOf
	MatchPattern  isListMatcher_MatchPattern `protobuf_oneof:"match_pattern"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListMatcher) Reset() {
	*x = ListMatcher{}
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListMatcher) ProtoMessage() {}

func (x *ListMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListMatcher.ProtoReflect.Descriptor instead.
func (*ListMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_value_proto_rawDescGZIP(), []int{1}
}

func (x *ListMatcher) GetMatchPattern() isListMatcher_MatchPattern {
	if x != nil {
		return x.MatchPattern
	}
	return nil
}

func (x *ListMatcher) GetOneOf() *ValueMatcher {
	if x != nil {
		if x, ok := x.MatchPattern.(*ListMatcher_OneOf); ok {
			return x.OneOf
		}
	}
	return nil
}

type isListMatcher_MatchPattern interface {
	isListMatcher_MatchPattern()
}

type ListMatcher_OneOf struct {
	// If specified, at least one of the values in the list must match the value specified.
	OneOf *ValueMatcher `protobuf:"bytes,1,opt,name=one_of,json=oneOf,proto3,oneof"`
}

func (*ListMatcher_OneOf) isListMatcher_MatchPattern() {}

// Specifies a list of alternatives for the match.
type OrMatcher struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ValueMatchers []*ValueMatcher        `protobuf:"bytes,1,rep,name=value_matchers,json=valueMatchers,proto3" json:"value_matchers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OrMatcher) Reset() {
	*x = OrMatcher{}
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OrMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OrMatcher) ProtoMessage() {}

func (x *OrMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OrMatcher.ProtoReflect.Descriptor instead.
func (*OrMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_value_proto_rawDescGZIP(), []int{2}
}

func (x *OrMatcher) GetValueMatchers() []*ValueMatcher {
	if x != nil {
		return x.ValueMatchers
	}
	return nil
}

// NullMatch is an empty message to specify a null value.
type ValueMatcher_NullMatch struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ValueMatcher_NullMatch) Reset() {
	*x = ValueMatcher_NullMatch{}
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ValueMatcher_NullMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ValueMatcher_NullMatch) ProtoMessage() {}

func (x *ValueMatcher_NullMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_value_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ValueMatcher_NullMatch.ProtoReflect.Descriptor instead.
func (*ValueMatcher_NullMatch) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_value_proto_rawDescGZIP(), []int{0, 0}
}

var File_envoy_type_matcher_v3_value_proto protoreflect.FileDescriptor

const file_envoy_type_matcher_v3_value_proto_rawDesc = "" +
	"\n" +
	"!envoy/type/matcher/v3/value.proto\x12\x15envoy.type.matcher.v3\x1a\"envoy/type/matcher/v3/number.proto\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xbd\x04\n" +
	"\fValueMatcher\x12N\n" +
	"\n" +
	"null_match\x18\x01 \x01(\v2-.envoy.type.matcher.v3.ValueMatcher.NullMatchH\x00R\tnullMatch\x12I\n" +
	"\fdouble_match\x18\x02 \x01(\v2$.envoy.type.matcher.v3.DoubleMatcherH\x00R\vdoubleMatch\x12I\n" +
	"\fstring_match\x18\x03 \x01(\v2$.envoy.type.matcher.v3.StringMatcherH\x00R\vstringMatch\x12\x1f\n" +
	"\n" +
	"bool_match\x18\x04 \x01(\bH\x00R\tboolMatch\x12%\n" +
	"\rpresent_match\x18\x05 \x01(\bH\x00R\fpresentMatch\x12C\n" +
	"\n" +
	"list_match\x18\x06 \x01(\v2\".envoy.type.matcher.v3.ListMatcherH\x00R\tlistMatch\x12=\n" +
	"\bor_match\x18\a \x01(\v2 .envoy.type.matcher.v3.OrMatcherH\x00R\aorMatch\x1a=\n" +
	"\tNullMatch:0\x9aň\x1e+\n" +
	")envoy.type.matcher.ValueMatcher.NullMatch:&\x9aň\x1e!\n" +
	"\x1fenvoy.type.matcher.ValueMatcherB\x14\n" +
	"\rmatch_pattern\x12\x03\xf8B\x01\"\x88\x01\n" +
	"\vListMatcher\x12<\n" +
	"\x06one_of\x18\x01 \x01(\v2#.envoy.type.matcher.v3.ValueMatcherH\x00R\x05oneOf:%\x9aň\x1e \n" +
	"\x1eenvoy.type.matcher.ListMatcherB\x14\n" +
	"\rmatch_pattern\x12\x03\xf8B\x01\"a\n" +
	"\tOrMatcher\x12T\n" +
	"\x0evalue_matchers\x18\x01 \x03(\v2#.envoy.type.matcher.v3.ValueMatcherB\b\xfaB\x05\x92\x01\x02\b\x02R\rvalueMatchersB\x83\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.type.matcher.v3B\n" +
	"ValueProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_type_matcher_v3_value_proto_rawDescOnce sync.Once
	file_envoy_type_matcher_v3_value_proto_rawDescData []byte
)

func file_envoy_type_matcher_v3_value_proto_rawDescGZIP() []byte {
	file_envoy_type_matcher_v3_value_proto_rawDescOnce.Do(func() {
		file_envoy_type_matcher_v3_value_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_value_proto_rawDesc), len(file_envoy_type_matcher_v3_value_proto_rawDesc)))
	})
	return file_envoy_type_matcher_v3_value_proto_rawDescData
}

var file_envoy_type_matcher_v3_value_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_type_matcher_v3_value_proto_goTypes = []any{
	(*ValueMatcher)(nil),           // 0: envoy.type.matcher.v3.ValueMatcher
	(*ListMatcher)(nil),            // 1: envoy.type.matcher.v3.ListMatcher
	(*OrMatcher)(nil),              // 2: envoy.type.matcher.v3.OrMatcher
	(*ValueMatcher_NullMatch)(nil), // 3: envoy.type.matcher.v3.ValueMatcher.NullMatch
	(*DoubleMatcher)(nil),          // 4: envoy.type.matcher.v3.DoubleMatcher
	(*StringMatcher)(nil),          // 5: envoy.type.matcher.v3.StringMatcher
}
var file_envoy_type_matcher_v3_value_proto_depIdxs = []int32{
	3, // 0: envoy.type.matcher.v3.ValueMatcher.null_match:type_name -> envoy.type.matcher.v3.ValueMatcher.NullMatch
	4, // 1: envoy.type.matcher.v3.ValueMatcher.double_match:type_name -> envoy.type.matcher.v3.DoubleMatcher
	5, // 2: envoy.type.matcher.v3.ValueMatcher.string_match:type_name -> envoy.type.matcher.v3.StringMatcher
	1, // 3: envoy.type.matcher.v3.ValueMatcher.list_match:type_name -> envoy.type.matcher.v3.ListMatcher
	2, // 4: envoy.type.matcher.v3.ValueMatcher.or_match:type_name -> envoy.type.matcher.v3.OrMatcher
	0, // 5: envoy.type.matcher.v3.ListMatcher.one_of:type_name -> envoy.type.matcher.v3.ValueMatcher
	0, // 6: envoy.type.matcher.v3.OrMatcher.value_matchers:type_name -> envoy.type.matcher.v3.ValueMatcher
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_envoy_type_matcher_v3_value_proto_init() }
func file_envoy_type_matcher_v3_value_proto_init() {
	if File_envoy_type_matcher_v3_value_proto != nil {
		return
	}
	file_envoy_type_matcher_v3_number_proto_init()
	file_envoy_type_matcher_v3_string_proto_init()
	file_envoy_type_matcher_v3_value_proto_msgTypes[0].OneofWrappers = []any{
		(*ValueMatcher_NullMatch_)(nil),
		(*ValueMatcher_DoubleMatch)(nil),
		(*ValueMatcher_StringMatch)(nil),
		(*ValueMatcher_BoolMatch)(nil),
		(*ValueMatcher_PresentMatch)(nil),
		(*ValueMatcher_ListMatch)(nil),
		(*ValueMatcher_OrMatch)(nil),
	}
	file_envoy_type_matcher_v3_value_proto_msgTypes[1].OneofWrappers = []any{
		(*ListMatcher_OneOf)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_value_proto_rawDesc), len(file_envoy_type_matcher_v3_value_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_matcher_v3_value_proto_goTypes,
		DependencyIndexes: file_envoy_type_matcher_v3_value_proto_depIdxs,
		MessageInfos:      file_envoy_type_matcher_v3_value_proto_msgTypes,
	}.Build()
	File_envoy_type_matcher_v3_value_proto = out.File
	file_envoy_type_matcher_v3_value_proto_goTypes = nil
	file_envoy_type_matcher_v3_value_proto_depIdxs = nil
}
