// Code generated by smithy-go-codegen DO NOT EDIT.

package types

type OAuth2ErrorCode string

// Enum values for OAuth2ErrorCode
const (
	// Token has expired and needs to be refreshed
	OAuth2ErrorCodeTokenExpired OAuth2ErrorCode = "TOKEN_EXPIRED"
	// User credentials have been changed
	OAuth2ErrorCodeUserCredentialsChanged OAuth2ErrorCode = "USER_CREDENTIALS_CHANGED"
	// Insufficient permissions to perform this operation
	OAuth2ErrorCodeInsufficientPermissions OAuth2ErrorCode = "INSUFFICIENT_PERMISSIONS"
	// Authorization code has expired
	OAuth2ErrorCodeAuthcodeExpired OAuth2ErrorCode = "AUTHCODE_EXPIRED"
	// Internal server error occurred
	OAuth2ErrorCodeServerError OAuth2ErrorCode = "server_error"
	// The request is missing a required parameter, includes an invalid parameter
	// value, or is otherwise malformed
	OAuth2ErrorCodeInvalidRequest OAuth2ErrorCode = "INVALID_REQUEST"
	// Requested resource was not found
	OAuth2ErrorCodeResourceNotFound OAuth2ErrorCode = "RESOURCE_NOT_FOUND"
	// Request conflicts with current state of the resource
	OAuth2ErrorCodeConflict OAuth2ErrorCode = "CONFLICT"
	// Request would cause a service quota to be exceeded
	OAuth2ErrorCodeServiceQuotaExceeded OAuth2ErrorCode = "SERVICE_QUOTA_EXCEEDED"
)

// Values returns all known values for OAuth2ErrorCode. Note that this can be
// expanded in the future, and so it is only as up to date as the client.
//
// The ordering of this slice is not guaranteed to be stable across updates.
func (OAuth2ErrorCode) Values() []OAuth2ErrorCode {
	return []OAuth2ErrorCode{
		"TOKEN_EXPIRED",
		"USER_CREDENTIALS_CHANGED",
		"INSUFFICIENT_PERMISSIONS",
		"AUTHCODE_EXPIRED",
		"server_error",
		"INVALID_REQUEST",
		"RESOURCE_NOT_FOUND",
		"CONFLICT",
		"SERVICE_QUOTA_EXCEEDED",
	}
}
