// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.2
// source: google/monitoring/v3/alert_service.proto

package monitoringpb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The protocol for the `CreateAlertPolicy` request.
type CreateAlertPolicyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The
	// [project](https://cloud.google.com/monitoring/api/v3#project_name) in which
	// to create the alerting policy. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	//
	// Note that this field names the parent container in which the alerting
	// policy will be written, not the name of the created policy. |name| must be
	// a host project of a Metrics Scope, otherwise INVALID_ARGUMENT error will
	// return. The alerting policy that is returned will have a name that contains
	// a normalized representation of this name as a prefix but adds a suffix of
	// the form `/alertPolicies/[ALERT_POLICY_ID]`, identifying the policy in the
	// container.
	Name string `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	// Required. The requested alerting policy. You should omit the `name` field
	// in this policy. The name will be returned in the new policy, including a
	// new `[ALERT_POLICY_ID]` value.
	AlertPolicy   *AlertPolicy `protobuf:"bytes,2,opt,name=alert_policy,json=alertPolicy,proto3" json:"alert_policy,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateAlertPolicyRequest) Reset() {
	*x = CreateAlertPolicyRequest{}
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateAlertPolicyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateAlertPolicyRequest) ProtoMessage() {}

func (x *CreateAlertPolicyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateAlertPolicyRequest.ProtoReflect.Descriptor instead.
func (*CreateAlertPolicyRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_alert_service_proto_rawDescGZIP(), []int{0}
}

func (x *CreateAlertPolicyRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *CreateAlertPolicyRequest) GetAlertPolicy() *AlertPolicy {
	if x != nil {
		return x.AlertPolicy
	}
	return nil
}

// The protocol for the `GetAlertPolicy` request.
type GetAlertPolicyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The alerting policy to retrieve. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/alertPolicies/[ALERT_POLICY_ID]
	Name          string `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetAlertPolicyRequest) Reset() {
	*x = GetAlertPolicyRequest{}
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetAlertPolicyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetAlertPolicyRequest) ProtoMessage() {}

func (x *GetAlertPolicyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetAlertPolicyRequest.ProtoReflect.Descriptor instead.
func (*GetAlertPolicyRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_alert_service_proto_rawDescGZIP(), []int{1}
}

func (x *GetAlertPolicyRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// The protocol for the `ListAlertPolicies` request.
type ListAlertPoliciesRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The
	// [project](https://cloud.google.com/monitoring/api/v3#project_name) whose
	// alert policies are to be listed. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]
	//
	// Note that this field names the parent container in which the alerting
	// policies to be listed are stored. To retrieve a single alerting policy
	// by name, use the
	// [GetAlertPolicy][google.monitoring.v3.AlertPolicyService.GetAlertPolicy]
	// operation, instead.
	Name string `protobuf:"bytes,4,opt,name=name,proto3" json:"name,omitempty"`
	// Optional. If provided, this field specifies the criteria that must be met
	// by alert policies to be included in the response.
	//
	// For more details, see [sorting and
	// filtering](https://cloud.google.com/monitoring/api/v3/sorting-and-filtering).
	Filter string `protobuf:"bytes,5,opt,name=filter,proto3" json:"filter,omitempty"`
	// Optional. A comma-separated list of fields by which to sort the result.
	// Supports the same set of field references as the `filter` field. Entries
	// can be prefixed with a minus sign to sort by the field in descending order.
	//
	// For more details, see [sorting and
	// filtering](https://cloud.google.com/monitoring/api/v3/sorting-and-filtering).
	OrderBy string `protobuf:"bytes,6,opt,name=order_by,json=orderBy,proto3" json:"order_by,omitempty"`
	// Optional. The maximum number of results to return in a single response.
	PageSize int32 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Optional. If this field is not empty then it must contain the
	// `nextPageToken` value returned by a previous call to this method.  Using
	// this field causes the method to return more results from the previous
	// method call.
	PageToken     string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListAlertPoliciesRequest) Reset() {
	*x = ListAlertPoliciesRequest{}
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListAlertPoliciesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListAlertPoliciesRequest) ProtoMessage() {}

func (x *ListAlertPoliciesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListAlertPoliciesRequest.ProtoReflect.Descriptor instead.
func (*ListAlertPoliciesRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_alert_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListAlertPoliciesRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ListAlertPoliciesRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

func (x *ListAlertPoliciesRequest) GetOrderBy() string {
	if x != nil {
		return x.OrderBy
	}
	return ""
}

func (x *ListAlertPoliciesRequest) GetPageSize() int32 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListAlertPoliciesRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

// The protocol for the `ListAlertPolicies` response.
type ListAlertPoliciesResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The returned alert policies.
	AlertPolicies []*AlertPolicy `protobuf:"bytes,3,rep,name=alert_policies,json=alertPolicies,proto3" json:"alert_policies,omitempty"`
	// If there might be more results than were returned, then this field is set
	// to a non-empty value. To see the additional results,
	// use that value as `page_token` in the next call to this method.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	// The total number of alert policies in all pages. This number is only an
	// estimate, and may change in subsequent pages. https://aip.dev/158
	TotalSize     int32 `protobuf:"varint,4,opt,name=total_size,json=totalSize,proto3" json:"total_size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListAlertPoliciesResponse) Reset() {
	*x = ListAlertPoliciesResponse{}
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListAlertPoliciesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListAlertPoliciesResponse) ProtoMessage() {}

func (x *ListAlertPoliciesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListAlertPoliciesResponse.ProtoReflect.Descriptor instead.
func (*ListAlertPoliciesResponse) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_alert_service_proto_rawDescGZIP(), []int{3}
}

func (x *ListAlertPoliciesResponse) GetAlertPolicies() []*AlertPolicy {
	if x != nil {
		return x.AlertPolicies
	}
	return nil
}

func (x *ListAlertPoliciesResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

func (x *ListAlertPoliciesResponse) GetTotalSize() int32 {
	if x != nil {
		return x.TotalSize
	}
	return 0
}

// The protocol for the `UpdateAlertPolicy` request.
type UpdateAlertPolicyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. A list of alerting policy field names. If this field is not
	// empty, each listed field in the existing alerting policy is set to the
	// value of the corresponding field in the supplied policy (`alert_policy`),
	// or to the field's default value if the field is not in the supplied
	// alerting policy.  Fields not listed retain their previous value.
	//
	// Examples of valid field masks include `display_name`, `documentation`,
	// `documentation.content`, `documentation.mime_type`, `user_labels`,
	// `user_label.nameofkey`, `enabled`, `conditions`, `combiner`, etc.
	//
	// If this field is empty, then the supplied alerting policy replaces the
	// existing policy. It is the same as deleting the existing policy and
	// adding the supplied policy, except for the following:
	//
	//   - The new policy will have the same `[ALERT_POLICY_ID]` as the former
	//     policy. This gives you continuity with the former policy in your
	//     notifications and incidents.
	//   - Conditions in the new policy will keep their former `[CONDITION_ID]` if
	//     the supplied condition includes the `name` field with that
	//     `[CONDITION_ID]`. If the supplied condition omits the `name` field,
	//     then a new `[CONDITION_ID]` is created.
	UpdateMask *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	// Required. The updated alerting policy or the updated values for the
	// fields listed in `update_mask`.
	// If `update_mask` is not empty, any fields in this policy that are
	// not in `update_mask` are ignored.
	AlertPolicy   *AlertPolicy `protobuf:"bytes,3,opt,name=alert_policy,json=alertPolicy,proto3" json:"alert_policy,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateAlertPolicyRequest) Reset() {
	*x = UpdateAlertPolicyRequest{}
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAlertPolicyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAlertPolicyRequest) ProtoMessage() {}

func (x *UpdateAlertPolicyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAlertPolicyRequest.ProtoReflect.Descriptor instead.
func (*UpdateAlertPolicyRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_alert_service_proto_rawDescGZIP(), []int{4}
}

func (x *UpdateAlertPolicyRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

func (x *UpdateAlertPolicyRequest) GetAlertPolicy() *AlertPolicy {
	if x != nil {
		return x.AlertPolicy
	}
	return nil
}

// The protocol for the `DeleteAlertPolicy` request.
type DeleteAlertPolicyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The alerting policy to delete. The format is:
	//
	//	projects/[PROJECT_ID_OR_NUMBER]/alertPolicies/[ALERT_POLICY_ID]
	//
	// For more information, see [AlertPolicy][google.monitoring.v3.AlertPolicy].
	Name          string `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteAlertPolicyRequest) Reset() {
	*x = DeleteAlertPolicyRequest{}
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteAlertPolicyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteAlertPolicyRequest) ProtoMessage() {}

func (x *DeleteAlertPolicyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_monitoring_v3_alert_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteAlertPolicyRequest.ProtoReflect.Descriptor instead.
func (*DeleteAlertPolicyRequest) Descriptor() ([]byte, []int) {
	return file_google_monitoring_v3_alert_service_proto_rawDescGZIP(), []int{5}
}

func (x *DeleteAlertPolicyRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

var File_google_monitoring_v3_alert_service_proto protoreflect.FileDescriptor

const file_google_monitoring_v3_alert_service_proto_rawDesc = "" +
	"\n" +
	"(google/monitoring/v3/alert_service.proto\x12\x14google.monitoring.v3\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a google/monitoring/v3/alert.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a google/protobuf/field_mask.proto\"\xa8\x01\n" +
	"\x18CreateAlertPolicyRequest\x12A\n" +
	"\x04name\x18\x03 \x01(\tB-\xe0A\x02\xfaA'\x12%monitoring.googleapis.com/AlertPolicyR\x04name\x12I\n" +
	"\falert_policy\x18\x02 \x01(\v2!.google.monitoring.v3.AlertPolicyB\x03\xe0A\x02R\valertPolicy\"Z\n" +
	"\x15GetAlertPolicyRequest\x12A\n" +
	"\x04name\x18\x03 \x01(\tB-\xe0A\x02\xfaA'\n" +
	"%monitoring.googleapis.com/AlertPolicyR\x04name\"\xe0\x01\n" +
	"\x18ListAlertPoliciesRequest\x12A\n" +
	"\x04name\x18\x04 \x01(\tB-\xe0A\x02\xfaA'\x12%monitoring.googleapis.com/AlertPolicyR\x04name\x12\x1b\n" +
	"\x06filter\x18\x05 \x01(\tB\x03\xe0A\x01R\x06filter\x12\x1e\n" +
	"\border_by\x18\x06 \x01(\tB\x03\xe0A\x01R\aorderBy\x12 \n" +
	"\tpage_size\x18\x02 \x01(\x05B\x03\xe0A\x01R\bpageSize\x12\"\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\x03\xe0A\x01R\tpageToken\"\xac\x01\n" +
	"\x19ListAlertPoliciesResponse\x12H\n" +
	"\x0ealert_policies\x18\x03 \x03(\v2!.google.monitoring.v3.AlertPolicyR\ralertPolicies\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\x12\x1d\n" +
	"\n" +
	"total_size\x18\x04 \x01(\x05R\ttotalSize\"\xa7\x01\n" +
	"\x18UpdateAlertPolicyRequest\x12@\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskB\x03\xe0A\x01R\n" +
	"updateMask\x12I\n" +
	"\falert_policy\x18\x03 \x01(\v2!.google.monitoring.v3.AlertPolicyB\x03\xe0A\x02R\valertPolicy\"]\n" +
	"\x18DeleteAlertPolicyRequest\x12A\n" +
	"\x04name\x18\x03 \x01(\tB-\xe0A\x02\xfaA'\n" +
	"%monitoring.googleapis.com/AlertPolicyR\x04name2\x9e\b\n" +
	"\x12AlertPolicyService\x12\xa8\x01\n" +
	"\x11ListAlertPolicies\x12..google.monitoring.v3.ListAlertPoliciesRequest\x1a/.google.monitoring.v3.ListAlertPoliciesResponse\"2\xdaA\x04name\x82\xd3\xe4\x93\x02%\x12#/v3/{name=projects/*}/alertPolicies\x12\x96\x01\n" +
	"\x0eGetAlertPolicy\x12+.google.monitoring.v3.GetAlertPolicyRequest\x1a!.google.monitoring.v3.AlertPolicy\"4\xdaA\x04name\x82\xd3\xe4\x93\x02'\x12%/v3/{name=projects/*/alertPolicies/*}\x12\xb5\x01\n" +
	"\x11CreateAlertPolicy\x12..google.monitoring.v3.CreateAlertPolicyRequest\x1a!.google.monitoring.v3.AlertPolicy\"M\xdaA\x11name,alert_policy\x82\xd3\xe4\x93\x023:\falert_policy\"#/v3/{name=projects/*}/alertPolicies\x12\x91\x01\n" +
	"\x11DeleteAlertPolicy\x12..google.monitoring.v3.DeleteAlertPolicyRequest\x1a\x16.google.protobuf.Empty\"4\xdaA\x04name\x82\xd3\xe4\x93\x02'*%/v3/{name=projects/*/alertPolicies/*}\x12\xcb\x01\n" +
	"\x11UpdateAlertPolicy\x12..google.monitoring.v3.UpdateAlertPolicyRequest\x1a!.google.monitoring.v3.AlertPolicy\"c\xdaA\x18update_mask,alert_policy\x82\xd3\xe4\x93\x02B:\falert_policy22/v3/{alert_policy.name=projects/*/alertPolicies/*}\x1a\xa9\x01\xcaA\x19monitoring.googleapis.com\xd2A\x89\x01https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/monitoring,https://www.googleapis.com/auth/monitoring.readB\xcc\x01\n" +
	"\x18com.google.monitoring.v3B\x11AlertServiceProtoP\x01ZAcloud.google.com/go/monitoring/apiv3/v2/monitoringpb;monitoringpb\xaa\x02\x1aGoogle.Cloud.Monitoring.V3\xca\x02\x1aGoogle\\Cloud\\Monitoring\\V3\xea\x02\x1dGoogle::Cloud::Monitoring::V3b\x06proto3"

var (
	file_google_monitoring_v3_alert_service_proto_rawDescOnce sync.Once
	file_google_monitoring_v3_alert_service_proto_rawDescData []byte
)

func file_google_monitoring_v3_alert_service_proto_rawDescGZIP() []byte {
	file_google_monitoring_v3_alert_service_proto_rawDescOnce.Do(func() {
		file_google_monitoring_v3_alert_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_alert_service_proto_rawDesc), len(file_google_monitoring_v3_alert_service_proto_rawDesc)))
	})
	return file_google_monitoring_v3_alert_service_proto_rawDescData
}

var file_google_monitoring_v3_alert_service_proto_msgTypes = make([]protoimpl.MessageInfo, 6)
var file_google_monitoring_v3_alert_service_proto_goTypes = []any{
	(*CreateAlertPolicyRequest)(nil),  // 0: google.monitoring.v3.CreateAlertPolicyRequest
	(*GetAlertPolicyRequest)(nil),     // 1: google.monitoring.v3.GetAlertPolicyRequest
	(*ListAlertPoliciesRequest)(nil),  // 2: google.monitoring.v3.ListAlertPoliciesRequest
	(*ListAlertPoliciesResponse)(nil), // 3: google.monitoring.v3.ListAlertPoliciesResponse
	(*UpdateAlertPolicyRequest)(nil),  // 4: google.monitoring.v3.UpdateAlertPolicyRequest
	(*DeleteAlertPolicyRequest)(nil),  // 5: google.monitoring.v3.DeleteAlertPolicyRequest
	(*AlertPolicy)(nil),               // 6: google.monitoring.v3.AlertPolicy
	(*fieldmaskpb.FieldMask)(nil),     // 7: google.protobuf.FieldMask
	(*emptypb.Empty)(nil),             // 8: google.protobuf.Empty
}
var file_google_monitoring_v3_alert_service_proto_depIdxs = []int32{
	6, // 0: google.monitoring.v3.CreateAlertPolicyRequest.alert_policy:type_name -> google.monitoring.v3.AlertPolicy
	6, // 1: google.monitoring.v3.ListAlertPoliciesResponse.alert_policies:type_name -> google.monitoring.v3.AlertPolicy
	7, // 2: google.monitoring.v3.UpdateAlertPolicyRequest.update_mask:type_name -> google.protobuf.FieldMask
	6, // 3: google.monitoring.v3.UpdateAlertPolicyRequest.alert_policy:type_name -> google.monitoring.v3.AlertPolicy
	2, // 4: google.monitoring.v3.AlertPolicyService.ListAlertPolicies:input_type -> google.monitoring.v3.ListAlertPoliciesRequest
	1, // 5: google.monitoring.v3.AlertPolicyService.GetAlertPolicy:input_type -> google.monitoring.v3.GetAlertPolicyRequest
	0, // 6: google.monitoring.v3.AlertPolicyService.CreateAlertPolicy:input_type -> google.monitoring.v3.CreateAlertPolicyRequest
	5, // 7: google.monitoring.v3.AlertPolicyService.DeleteAlertPolicy:input_type -> google.monitoring.v3.DeleteAlertPolicyRequest
	4, // 8: google.monitoring.v3.AlertPolicyService.UpdateAlertPolicy:input_type -> google.monitoring.v3.UpdateAlertPolicyRequest
	3, // 9: google.monitoring.v3.AlertPolicyService.ListAlertPolicies:output_type -> google.monitoring.v3.ListAlertPoliciesResponse
	6, // 10: google.monitoring.v3.AlertPolicyService.GetAlertPolicy:output_type -> google.monitoring.v3.AlertPolicy
	6, // 11: google.monitoring.v3.AlertPolicyService.CreateAlertPolicy:output_type -> google.monitoring.v3.AlertPolicy
	8, // 12: google.monitoring.v3.AlertPolicyService.DeleteAlertPolicy:output_type -> google.protobuf.Empty
	6, // 13: google.monitoring.v3.AlertPolicyService.UpdateAlertPolicy:output_type -> google.monitoring.v3.AlertPolicy
	9, // [9:14] is the sub-list for method output_type
	4, // [4:9] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_google_monitoring_v3_alert_service_proto_init() }
func file_google_monitoring_v3_alert_service_proto_init() {
	if File_google_monitoring_v3_alert_service_proto != nil {
		return
	}
	file_google_monitoring_v3_alert_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_monitoring_v3_alert_service_proto_rawDesc), len(file_google_monitoring_v3_alert_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   6,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_google_monitoring_v3_alert_service_proto_goTypes,
		DependencyIndexes: file_google_monitoring_v3_alert_service_proto_depIdxs,
		MessageInfos:      file_google_monitoring_v3_alert_service_proto_msgTypes,
	}.Build()
	File_google_monitoring_v3_alert_service_proto = out.File
	file_google_monitoring_v3_alert_service_proto_goTypes = nil
	file_google_monitoring_v3_alert_service_proto_depIdxs = nil
}
