// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Code generated by "deep-copy -type LVMLogicalVolumeSpecSpec -type LVMLogicalVolumeStatusSpec -type LVMPhysicalVolumeSpecSpec -type LVMPhysicalVolumeStatusSpec -type LVMRefreshRequestSpec -type LVMValidationErrorSpec -type LVMVolumeGroupSpecSpec -type LVMVolumeGroupStatusSpec -header-file ../../../../hack/boilerplate.txt -o deep_copy.generated.go ."; DO NOT EDIT.

package storage

// DeepCopy generates a deep copy of LVMLogicalVolumeSpecSpec.
func (o LVMLogicalVolumeSpecSpec) DeepCopy() LVMLogicalVolumeSpecSpec {
	var cp LVMLogicalVolumeSpecSpec = o
	return cp
}

// DeepCopy generates a deep copy of LVMLogicalVolumeStatusSpec.
func (o LVMLogicalVolumeStatusSpec) DeepCopy() LVMLogicalVolumeStatusSpec {
	var cp LVMLogicalVolumeStatusSpec = o
	if o.Tags != nil {
		cp.Tags = make([]string, len(o.Tags))
		copy(cp.Tags, o.Tags)
	}
	return cp
}

// DeepCopy generates a deep copy of LVMPhysicalVolumeSpecSpec.
func (o LVMPhysicalVolumeSpecSpec) DeepCopy() LVMPhysicalVolumeSpecSpec {
	var cp LVMPhysicalVolumeSpecSpec = o
	return cp
}

// DeepCopy generates a deep copy of LVMPhysicalVolumeStatusSpec.
func (o LVMPhysicalVolumeStatusSpec) DeepCopy() LVMPhysicalVolumeStatusSpec {
	var cp LVMPhysicalVolumeStatusSpec = o
	if o.Tags != nil {
		cp.Tags = make([]string, len(o.Tags))
		copy(cp.Tags, o.Tags)
	}
	return cp
}

// DeepCopy generates a deep copy of LVMRefreshRequestSpec.
func (o LVMRefreshRequestSpec) DeepCopy() LVMRefreshRequestSpec {
	var cp LVMRefreshRequestSpec = o
	return cp
}

// DeepCopy generates a deep copy of LVMValidationErrorSpec.
func (o LVMValidationErrorSpec) DeepCopy() LVMValidationErrorSpec {
	var cp LVMValidationErrorSpec = o
	return cp
}

// DeepCopy generates a deep copy of LVMVolumeGroupSpecSpec.
func (o LVMVolumeGroupSpecSpec) DeepCopy() LVMVolumeGroupSpecSpec {
	var cp LVMVolumeGroupSpecSpec = o
	if o.PhysicalVolumes != nil {
		cp.PhysicalVolumes = make([]string, len(o.PhysicalVolumes))
		copy(cp.PhysicalVolumes, o.PhysicalVolumes)
	}
	return cp
}

// DeepCopy generates a deep copy of LVMVolumeGroupStatusSpec.
func (o LVMVolumeGroupStatusSpec) DeepCopy() LVMVolumeGroupStatusSpec {
	var cp LVMVolumeGroupStatusSpec = o
	if o.Tags != nil {
		cp.Tags = make([]string, len(o.Tags))
		copy(cp.Tags, o.Tags)
	}
	return cp
}
