// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/common/matcher/v3/matcher.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/route/v3"
	v32 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A matcher, which may traverse a matching tree in order to result in a match action.
// During matching, the tree will be traversed until a match is found, or if no match
// is found the action specified by the most specific on_no_match will be evaluated.
// As an on_no_match might result in another matching tree being evaluated, this process
// might repeat several times until the final OnMatch (or no match) is decided.
//
// .. note::
//
//	Please use the syntactically equivalent :ref:`matching API <envoy_v3_api_msg_.xds.type.matcher.v3.Matcher>`
type Matcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatcherType:
	//
	//	*Matcher_MatcherList_
	//	*Matcher_MatcherTree_
	MatcherType isMatcher_MatcherType `protobuf_oneof:"matcher_type"`
	// Optional “OnMatch“ to use if the matcher failed.
	// If specified, the “OnMatch“ is used, and the matcher is considered
	// to have matched.
	// If not specified, the matcher is considered not to have matched.
	OnNoMatch     *Matcher_OnMatch `protobuf:"bytes,3,opt,name=on_no_match,json=onNoMatch,proto3" json:"on_no_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher) Reset() {
	*x = Matcher{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher) ProtoMessage() {}

func (x *Matcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher.ProtoReflect.Descriptor instead.
func (*Matcher) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0}
}

func (x *Matcher) GetMatcherType() isMatcher_MatcherType {
	if x != nil {
		return x.MatcherType
	}
	return nil
}

func (x *Matcher) GetMatcherList() *Matcher_MatcherList {
	if x != nil {
		if x, ok := x.MatcherType.(*Matcher_MatcherList_); ok {
			return x.MatcherList
		}
	}
	return nil
}

func (x *Matcher) GetMatcherTree() *Matcher_MatcherTree {
	if x != nil {
		if x, ok := x.MatcherType.(*Matcher_MatcherTree_); ok {
			return x.MatcherTree
		}
	}
	return nil
}

func (x *Matcher) GetOnNoMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnNoMatch
	}
	return nil
}

type isMatcher_MatcherType interface {
	isMatcher_MatcherType()
}

type Matcher_MatcherList_ struct {
	// A linear list of matchers to evaluate.
	MatcherList *Matcher_MatcherList `protobuf:"bytes,1,opt,name=matcher_list,json=matcherList,proto3,oneof"`
}

type Matcher_MatcherTree_ struct {
	// A match tree to evaluate.
	MatcherTree *Matcher_MatcherTree `protobuf:"bytes,2,opt,name=matcher_tree,json=matcherTree,proto3,oneof"`
}

func (*Matcher_MatcherList_) isMatcher_MatcherType() {}

func (*Matcher_MatcherTree_) isMatcher_MatcherType() {}

// Match configuration. This is a recursive structure which allows complex nested match
// configurations to be built using various logical operators.
// [#next-free-field: 11]
type MatchPredicate struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Rule:
	//
	//	*MatchPredicate_OrMatch
	//	*MatchPredicate_AndMatch
	//	*MatchPredicate_NotMatch
	//	*MatchPredicate_AnyMatch
	//	*MatchPredicate_HttpRequestHeadersMatch
	//	*MatchPredicate_HttpRequestTrailersMatch
	//	*MatchPredicate_HttpResponseHeadersMatch
	//	*MatchPredicate_HttpResponseTrailersMatch
	//	*MatchPredicate_HttpRequestGenericBodyMatch
	//	*MatchPredicate_HttpResponseGenericBodyMatch
	Rule          isMatchPredicate_Rule `protobuf_oneof:"rule"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MatchPredicate) Reset() {
	*x = MatchPredicate{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MatchPredicate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MatchPredicate) ProtoMessage() {}

func (x *MatchPredicate) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MatchPredicate.ProtoReflect.Descriptor instead.
func (*MatchPredicate) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{1}
}

func (x *MatchPredicate) GetRule() isMatchPredicate_Rule {
	if x != nil {
		return x.Rule
	}
	return nil
}

func (x *MatchPredicate) GetOrMatch() *MatchPredicate_MatchSet {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_OrMatch); ok {
			return x.OrMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetAndMatch() *MatchPredicate_MatchSet {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_AndMatch); ok {
			return x.AndMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetNotMatch() *MatchPredicate {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_NotMatch); ok {
			return x.NotMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetAnyMatch() bool {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_AnyMatch); ok {
			return x.AnyMatch
		}
	}
	return false
}

func (x *MatchPredicate) GetHttpRequestHeadersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpRequestHeadersMatch); ok {
			return x.HttpRequestHeadersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpRequestTrailersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpRequestTrailersMatch); ok {
			return x.HttpRequestTrailersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpResponseHeadersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpResponseHeadersMatch); ok {
			return x.HttpResponseHeadersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpResponseTrailersMatch() *HttpHeadersMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpResponseTrailersMatch); ok {
			return x.HttpResponseTrailersMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpRequestGenericBodyMatch() *HttpGenericBodyMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpRequestGenericBodyMatch); ok {
			return x.HttpRequestGenericBodyMatch
		}
	}
	return nil
}

func (x *MatchPredicate) GetHttpResponseGenericBodyMatch() *HttpGenericBodyMatch {
	if x != nil {
		if x, ok := x.Rule.(*MatchPredicate_HttpResponseGenericBodyMatch); ok {
			return x.HttpResponseGenericBodyMatch
		}
	}
	return nil
}

type isMatchPredicate_Rule interface {
	isMatchPredicate_Rule()
}

type MatchPredicate_OrMatch struct {
	// A set that describes a logical OR. If any member of the set matches, the match configuration
	// matches.
	OrMatch *MatchPredicate_MatchSet `protobuf:"bytes,1,opt,name=or_match,json=orMatch,proto3,oneof"`
}

type MatchPredicate_AndMatch struct {
	// A set that describes a logical AND. If all members of the set match, the match configuration
	// matches.
	AndMatch *MatchPredicate_MatchSet `protobuf:"bytes,2,opt,name=and_match,json=andMatch,proto3,oneof"`
}

type MatchPredicate_NotMatch struct {
	// A negation match. The match configuration will match if the negated match condition matches.
	NotMatch *MatchPredicate `protobuf:"bytes,3,opt,name=not_match,json=notMatch,proto3,oneof"`
}

type MatchPredicate_AnyMatch struct {
	// The match configuration will always match.
	AnyMatch bool `protobuf:"varint,4,opt,name=any_match,json=anyMatch,proto3,oneof"`
}

type MatchPredicate_HttpRequestHeadersMatch struct {
	// HTTP request headers match configuration.
	HttpRequestHeadersMatch *HttpHeadersMatch `protobuf:"bytes,5,opt,name=http_request_headers_match,json=httpRequestHeadersMatch,proto3,oneof"`
}

type MatchPredicate_HttpRequestTrailersMatch struct {
	// HTTP request trailers match configuration.
	HttpRequestTrailersMatch *HttpHeadersMatch `protobuf:"bytes,6,opt,name=http_request_trailers_match,json=httpRequestTrailersMatch,proto3,oneof"`
}

type MatchPredicate_HttpResponseHeadersMatch struct {
	// HTTP response headers match configuration.
	HttpResponseHeadersMatch *HttpHeadersMatch `protobuf:"bytes,7,opt,name=http_response_headers_match,json=httpResponseHeadersMatch,proto3,oneof"`
}

type MatchPredicate_HttpResponseTrailersMatch struct {
	// HTTP response trailers match configuration.
	HttpResponseTrailersMatch *HttpHeadersMatch `protobuf:"bytes,8,opt,name=http_response_trailers_match,json=httpResponseTrailersMatch,proto3,oneof"`
}

type MatchPredicate_HttpRequestGenericBodyMatch struct {
	// HTTP request generic body match configuration.
	HttpRequestGenericBodyMatch *HttpGenericBodyMatch `protobuf:"bytes,9,opt,name=http_request_generic_body_match,json=httpRequestGenericBodyMatch,proto3,oneof"`
}

type MatchPredicate_HttpResponseGenericBodyMatch struct {
	// HTTP response generic body match configuration.
	HttpResponseGenericBodyMatch *HttpGenericBodyMatch `protobuf:"bytes,10,opt,name=http_response_generic_body_match,json=httpResponseGenericBodyMatch,proto3,oneof"`
}

func (*MatchPredicate_OrMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_AndMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_NotMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_AnyMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpRequestHeadersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpRequestTrailersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpResponseHeadersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpResponseTrailersMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpRequestGenericBodyMatch) isMatchPredicate_Rule() {}

func (*MatchPredicate_HttpResponseGenericBodyMatch) isMatchPredicate_Rule() {}

// HTTP headers match configuration.
type HttpHeadersMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// HTTP headers to match.
	Headers       []*v3.HeaderMatcher `protobuf:"bytes,1,rep,name=headers,proto3" json:"headers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpHeadersMatch) Reset() {
	*x = HttpHeadersMatch{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpHeadersMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpHeadersMatch) ProtoMessage() {}

func (x *HttpHeadersMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpHeadersMatch.ProtoReflect.Descriptor instead.
func (*HttpHeadersMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{2}
}

func (x *HttpHeadersMatch) GetHeaders() []*v3.HeaderMatcher {
	if x != nil {
		return x.Headers
	}
	return nil
}

// HTTP generic body match configuration.
// List of text strings and hex strings to be located in HTTP body.
// All specified strings must be found in the HTTP body for positive match.
// The search may be limited to specified number of bytes from the body start.
//
// .. attention::
//
//	Searching for patterns in HTTP body is potentially CPU-intensive. For each specified pattern, HTTP body is scanned byte by byte to find a match.
//	If multiple patterns are specified, the process is repeated for each pattern. If location of a pattern is known, ``bytes_limit`` should be specified
//	to scan only part of the HTTP body.
type HttpGenericBodyMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Limits search to specified number of bytes - default zero (no limit - match entire captured buffer).
	BytesLimit uint32 `protobuf:"varint,1,opt,name=bytes_limit,json=bytesLimit,proto3" json:"bytes_limit,omitempty"`
	// List of patterns to match.
	Patterns      []*HttpGenericBodyMatch_GenericTextMatch `protobuf:"bytes,2,rep,name=patterns,proto3" json:"patterns,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpGenericBodyMatch) Reset() {
	*x = HttpGenericBodyMatch{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpGenericBodyMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpGenericBodyMatch) ProtoMessage() {}

func (x *HttpGenericBodyMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpGenericBodyMatch.ProtoReflect.Descriptor instead.
func (*HttpGenericBodyMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{3}
}

func (x *HttpGenericBodyMatch) GetBytesLimit() uint32 {
	if x != nil {
		return x.BytesLimit
	}
	return 0
}

func (x *HttpGenericBodyMatch) GetPatterns() []*HttpGenericBodyMatch_GenericTextMatch {
	if x != nil {
		return x.Patterns
	}
	return nil
}

// What to do if a match is successful.
type Matcher_OnMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to OnMatch:
	//
	//	*Matcher_OnMatch_Matcher
	//	*Matcher_OnMatch_Action
	OnMatch isMatcher_OnMatch_OnMatch `protobuf_oneof:"on_match"`
	// If true, the action will be taken but the caller will behave as if no
	// match was found. This applies both to actions directly encoded in the
	// action field and to actions returned from a nested matcher tree in the
	// matcher field. A subsequent matcher on_no_match action will be used
	// instead.
	//
	// This field is not supported in all contexts in which the matcher API is
	// used. If this field is set in a context in which it's not supported,
	// the resource will be rejected.
	KeepMatching  bool `protobuf:"varint,3,opt,name=keep_matching,json=keepMatching,proto3" json:"keep_matching,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_OnMatch) Reset() {
	*x = Matcher_OnMatch{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_OnMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_OnMatch) ProtoMessage() {}

func (x *Matcher_OnMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_OnMatch.ProtoReflect.Descriptor instead.
func (*Matcher_OnMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 0}
}

func (x *Matcher_OnMatch) GetOnMatch() isMatcher_OnMatch_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

func (x *Matcher_OnMatch) GetMatcher() *Matcher {
	if x != nil {
		if x, ok := x.OnMatch.(*Matcher_OnMatch_Matcher); ok {
			return x.Matcher
		}
	}
	return nil
}

func (x *Matcher_OnMatch) GetAction() *v31.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.OnMatch.(*Matcher_OnMatch_Action); ok {
			return x.Action
		}
	}
	return nil
}

func (x *Matcher_OnMatch) GetKeepMatching() bool {
	if x != nil {
		return x.KeepMatching
	}
	return false
}

type isMatcher_OnMatch_OnMatch interface {
	isMatcher_OnMatch_OnMatch()
}

type Matcher_OnMatch_Matcher struct {
	// Nested matcher to evaluate.
	// If the nested matcher does not match and does not specify
	// on_no_match, then this matcher is considered not to have
	// matched, even if a predicate at this level or above returned
	// true.
	Matcher *Matcher `protobuf:"bytes,1,opt,name=matcher,proto3,oneof"`
}

type Matcher_OnMatch_Action struct {
	// Protocol-specific action to take.
	Action *v31.TypedExtensionConfig `protobuf:"bytes,2,opt,name=action,proto3,oneof"`
}

func (*Matcher_OnMatch_Matcher) isMatcher_OnMatch_OnMatch() {}

func (*Matcher_OnMatch_Action) isMatcher_OnMatch_OnMatch() {}

// A linear list of field matchers.
// The field matchers are evaluated in order, and the first match
// wins.
type Matcher_MatcherList struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A list of matchers. First match wins.
	Matchers      []*Matcher_MatcherList_FieldMatcher `protobuf:"bytes,1,rep,name=matchers,proto3" json:"matchers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList) Reset() {
	*x = Matcher_MatcherList{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList) ProtoMessage() {}

func (x *Matcher_MatcherList) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1}
}

func (x *Matcher_MatcherList) GetMatchers() []*Matcher_MatcherList_FieldMatcher {
	if x != nil {
		return x.Matchers
	}
	return nil
}

type Matcher_MatcherTree struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Protocol-specific specification of input field to match on.
	Input *v31.TypedExtensionConfig `protobuf:"bytes,1,opt,name=input,proto3" json:"input,omitempty"`
	// Exact or prefix match maps in which to look up the input value.
	// If the lookup succeeds, the match is considered successful, and
	// the corresponding OnMatch is used.
	//
	// Types that are valid to be assigned to TreeType:
	//
	//	*Matcher_MatcherTree_ExactMatchMap
	//	*Matcher_MatcherTree_PrefixMatchMap
	//	*Matcher_MatcherTree_CustomMatch
	TreeType      isMatcher_MatcherTree_TreeType `protobuf_oneof:"tree_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherTree) Reset() {
	*x = Matcher_MatcherTree{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherTree) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherTree) ProtoMessage() {}

func (x *Matcher_MatcherTree) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherTree.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherTree) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 2}
}

func (x *Matcher_MatcherTree) GetInput() *v31.TypedExtensionConfig {
	if x != nil {
		return x.Input
	}
	return nil
}

func (x *Matcher_MatcherTree) GetTreeType() isMatcher_MatcherTree_TreeType {
	if x != nil {
		return x.TreeType
	}
	return nil
}

func (x *Matcher_MatcherTree) GetExactMatchMap() *Matcher_MatcherTree_MatchMap {
	if x != nil {
		if x, ok := x.TreeType.(*Matcher_MatcherTree_ExactMatchMap); ok {
			return x.ExactMatchMap
		}
	}
	return nil
}

func (x *Matcher_MatcherTree) GetPrefixMatchMap() *Matcher_MatcherTree_MatchMap {
	if x != nil {
		if x, ok := x.TreeType.(*Matcher_MatcherTree_PrefixMatchMap); ok {
			return x.PrefixMatchMap
		}
	}
	return nil
}

func (x *Matcher_MatcherTree) GetCustomMatch() *v31.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.TreeType.(*Matcher_MatcherTree_CustomMatch); ok {
			return x.CustomMatch
		}
	}
	return nil
}

type isMatcher_MatcherTree_TreeType interface {
	isMatcher_MatcherTree_TreeType()
}

type Matcher_MatcherTree_ExactMatchMap struct {
	ExactMatchMap *Matcher_MatcherTree_MatchMap `protobuf:"bytes,2,opt,name=exact_match_map,json=exactMatchMap,proto3,oneof"`
}

type Matcher_MatcherTree_PrefixMatchMap struct {
	// Longest matching prefix wins.
	PrefixMatchMap *Matcher_MatcherTree_MatchMap `protobuf:"bytes,3,opt,name=prefix_match_map,json=prefixMatchMap,proto3,oneof"`
}

type Matcher_MatcherTree_CustomMatch struct {
	// Extension for custom matching logic.
	CustomMatch *v31.TypedExtensionConfig `protobuf:"bytes,4,opt,name=custom_match,json=customMatch,proto3,oneof"`
}

func (*Matcher_MatcherTree_ExactMatchMap) isMatcher_MatcherTree_TreeType() {}

func (*Matcher_MatcherTree_PrefixMatchMap) isMatcher_MatcherTree_TreeType() {}

func (*Matcher_MatcherTree_CustomMatch) isMatcher_MatcherTree_TreeType() {}

// Predicate to determine if a match is successful.
type Matcher_MatcherList_Predicate struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatchType:
	//
	//	*Matcher_MatcherList_Predicate_SinglePredicate_
	//	*Matcher_MatcherList_Predicate_OrMatcher
	//	*Matcher_MatcherList_Predicate_AndMatcher
	//	*Matcher_MatcherList_Predicate_NotMatcher
	MatchType     isMatcher_MatcherList_Predicate_MatchType `protobuf_oneof:"match_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_Predicate) Reset() {
	*x = Matcher_MatcherList_Predicate{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_Predicate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_Predicate) ProtoMessage() {}

func (x *Matcher_MatcherList_Predicate) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_Predicate.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_Predicate) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 0}
}

func (x *Matcher_MatcherList_Predicate) GetMatchType() isMatcher_MatcherList_Predicate_MatchType {
	if x != nil {
		return x.MatchType
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetSinglePredicate() *Matcher_MatcherList_Predicate_SinglePredicate {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_SinglePredicate_); ok {
			return x.SinglePredicate
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetOrMatcher() *Matcher_MatcherList_Predicate_PredicateList {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_OrMatcher); ok {
			return x.OrMatcher
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetAndMatcher() *Matcher_MatcherList_Predicate_PredicateList {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_AndMatcher); ok {
			return x.AndMatcher
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetNotMatcher() *Matcher_MatcherList_Predicate {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_NotMatcher); ok {
			return x.NotMatcher
		}
	}
	return nil
}

type isMatcher_MatcherList_Predicate_MatchType interface {
	isMatcher_MatcherList_Predicate_MatchType()
}

type Matcher_MatcherList_Predicate_SinglePredicate_ struct {
	// A single predicate to evaluate.
	SinglePredicate *Matcher_MatcherList_Predicate_SinglePredicate `protobuf:"bytes,1,opt,name=single_predicate,json=singlePredicate,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_OrMatcher struct {
	// A list of predicates to be OR-ed together.
	OrMatcher *Matcher_MatcherList_Predicate_PredicateList `protobuf:"bytes,2,opt,name=or_matcher,json=orMatcher,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_AndMatcher struct {
	// A list of predicates to be AND-ed together.
	AndMatcher *Matcher_MatcherList_Predicate_PredicateList `protobuf:"bytes,3,opt,name=and_matcher,json=andMatcher,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_NotMatcher struct {
	// The inverse of a predicate
	NotMatcher *Matcher_MatcherList_Predicate `protobuf:"bytes,4,opt,name=not_matcher,json=notMatcher,proto3,oneof"`
}

func (*Matcher_MatcherList_Predicate_SinglePredicate_) isMatcher_MatcherList_Predicate_MatchType() {}

func (*Matcher_MatcherList_Predicate_OrMatcher) isMatcher_MatcherList_Predicate_MatchType() {}

func (*Matcher_MatcherList_Predicate_AndMatcher) isMatcher_MatcherList_Predicate_MatchType() {}

func (*Matcher_MatcherList_Predicate_NotMatcher) isMatcher_MatcherList_Predicate_MatchType() {}

// An individual matcher.
type Matcher_MatcherList_FieldMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Determines if the match succeeds.
	Predicate *Matcher_MatcherList_Predicate `protobuf:"bytes,1,opt,name=predicate,proto3" json:"predicate,omitempty"`
	// What to do if the match succeeds.
	OnMatch       *Matcher_OnMatch `protobuf:"bytes,2,opt,name=on_match,json=onMatch,proto3" json:"on_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_FieldMatcher) Reset() {
	*x = Matcher_MatcherList_FieldMatcher{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_FieldMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_FieldMatcher) ProtoMessage() {}

func (x *Matcher_MatcherList_FieldMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_FieldMatcher.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_FieldMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 1}
}

func (x *Matcher_MatcherList_FieldMatcher) GetPredicate() *Matcher_MatcherList_Predicate {
	if x != nil {
		return x.Predicate
	}
	return nil
}

func (x *Matcher_MatcherList_FieldMatcher) GetOnMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

// Predicate for a single input field.
type Matcher_MatcherList_Predicate_SinglePredicate struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Protocol-specific specification of input field to match on.
	// [#extension-category: envoy.matching.common_inputs]
	Input *v31.TypedExtensionConfig `protobuf:"bytes,1,opt,name=input,proto3" json:"input,omitempty"`
	// Types that are valid to be assigned to Matcher:
	//
	//	*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch
	//	*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch
	Matcher       isMatcher_MatcherList_Predicate_SinglePredicate_Matcher `protobuf_oneof:"matcher"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) Reset() {
	*x = Matcher_MatcherList_Predicate_SinglePredicate{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_Predicate_SinglePredicate) ProtoMessage() {}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_Predicate_SinglePredicate.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_Predicate_SinglePredicate) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 0, 0}
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetInput() *v31.TypedExtensionConfig {
	if x != nil {
		return x.Input
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetMatcher() isMatcher_MatcherList_Predicate_SinglePredicate_Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetValueMatch() *v32.StringMatcher {
	if x != nil {
		if x, ok := x.Matcher.(*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch); ok {
			return x.ValueMatch
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetCustomMatch() *v31.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.Matcher.(*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch); ok {
			return x.CustomMatch
		}
	}
	return nil
}

type isMatcher_MatcherList_Predicate_SinglePredicate_Matcher interface {
	isMatcher_MatcherList_Predicate_SinglePredicate_Matcher()
}

type Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch struct {
	// Built-in string matcher.
	ValueMatch *v32.StringMatcher `protobuf:"bytes,2,opt,name=value_match,json=valueMatch,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch struct {
	// Extension for custom matching logic.
	// [#extension-category: envoy.matching.input_matchers]
	CustomMatch *v31.TypedExtensionConfig `protobuf:"bytes,3,opt,name=custom_match,json=customMatch,proto3,oneof"`
}

func (*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch) isMatcher_MatcherList_Predicate_SinglePredicate_Matcher() {
}

func (*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch) isMatcher_MatcherList_Predicate_SinglePredicate_Matcher() {
}

// A list of two or more matchers. Used to allow using a list within a oneof.
type Matcher_MatcherList_Predicate_PredicateList struct {
	state         protoimpl.MessageState           `protogen:"open.v1"`
	Predicate     []*Matcher_MatcherList_Predicate `protobuf:"bytes,1,rep,name=predicate,proto3" json:"predicate,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_Predicate_PredicateList) Reset() {
	*x = Matcher_MatcherList_Predicate_PredicateList{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_Predicate_PredicateList) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_Predicate_PredicateList) ProtoMessage() {}

func (x *Matcher_MatcherList_Predicate_PredicateList) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_Predicate_PredicateList.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_Predicate_PredicateList) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 0, 1}
}

func (x *Matcher_MatcherList_Predicate_PredicateList) GetPredicate() []*Matcher_MatcherList_Predicate {
	if x != nil {
		return x.Predicate
	}
	return nil
}

// A map of configured matchers. Used to allow using a map within a oneof.
type Matcher_MatcherTree_MatchMap struct {
	state         protoimpl.MessageState      `protogen:"open.v1"`
	Map           map[string]*Matcher_OnMatch `protobuf:"bytes,1,rep,name=map,proto3" json:"map,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherTree_MatchMap) Reset() {
	*x = Matcher_MatcherTree_MatchMap{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherTree_MatchMap) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherTree_MatchMap) ProtoMessage() {}

func (x *Matcher_MatcherTree_MatchMap) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherTree_MatchMap.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherTree_MatchMap) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 2, 0}
}

func (x *Matcher_MatcherTree_MatchMap) GetMap() map[string]*Matcher_OnMatch {
	if x != nil {
		return x.Map
	}
	return nil
}

// A set of match configurations used for logical operations.
type MatchPredicate_MatchSet struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The list of rules that make up the set.
	Rules         []*MatchPredicate `protobuf:"bytes,1,rep,name=rules,proto3" json:"rules,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MatchPredicate_MatchSet) Reset() {
	*x = MatchPredicate_MatchSet{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MatchPredicate_MatchSet) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MatchPredicate_MatchSet) ProtoMessage() {}

func (x *MatchPredicate_MatchSet) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MatchPredicate_MatchSet.ProtoReflect.Descriptor instead.
func (*MatchPredicate_MatchSet) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{1, 0}
}

func (x *MatchPredicate_MatchSet) GetRules() []*MatchPredicate {
	if x != nil {
		return x.Rules
	}
	return nil
}

type HttpGenericBodyMatch_GenericTextMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Rule:
	//
	//	*HttpGenericBodyMatch_GenericTextMatch_StringMatch
	//	*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch
	Rule          isHttpGenericBodyMatch_GenericTextMatch_Rule `protobuf_oneof:"rule"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpGenericBodyMatch_GenericTextMatch) Reset() {
	*x = HttpGenericBodyMatch_GenericTextMatch{}
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpGenericBodyMatch_GenericTextMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpGenericBodyMatch_GenericTextMatch) ProtoMessage() {}

func (x *HttpGenericBodyMatch_GenericTextMatch) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpGenericBodyMatch_GenericTextMatch.ProtoReflect.Descriptor instead.
func (*HttpGenericBodyMatch_GenericTextMatch) Descriptor() ([]byte, []int) {
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP(), []int{3, 0}
}

func (x *HttpGenericBodyMatch_GenericTextMatch) GetRule() isHttpGenericBodyMatch_GenericTextMatch_Rule {
	if x != nil {
		return x.Rule
	}
	return nil
}

func (x *HttpGenericBodyMatch_GenericTextMatch) GetStringMatch() string {
	if x != nil {
		if x, ok := x.Rule.(*HttpGenericBodyMatch_GenericTextMatch_StringMatch); ok {
			return x.StringMatch
		}
	}
	return ""
}

func (x *HttpGenericBodyMatch_GenericTextMatch) GetBinaryMatch() []byte {
	if x != nil {
		if x, ok := x.Rule.(*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch); ok {
			return x.BinaryMatch
		}
	}
	return nil
}

type isHttpGenericBodyMatch_GenericTextMatch_Rule interface {
	isHttpGenericBodyMatch_GenericTextMatch_Rule()
}

type HttpGenericBodyMatch_GenericTextMatch_StringMatch struct {
	// Text string to be located in HTTP body.
	StringMatch string `protobuf:"bytes,1,opt,name=string_match,json=stringMatch,proto3,oneof"`
}

type HttpGenericBodyMatch_GenericTextMatch_BinaryMatch struct {
	// Sequence of bytes to be located in HTTP body.
	BinaryMatch []byte `protobuf:"bytes,2,opt,name=binary_match,json=binaryMatch,proto3,oneof"`
}

func (*HttpGenericBodyMatch_GenericTextMatch_StringMatch) isHttpGenericBodyMatch_GenericTextMatch_Rule() {
}

func (*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch) isHttpGenericBodyMatch_GenericTextMatch_Rule() {
}

var File_envoy_config_common_matcher_v3_matcher_proto protoreflect.FileDescriptor

const file_envoy_config_common_matcher_v3_matcher_proto_rawDesc = "" +
	"\n" +
	",envoy/config/common/matcher/v3/matcher.proto\x12\x1eenvoy.config.common.matcher.v3\x1a$envoy/config/core/v3/extension.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xff\x11\n" +
	"\aMatcher\x12X\n" +
	"\fmatcher_list\x18\x01 \x01(\v23.envoy.config.common.matcher.v3.Matcher.MatcherListH\x00R\vmatcherList\x12X\n" +
	"\fmatcher_tree\x18\x02 \x01(\v23.envoy.config.common.matcher.v3.Matcher.MatcherTreeH\x00R\vmatcherTree\x12O\n" +
	"\von_no_match\x18\x03 \x01(\v2/.envoy.config.common.matcher.v3.Matcher.OnMatchR\tonNoMatch\x1a\xca\x01\n" +
	"\aOnMatch\x12C\n" +
	"\amatcher\x18\x01 \x01(\v2'.envoy.config.common.matcher.v3.MatcherH\x00R\amatcher\x12D\n" +
	"\x06action\x18\x02 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigH\x00R\x06action\x12#\n" +
	"\rkeep_matching\x18\x03 \x01(\bR\fkeepMatchingB\x0f\n" +
	"\bon_match\x12\x03\xf8B\x01\x1a\xa2\t\n" +
	"\vMatcherList\x12f\n" +
	"\bmatchers\x18\x01 \x03(\v2@.envoy.config.common.matcher.v3.Matcher.MatcherList.FieldMatcherB\b\xfaB\x05\x92\x01\x02\b\x01R\bmatchers\x1a\xdc\x06\n" +
	"\tPredicate\x12z\n" +
	"\x10single_predicate\x18\x01 \x01(\v2M.envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicateH\x00R\x0fsinglePredicate\x12l\n" +
	"\n" +
	"or_matcher\x18\x02 \x01(\v2K.envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.PredicateListH\x00R\torMatcher\x12n\n" +
	"\vand_matcher\x18\x03 \x01(\v2K.envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.PredicateListH\x00R\n" +
	"andMatcher\x12`\n" +
	"\vnot_matcher\x18\x04 \x01(\v2=.envoy.config.common.matcher.v3.Matcher.MatcherList.PredicateH\x00R\n" +
	"notMatcher\x1a\x87\x02\n" +
	"\x0fSinglePredicate\x12J\n" +
	"\x05input\x18\x01 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05input\x12G\n" +
	"\vvalue_match\x18\x02 \x01(\v2$.envoy.type.matcher.v3.StringMatcherH\x00R\n" +
	"valueMatch\x12O\n" +
	"\fcustom_match\x18\x03 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigH\x00R\vcustomMatchB\x0e\n" +
	"\amatcher\x12\x03\xf8B\x01\x1av\n" +
	"\rPredicateList\x12e\n" +
	"\tpredicate\x18\x01 \x03(\v2=.envoy.config.common.matcher.v3.Matcher.MatcherList.PredicateB\b\xfaB\x05\x92\x01\x02\b\x02R\tpredicateB\x11\n" +
	"\n" +
	"match_type\x12\x03\xf8B\x01\x1a\xcb\x01\n" +
	"\fFieldMatcher\x12e\n" +
	"\tpredicate\x18\x01 \x01(\v2=.envoy.config.common.matcher.v3.Matcher.MatcherList.PredicateB\b\xfaB\x05\x8a\x01\x02\x10\x01R\tpredicate\x12T\n" +
	"\bon_match\x18\x02 \x01(\v2/.envoy.config.common.matcher.v3.Matcher.OnMatchB\b\xfaB\x05\x8a\x01\x02\x10\x01R\aonMatch\x1a\xe7\x04\n" +
	"\vMatcherTree\x12J\n" +
	"\x05input\x18\x01 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05input\x12f\n" +
	"\x0fexact_match_map\x18\x02 \x01(\v2<.envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMapH\x00R\rexactMatchMap\x12h\n" +
	"\x10prefix_match_map\x18\x03 \x01(\v2<.envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMapH\x00R\x0eprefixMatchMap\x12O\n" +
	"\fcustom_match\x18\x04 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigH\x00R\vcustomMatch\x1a\xd6\x01\n" +
	"\bMatchMap\x12a\n" +
	"\x03map\x18\x01 \x03(\v2E.envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntryB\b\xfaB\x05\x9a\x01\x02\b\x01R\x03map\x1ag\n" +
	"\bMapEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12E\n" +
	"\x05value\x18\x02 \x01(\v2/.envoy.config.common.matcher.v3.Matcher.OnMatchR\x05value:\x028\x01B\x10\n" +
	"\ttree_type\x12\x03\xf8B\x01B\x13\n" +
	"\fmatcher_type\x12\x03\xf8B\x01\"\xe8\b\n" +
	"\x0eMatchPredicate\x12T\n" +
	"\bor_match\x18\x01 \x01(\v27.envoy.config.common.matcher.v3.MatchPredicate.MatchSetH\x00R\aorMatch\x12V\n" +
	"\tand_match\x18\x02 \x01(\v27.envoy.config.common.matcher.v3.MatchPredicate.MatchSetH\x00R\bandMatch\x12M\n" +
	"\tnot_match\x18\x03 \x01(\v2..envoy.config.common.matcher.v3.MatchPredicateH\x00R\bnotMatch\x12&\n" +
	"\tany_match\x18\x04 \x01(\bB\a\xfaB\x04j\x02\b\x01H\x00R\banyMatch\x12o\n" +
	"\x1ahttp_request_headers_match\x18\x05 \x01(\v20.envoy.config.common.matcher.v3.HttpHeadersMatchH\x00R\x17httpRequestHeadersMatch\x12q\n" +
	"\x1bhttp_request_trailers_match\x18\x06 \x01(\v20.envoy.config.common.matcher.v3.HttpHeadersMatchH\x00R\x18httpRequestTrailersMatch\x12q\n" +
	"\x1bhttp_response_headers_match\x18\a \x01(\v20.envoy.config.common.matcher.v3.HttpHeadersMatchH\x00R\x18httpResponseHeadersMatch\x12s\n" +
	"\x1chttp_response_trailers_match\x18\b \x01(\v20.envoy.config.common.matcher.v3.HttpHeadersMatchH\x00R\x19httpResponseTrailersMatch\x12|\n" +
	"\x1fhttp_request_generic_body_match\x18\t \x01(\v24.envoy.config.common.matcher.v3.HttpGenericBodyMatchH\x00R\x1bhttpRequestGenericBodyMatch\x12~\n" +
	" http_response_generic_body_match\x18\n" +
	" \x01(\v24.envoy.config.common.matcher.v3.HttpGenericBodyMatchH\x00R\x1chttpResponseGenericBodyMatch\x1aZ\n" +
	"\bMatchSet\x12N\n" +
	"\x05rules\x18\x01 \x03(\v2..envoy.config.common.matcher.v3.MatchPredicateB\b\xfaB\x05\x92\x01\x02\b\x02R\x05rulesB\v\n" +
	"\x04rule\x12\x03\xf8B\x01\"R\n" +
	"\x10HttpHeadersMatch\x12>\n" +
	"\aheaders\x18\x01 \x03(\v2$.envoy.config.route.v3.HeaderMatcherR\aheaders\"\xa1\x02\n" +
	"\x14HttpGenericBodyMatch\x12\x1f\n" +
	"\vbytes_limit\x18\x01 \x01(\rR\n" +
	"bytesLimit\x12k\n" +
	"\bpatterns\x18\x02 \x03(\v2E.envoy.config.common.matcher.v3.HttpGenericBodyMatch.GenericTextMatchB\b\xfaB\x05\x92\x01\x02\b\x01R\bpatterns\x1a{\n" +
	"\x10GenericTextMatch\x12,\n" +
	"\fstring_match\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\vstringMatch\x12,\n" +
	"\fbinary_match\x18\x02 \x01(\fB\a\xfaB\x04z\x02\x10\x01H\x00R\vbinaryMatchB\v\n" +
	"\x04rule\x12\x03\xf8B\x01B\x97\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	",io.envoyproxy.envoy.config.common.matcher.v3B\fMatcherProtoP\x01ZOgithub.com/envoyproxy/go-control-plane/envoy/config/common/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_config_common_matcher_v3_matcher_proto_rawDescOnce sync.Once
	file_envoy_config_common_matcher_v3_matcher_proto_rawDescData []byte
)

func file_envoy_config_common_matcher_v3_matcher_proto_rawDescGZIP() []byte {
	file_envoy_config_common_matcher_v3_matcher_proto_rawDescOnce.Do(func() {
		file_envoy_config_common_matcher_v3_matcher_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_common_matcher_v3_matcher_proto_rawDesc), len(file_envoy_config_common_matcher_v3_matcher_proto_rawDesc)))
	})
	return file_envoy_config_common_matcher_v3_matcher_proto_rawDescData
}

var file_envoy_config_common_matcher_v3_matcher_proto_msgTypes = make([]protoimpl.MessageInfo, 15)
var file_envoy_config_common_matcher_v3_matcher_proto_goTypes = []any{
	(*Matcher)(nil),                                       // 0: envoy.config.common.matcher.v3.Matcher
	(*MatchPredicate)(nil),                                // 1: envoy.config.common.matcher.v3.MatchPredicate
	(*HttpHeadersMatch)(nil),                              // 2: envoy.config.common.matcher.v3.HttpHeadersMatch
	(*HttpGenericBodyMatch)(nil),                          // 3: envoy.config.common.matcher.v3.HttpGenericBodyMatch
	(*Matcher_OnMatch)(nil),                               // 4: envoy.config.common.matcher.v3.Matcher.OnMatch
	(*Matcher_MatcherList)(nil),                           // 5: envoy.config.common.matcher.v3.Matcher.MatcherList
	(*Matcher_MatcherTree)(nil),                           // 6: envoy.config.common.matcher.v3.Matcher.MatcherTree
	(*Matcher_MatcherList_Predicate)(nil),                 // 7: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate
	(*Matcher_MatcherList_FieldMatcher)(nil),              // 8: envoy.config.common.matcher.v3.Matcher.MatcherList.FieldMatcher
	(*Matcher_MatcherList_Predicate_SinglePredicate)(nil), // 9: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate
	(*Matcher_MatcherList_Predicate_PredicateList)(nil),   // 10: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.PredicateList
	(*Matcher_MatcherTree_MatchMap)(nil),                  // 11: envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap
	nil,                                                   // 12: envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry
	(*MatchPredicate_MatchSet)(nil),                       // 13: envoy.config.common.matcher.v3.MatchPredicate.MatchSet
	(*HttpGenericBodyMatch_GenericTextMatch)(nil),         // 14: envoy.config.common.matcher.v3.HttpGenericBodyMatch.GenericTextMatch
	(*v3.HeaderMatcher)(nil),                              // 15: envoy.config.route.v3.HeaderMatcher
	(*v31.TypedExtensionConfig)(nil),                      // 16: envoy.config.core.v3.TypedExtensionConfig
	(*v32.StringMatcher)(nil),                             // 17: envoy.type.matcher.v3.StringMatcher
}
var file_envoy_config_common_matcher_v3_matcher_proto_depIdxs = []int32{
	5,  // 0: envoy.config.common.matcher.v3.Matcher.matcher_list:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList
	6,  // 1: envoy.config.common.matcher.v3.Matcher.matcher_tree:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherTree
	4,  // 2: envoy.config.common.matcher.v3.Matcher.on_no_match:type_name -> envoy.config.common.matcher.v3.Matcher.OnMatch
	13, // 3: envoy.config.common.matcher.v3.MatchPredicate.or_match:type_name -> envoy.config.common.matcher.v3.MatchPredicate.MatchSet
	13, // 4: envoy.config.common.matcher.v3.MatchPredicate.and_match:type_name -> envoy.config.common.matcher.v3.MatchPredicate.MatchSet
	1,  // 5: envoy.config.common.matcher.v3.MatchPredicate.not_match:type_name -> envoy.config.common.matcher.v3.MatchPredicate
	2,  // 6: envoy.config.common.matcher.v3.MatchPredicate.http_request_headers_match:type_name -> envoy.config.common.matcher.v3.HttpHeadersMatch
	2,  // 7: envoy.config.common.matcher.v3.MatchPredicate.http_request_trailers_match:type_name -> envoy.config.common.matcher.v3.HttpHeadersMatch
	2,  // 8: envoy.config.common.matcher.v3.MatchPredicate.http_response_headers_match:type_name -> envoy.config.common.matcher.v3.HttpHeadersMatch
	2,  // 9: envoy.config.common.matcher.v3.MatchPredicate.http_response_trailers_match:type_name -> envoy.config.common.matcher.v3.HttpHeadersMatch
	3,  // 10: envoy.config.common.matcher.v3.MatchPredicate.http_request_generic_body_match:type_name -> envoy.config.common.matcher.v3.HttpGenericBodyMatch
	3,  // 11: envoy.config.common.matcher.v3.MatchPredicate.http_response_generic_body_match:type_name -> envoy.config.common.matcher.v3.HttpGenericBodyMatch
	15, // 12: envoy.config.common.matcher.v3.HttpHeadersMatch.headers:type_name -> envoy.config.route.v3.HeaderMatcher
	14, // 13: envoy.config.common.matcher.v3.HttpGenericBodyMatch.patterns:type_name -> envoy.config.common.matcher.v3.HttpGenericBodyMatch.GenericTextMatch
	0,  // 14: envoy.config.common.matcher.v3.Matcher.OnMatch.matcher:type_name -> envoy.config.common.matcher.v3.Matcher
	16, // 15: envoy.config.common.matcher.v3.Matcher.OnMatch.action:type_name -> envoy.config.core.v3.TypedExtensionConfig
	8,  // 16: envoy.config.common.matcher.v3.Matcher.MatcherList.matchers:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.FieldMatcher
	16, // 17: envoy.config.common.matcher.v3.Matcher.MatcherTree.input:type_name -> envoy.config.core.v3.TypedExtensionConfig
	11, // 18: envoy.config.common.matcher.v3.Matcher.MatcherTree.exact_match_map:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap
	11, // 19: envoy.config.common.matcher.v3.Matcher.MatcherTree.prefix_match_map:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap
	16, // 20: envoy.config.common.matcher.v3.Matcher.MatcherTree.custom_match:type_name -> envoy.config.core.v3.TypedExtensionConfig
	9,  // 21: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.single_predicate:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate
	10, // 22: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.or_matcher:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.PredicateList
	10, // 23: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.and_matcher:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.PredicateList
	7,  // 24: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.not_matcher:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate
	7,  // 25: envoy.config.common.matcher.v3.Matcher.MatcherList.FieldMatcher.predicate:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate
	4,  // 26: envoy.config.common.matcher.v3.Matcher.MatcherList.FieldMatcher.on_match:type_name -> envoy.config.common.matcher.v3.Matcher.OnMatch
	16, // 27: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate.input:type_name -> envoy.config.core.v3.TypedExtensionConfig
	17, // 28: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate.value_match:type_name -> envoy.type.matcher.v3.StringMatcher
	16, // 29: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate.custom_match:type_name -> envoy.config.core.v3.TypedExtensionConfig
	7,  // 30: envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate.PredicateList.predicate:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherList.Predicate
	12, // 31: envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap.map:type_name -> envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry
	4,  // 32: envoy.config.common.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry.value:type_name -> envoy.config.common.matcher.v3.Matcher.OnMatch
	1,  // 33: envoy.config.common.matcher.v3.MatchPredicate.MatchSet.rules:type_name -> envoy.config.common.matcher.v3.MatchPredicate
	34, // [34:34] is the sub-list for method output_type
	34, // [34:34] is the sub-list for method input_type
	34, // [34:34] is the sub-list for extension type_name
	34, // [34:34] is the sub-list for extension extendee
	0,  // [0:34] is the sub-list for field type_name
}

func init() { file_envoy_config_common_matcher_v3_matcher_proto_init() }
func file_envoy_config_common_matcher_v3_matcher_proto_init() {
	if File_envoy_config_common_matcher_v3_matcher_proto != nil {
		return
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[0].OneofWrappers = []any{
		(*Matcher_MatcherList_)(nil),
		(*Matcher_MatcherTree_)(nil),
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[1].OneofWrappers = []any{
		(*MatchPredicate_OrMatch)(nil),
		(*MatchPredicate_AndMatch)(nil),
		(*MatchPredicate_NotMatch)(nil),
		(*MatchPredicate_AnyMatch)(nil),
		(*MatchPredicate_HttpRequestHeadersMatch)(nil),
		(*MatchPredicate_HttpRequestTrailersMatch)(nil),
		(*MatchPredicate_HttpResponseHeadersMatch)(nil),
		(*MatchPredicate_HttpResponseTrailersMatch)(nil),
		(*MatchPredicate_HttpRequestGenericBodyMatch)(nil),
		(*MatchPredicate_HttpResponseGenericBodyMatch)(nil),
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[4].OneofWrappers = []any{
		(*Matcher_OnMatch_Matcher)(nil),
		(*Matcher_OnMatch_Action)(nil),
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[6].OneofWrappers = []any{
		(*Matcher_MatcherTree_ExactMatchMap)(nil),
		(*Matcher_MatcherTree_PrefixMatchMap)(nil),
		(*Matcher_MatcherTree_CustomMatch)(nil),
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[7].OneofWrappers = []any{
		(*Matcher_MatcherList_Predicate_SinglePredicate_)(nil),
		(*Matcher_MatcherList_Predicate_OrMatcher)(nil),
		(*Matcher_MatcherList_Predicate_AndMatcher)(nil),
		(*Matcher_MatcherList_Predicate_NotMatcher)(nil),
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[9].OneofWrappers = []any{
		(*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch)(nil),
		(*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch)(nil),
	}
	file_envoy_config_common_matcher_v3_matcher_proto_msgTypes[14].OneofWrappers = []any{
		(*HttpGenericBodyMatch_GenericTextMatch_StringMatch)(nil),
		(*HttpGenericBodyMatch_GenericTextMatch_BinaryMatch)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_common_matcher_v3_matcher_proto_rawDesc), len(file_envoy_config_common_matcher_v3_matcher_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   15,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_common_matcher_v3_matcher_proto_goTypes,
		DependencyIndexes: file_envoy_config_common_matcher_v3_matcher_proto_depIdxs,
		MessageInfos:      file_envoy_config_common_matcher_v3_matcher_proto_msgTypes,
	}.Build()
	File_envoy_config_common_matcher_v3_matcher_proto = out.File
	file_envoy_config_common_matcher_v3_matcher_proto_goTypes = nil
	file_envoy_config_common_matcher_v3_matcher_proto_depIdxs = nil
}
