// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

package nethelpers

// RoutingTable is a routing table ID.
type RoutingTable uint32

// RoutingTable constants.
//
//structprotogen:gen_enum
const (
	TableUnspec  RoutingTable = 0   // unspec
	Table1       RoutingTable = 1   // 1
	Table2       RoutingTable = 2   // 2
	Table3       RoutingTable = 3   // 3
	Table4       RoutingTable = 4   // 4
	Table5       RoutingTable = 5   // 5
	Table6       RoutingTable = 6   // 6
	Table7       RoutingTable = 7   // 7
	Table8       RoutingTable = 8   // 8
	Table9       RoutingTable = 9   // 9
	Table10      RoutingTable = 10  // 10
	Table11      RoutingTable = 11  // 11
	Table12      RoutingTable = 12  // 12
	Table13      RoutingTable = 13  // 13
	Table14      RoutingTable = 14  // 14
	Table15      RoutingTable = 15  // 15
	Table16      RoutingTable = 16  // 16
	Table17      RoutingTable = 17  // 17
	Table18      RoutingTable = 18  // 18
	Table19      RoutingTable = 19  // 19
	Table20      RoutingTable = 20  // 20
	Table21      RoutingTable = 21  // 21
	Table22      RoutingTable = 22  // 22
	Table23      RoutingTable = 23  // 23
	Table24      RoutingTable = 24  // 24
	Table25      RoutingTable = 25  // 25
	Table26      RoutingTable = 26  // 26
	Table27      RoutingTable = 27  // 27
	Table28      RoutingTable = 28  // 28
	Table29      RoutingTable = 29  // 29
	Table30      RoutingTable = 30  // 30
	Table31      RoutingTable = 31  // 31
	Table32      RoutingTable = 32  // 32
	Table33      RoutingTable = 33  // 33
	Table34      RoutingTable = 34  // 34
	Table35      RoutingTable = 35  // 35
	Table36      RoutingTable = 36  // 36
	Table37      RoutingTable = 37  // 37
	Table38      RoutingTable = 38  // 38
	Table39      RoutingTable = 39  // 39
	Table40      RoutingTable = 40  // 40
	Table41      RoutingTable = 41  // 41
	Table42      RoutingTable = 42  // 42
	Table43      RoutingTable = 43  // 43
	Table44      RoutingTable = 44  // 44
	Table45      RoutingTable = 45  // 45
	Table46      RoutingTable = 46  // 46
	Table47      RoutingTable = 47  // 47
	Table48      RoutingTable = 48  // 48
	Table49      RoutingTable = 49  // 49
	Table50      RoutingTable = 50  // 50
	Table51      RoutingTable = 51  // 51
	Table52      RoutingTable = 52  // 52
	Table53      RoutingTable = 53  // 53
	Table54      RoutingTable = 54  // 54
	Table55      RoutingTable = 55  // 55
	Table56      RoutingTable = 56  // 56
	Table57      RoutingTable = 57  // 57
	Table58      RoutingTable = 58  // 58
	Table59      RoutingTable = 59  // 59
	Table60      RoutingTable = 60  // 60
	Table61      RoutingTable = 61  // 61
	Table62      RoutingTable = 62  // 62
	Table63      RoutingTable = 63  // 63
	Table64      RoutingTable = 64  // 64
	Table65      RoutingTable = 65  // 65
	Table66      RoutingTable = 66  // 66
	Table67      RoutingTable = 67  // 67
	Table68      RoutingTable = 68  // 68
	Table69      RoutingTable = 69  // 69
	Table70      RoutingTable = 70  // 70
	Table71      RoutingTable = 71  // 71
	Table72      RoutingTable = 72  // 72
	Table73      RoutingTable = 73  // 73
	Table74      RoutingTable = 74  // 74
	Table75      RoutingTable = 75  // 75
	Table76      RoutingTable = 76  // 76
	Table77      RoutingTable = 77  // 77
	Table78      RoutingTable = 78  // 78
	Table79      RoutingTable = 79  // 79
	Table80      RoutingTable = 80  // 80
	Table81      RoutingTable = 81  // 81
	Table82      RoutingTable = 82  // 82
	Table83      RoutingTable = 83  // 83
	Table84      RoutingTable = 84  // 84
	Table85      RoutingTable = 85  // 85
	Table86      RoutingTable = 86  // 86
	Table87      RoutingTable = 87  // 87
	Table88      RoutingTable = 88  // 88
	Table89      RoutingTable = 89  // 89
	Table90      RoutingTable = 90  // 90
	Table91      RoutingTable = 91  // 91
	Table92      RoutingTable = 92  // 92
	Table93      RoutingTable = 93  // 93
	Table94      RoutingTable = 94  // 94
	Table95      RoutingTable = 95  // 95
	Table96      RoutingTable = 96  // 96
	Table97      RoutingTable = 97  // 97
	Table98      RoutingTable = 98  // 98
	Table99      RoutingTable = 99  // 99
	Table100     RoutingTable = 100 // 100
	Table101     RoutingTable = 101 // 101
	Table102     RoutingTable = 102 // 102
	Table103     RoutingTable = 103 // 103
	Table104     RoutingTable = 104 // 104
	Table105     RoutingTable = 105 // 105
	Table106     RoutingTable = 106 // 106
	Table107     RoutingTable = 107 // 107
	Table108     RoutingTable = 108 // 108
	Table109     RoutingTable = 109 // 109
	Table110     RoutingTable = 110 // 110
	Table111     RoutingTable = 111 // 111
	Table112     RoutingTable = 112 // 112
	Table113     RoutingTable = 113 // 113
	Table114     RoutingTable = 114 // 114
	Table115     RoutingTable = 115 // 115
	Table116     RoutingTable = 116 // 116
	Table117     RoutingTable = 117 // 117
	Table118     RoutingTable = 118 // 118
	Table119     RoutingTable = 119 // 119
	Table120     RoutingTable = 120 // 120
	Table121     RoutingTable = 121 // 121
	Table122     RoutingTable = 122 // 122
	Table123     RoutingTable = 123 // 123
	Table124     RoutingTable = 124 // 124
	Table125     RoutingTable = 125 // 125
	Table126     RoutingTable = 126 // 126
	Table127     RoutingTable = 127 // 127
	Table128     RoutingTable = 128 // 128
	Table129     RoutingTable = 129 // 129
	Table130     RoutingTable = 130 // 130
	Table131     RoutingTable = 131 // 131
	Table132     RoutingTable = 132 // 132
	Table133     RoutingTable = 133 // 133
	Table134     RoutingTable = 134 // 134
	Table135     RoutingTable = 135 // 135
	Table136     RoutingTable = 136 // 136
	Table137     RoutingTable = 137 // 137
	Table138     RoutingTable = 138 // 138
	Table139     RoutingTable = 139 // 139
	Table140     RoutingTable = 140 // 140
	Table141     RoutingTable = 141 // 141
	Table142     RoutingTable = 142 // 142
	Table143     RoutingTable = 143 // 143
	Table144     RoutingTable = 144 // 144
	Table145     RoutingTable = 145 // 145
	Table146     RoutingTable = 146 // 146
	Table147     RoutingTable = 147 // 147
	Table148     RoutingTable = 148 // 148
	Table149     RoutingTable = 149 // 149
	Table150     RoutingTable = 150 // 150
	Table151     RoutingTable = 151 // 151
	Table152     RoutingTable = 152 // 152
	Table153     RoutingTable = 153 // 153
	Table154     RoutingTable = 154 // 154
	Table155     RoutingTable = 155 // 155
	Table156     RoutingTable = 156 // 156
	Table157     RoutingTable = 157 // 157
	Table158     RoutingTable = 158 // 158
	Table159     RoutingTable = 159 // 159
	Table160     RoutingTable = 160 // 160
	Table161     RoutingTable = 161 // 161
	Table162     RoutingTable = 162 // 162
	Table163     RoutingTable = 163 // 163
	Table164     RoutingTable = 164 // 164
	Table165     RoutingTable = 165 // 165
	Table166     RoutingTable = 166 // 166
	Table167     RoutingTable = 167 // 167
	Table168     RoutingTable = 168 // 168
	Table169     RoutingTable = 169 // 169
	Table170     RoutingTable = 170 // 170
	Table171     RoutingTable = 171 // 171
	Table172     RoutingTable = 172 // 172
	Table173     RoutingTable = 173 // 173
	Table174     RoutingTable = 174 // 174
	Table175     RoutingTable = 175 // 175
	Table176     RoutingTable = 176 // 176
	Table177     RoutingTable = 177 // 177
	Table178     RoutingTable = 178 // 178
	Table179     RoutingTable = 179 // 179
	Table180     RoutingTable = 180 // 180
	Table181     RoutingTable = 181 // 181
	Table182     RoutingTable = 182 // 182
	Table183     RoutingTable = 183 // 183
	Table184     RoutingTable = 184 // 184
	Table185     RoutingTable = 185 // 185
	Table186     RoutingTable = 186 // 186
	Table187     RoutingTable = 187 // 187
	Table188     RoutingTable = 188 // 188
	Table189     RoutingTable = 189 // 189
	Table190     RoutingTable = 190 // 190
	Table191     RoutingTable = 191 // 191
	Table192     RoutingTable = 192 // 192
	Table193     RoutingTable = 193 // 193
	Table194     RoutingTable = 194 // 194
	Table195     RoutingTable = 195 // 195
	Table196     RoutingTable = 196 // 196
	Table197     RoutingTable = 197 // 197
	Table198     RoutingTable = 198 // 198
	Table199     RoutingTable = 199 // 199
	Table200     RoutingTable = 200 // 200
	Table201     RoutingTable = 201 // 201
	Table202     RoutingTable = 202 // 202
	Table203     RoutingTable = 203 // 203
	Table204     RoutingTable = 204 // 204
	Table205     RoutingTable = 205 // 205
	Table206     RoutingTable = 206 // 206
	Table207     RoutingTable = 207 // 207
	Table208     RoutingTable = 208 // 208
	Table209     RoutingTable = 209 // 209
	Table210     RoutingTable = 210 // 210
	Table211     RoutingTable = 211 // 211
	Table212     RoutingTable = 212 // 212
	Table213     RoutingTable = 213 // 213
	Table214     RoutingTable = 214 // 214
	Table215     RoutingTable = 215 // 215
	Table216     RoutingTable = 216 // 216
	Table217     RoutingTable = 217 // 217
	Table218     RoutingTable = 218 // 218
	Table219     RoutingTable = 219 // 219
	Table220     RoutingTable = 220 // 220
	Table221     RoutingTable = 221 // 221
	Table222     RoutingTable = 222 // 222
	Table223     RoutingTable = 223 // 223
	Table224     RoutingTable = 224 // 224
	Table225     RoutingTable = 225 // 225
	Table226     RoutingTable = 226 // 226
	Table227     RoutingTable = 227 // 227
	Table228     RoutingTable = 228 // 228
	Table229     RoutingTable = 229 // 229
	Table230     RoutingTable = 230 // 230
	Table231     RoutingTable = 231 // 231
	Table232     RoutingTable = 232 // 232
	Table233     RoutingTable = 233 // 233
	Table234     RoutingTable = 234 // 234
	Table235     RoutingTable = 235 // 235
	Table236     RoutingTable = 236 // 236
	Table237     RoutingTable = 237 // 237
	Table238     RoutingTable = 238 // 238
	Table239     RoutingTable = 239 // 239
	Table240     RoutingTable = 240 // 240
	Table241     RoutingTable = 241 // 241
	Table242     RoutingTable = 242 // 242
	Table243     RoutingTable = 243 // 243
	Table244     RoutingTable = 244 // 244
	Table245     RoutingTable = 245 // 245
	Table246     RoutingTable = 246 // 246
	Table247     RoutingTable = 247 // 247
	Table248     RoutingTable = 248 // 248
	Table249     RoutingTable = 249 // 249
	Table250     RoutingTable = 250 // 250
	Table251     RoutingTable = 251 // 251
	Table252     RoutingTable = 252 // 252
	TableDefault RoutingTable = 253 // default
	TableMain    RoutingTable = 254 // main
	TableLocal   RoutingTable = 255 // local
)
