// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/admin/v3/metrics.proto

package adminv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type SimpleMetric_Type int32

const (
	SimpleMetric_COUNTER SimpleMetric_Type = 0
	SimpleMetric_GAUGE   SimpleMetric_Type = 1
)

// Enum value maps for SimpleMetric_Type.
var (
	SimpleMetric_Type_name = map[int32]string{
		0: "COUNTER",
		1: "GAUGE",
	}
	SimpleMetric_Type_value = map[string]int32{
		"COUNTER": 0,
		"GAUGE":   1,
	}
)

func (x SimpleMetric_Type) Enum() *SimpleMetric_Type {
	p := new(SimpleMetric_Type)
	*p = x
	return p
}

func (x SimpleMetric_Type) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (SimpleMetric_Type) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_admin_v3_metrics_proto_enumTypes[0].Descriptor()
}

func (SimpleMetric_Type) Type() protoreflect.EnumType {
	return &file_envoy_admin_v3_metrics_proto_enumTypes[0]
}

func (x SimpleMetric_Type) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use SimpleMetric_Type.Descriptor instead.
func (SimpleMetric_Type) EnumDescriptor() ([]byte, []int) {
	return file_envoy_admin_v3_metrics_proto_rawDescGZIP(), []int{0, 0}
}

// Proto representation of an Envoy Counter or Gauge value.
type SimpleMetric struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Type of the metric represented.
	Type SimpleMetric_Type `protobuf:"varint,1,opt,name=type,proto3,enum=envoy.admin.v3.SimpleMetric_Type" json:"type,omitempty"`
	// Current metric value.
	Value uint64 `protobuf:"varint,2,opt,name=value,proto3" json:"value,omitempty"`
	// Name of the metric.
	Name          string `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SimpleMetric) Reset() {
	*x = SimpleMetric{}
	mi := &file_envoy_admin_v3_metrics_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SimpleMetric) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SimpleMetric) ProtoMessage() {}

func (x *SimpleMetric) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_admin_v3_metrics_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SimpleMetric.ProtoReflect.Descriptor instead.
func (*SimpleMetric) Descriptor() ([]byte, []int) {
	return file_envoy_admin_v3_metrics_proto_rawDescGZIP(), []int{0}
}

func (x *SimpleMetric) GetType() SimpleMetric_Type {
	if x != nil {
		return x.Type
	}
	return SimpleMetric_COUNTER
}

func (x *SimpleMetric) GetValue() uint64 {
	if x != nil {
		return x.Value
	}
	return 0
}

func (x *SimpleMetric) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

var File_envoy_admin_v3_metrics_proto protoreflect.FileDescriptor

const file_envoy_admin_v3_metrics_proto_rawDesc = "" +
	"\n" +
	"\x1cenvoy/admin/v3/metrics.proto\x12\x0eenvoy.admin.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xb8\x01\n" +
	"\fSimpleMetric\x125\n" +
	"\x04type\x18\x01 \x01(\x0e2!.envoy.admin.v3.SimpleMetric.TypeR\x04type\x12\x14\n" +
	"\x05value\x18\x02 \x01(\x04R\x05value\x12\x12\n" +
	"\x04name\x18\x03 \x01(\tR\x04name\"\x1e\n" +
	"\x04Type\x12\v\n" +
	"\aCOUNTER\x10\x00\x12\t\n" +
	"\x05GAUGE\x10\x01:'\x9aň\x1e\"\n" +
	" envoy.admin.v2alpha.SimpleMetricBu\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"\x1cio.envoyproxy.envoy.admin.v3B\fMetricsProtoP\x01Z=github.com/envoyproxy/go-control-plane/envoy/admin/v3;adminv3b\x06proto3"

var (
	file_envoy_admin_v3_metrics_proto_rawDescOnce sync.Once
	file_envoy_admin_v3_metrics_proto_rawDescData []byte
)

func file_envoy_admin_v3_metrics_proto_rawDescGZIP() []byte {
	file_envoy_admin_v3_metrics_proto_rawDescOnce.Do(func() {
		file_envoy_admin_v3_metrics_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_metrics_proto_rawDesc), len(file_envoy_admin_v3_metrics_proto_rawDesc)))
	})
	return file_envoy_admin_v3_metrics_proto_rawDescData
}

var file_envoy_admin_v3_metrics_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_admin_v3_metrics_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_admin_v3_metrics_proto_goTypes = []any{
	(SimpleMetric_Type)(0), // 0: envoy.admin.v3.SimpleMetric.Type
	(*SimpleMetric)(nil),   // 1: envoy.admin.v3.SimpleMetric
}
var file_envoy_admin_v3_metrics_proto_depIdxs = []int32{
	0, // 0: envoy.admin.v3.SimpleMetric.type:type_name -> envoy.admin.v3.SimpleMetric.Type
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_admin_v3_metrics_proto_init() }
func file_envoy_admin_v3_metrics_proto_init() {
	if File_envoy_admin_v3_metrics_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_admin_v3_metrics_proto_rawDesc), len(file_envoy_admin_v3_metrics_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_admin_v3_metrics_proto_goTypes,
		DependencyIndexes: file_envoy_admin_v3_metrics_proto_depIdxs,
		EnumInfos:         file_envoy_admin_v3_metrics_proto_enumTypes,
		MessageInfos:      file_envoy_admin_v3_metrics_proto_msgTypes,
	}.Build()
	File_envoy_admin_v3_metrics_proto = out.File
	file_envoy_admin_v3_metrics_proto_goTypes = nil
	file_envoy_admin_v3_metrics_proto_depIdxs = nil
}
