package ignore

// WARNING: Auto generated version file. Do not edit this file by hand.
// WARNING: go get github.com/sabhiram/gover to manage this file.
// Version: 1.1.0
const (
	Major = 1
	Minor = 1
	Patch = 0

	Version = "1.1.0"
)
