//go:build linux
// +build linux

// Code generated by "go run mklinkmodes.go"; DO NOT EDIT.

package ethtool

import "golang.org/x/sys/unix"

var linkModes = [...]struct {
	bit uint32
	str string
}{
	{bit: unix.ETHTOOL_LINK_MODE_10baseT_Half_BIT, str: "10baseT/Half"},
	{bit: unix.ETHTOOL_LINK_MODE_10baseT_Full_BIT, str: "10baseT/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100baseT_Half_BIT, str: "100baseT/Half"},
	{bit: unix.ETHTOOL_LINK_MODE_100baseT_Full_BIT, str: "100baseT/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_1000baseT_Half_BIT, str: "1000baseT/Half"},
	{bit: unix.ETHTOOL_LINK_MODE_1000baseT_Full_BIT, str: "1000baseT/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_Autoneg_BIT, str: "Autoneg"},
	{bit: unix.ETHTOOL_LINK_MODE_TP_BIT, str: "TP"},
	{bit: unix.ETHTOOL_LINK_MODE_AUI_BIT, str: "AUI"},
	{bit: unix.ETHTOOL_LINK_MODE_MII_BIT, str: "MII"},
	{bit: unix.ETHTOOL_LINK_MODE_FIBRE_BIT, str: "FIBRE"},
	{bit: unix.ETHTOOL_LINK_MODE_BNC_BIT, str: "BNC"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseT_Full_BIT, str: "10000baseT/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_Pause_BIT, str: "Pause"},
	{bit: unix.ETHTOOL_LINK_MODE_Asym_Pause_BIT, str: "Asym/Pause"},
	{bit: unix.ETHTOOL_LINK_MODE_2500baseX_Full_BIT, str: "2500baseX/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_Backplane_BIT, str: "Backplane"},
	{bit: unix.ETHTOOL_LINK_MODE_1000baseKX_Full_BIT, str: "1000baseKX/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseKX4_Full_BIT, str: "10000baseKX4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseKR_Full_BIT, str: "10000baseKR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseR_FEC_BIT, str: "10000baseR/FEC"},
	{bit: unix.ETHTOOL_LINK_MODE_20000baseMLD2_Full_BIT, str: "20000baseMLD2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_20000baseKR2_Full_BIT, str: "20000baseKR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_40000baseKR4_Full_BIT, str: "40000baseKR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_40000baseCR4_Full_BIT, str: "40000baseCR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_40000baseSR4_Full_BIT, str: "40000baseSR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_40000baseLR4_Full_BIT, str: "40000baseLR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_56000baseKR4_Full_BIT, str: "56000baseKR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_56000baseCR4_Full_BIT, str: "56000baseCR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_56000baseSR4_Full_BIT, str: "56000baseSR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_56000baseLR4_Full_BIT, str: "56000baseLR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_25000baseCR_Full_BIT, str: "25000baseCR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_25000baseKR_Full_BIT, str: "25000baseKR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_25000baseSR_Full_BIT, str: "25000baseSR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseCR2_Full_BIT, str: "50000baseCR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseKR2_Full_BIT, str: "50000baseKR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseKR4_Full_BIT, str: "100000baseKR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseSR4_Full_BIT, str: "100000baseSR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseCR4_Full_BIT, str: "100000baseCR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseLR4_ER4_Full_BIT, str: "100000baseLR4/ER4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseSR2_Full_BIT, str: "50000baseSR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_1000baseX_Full_BIT, str: "1000baseX/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseCR_Full_BIT, str: "10000baseCR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseSR_Full_BIT, str: "10000baseSR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseLR_Full_BIT, str: "10000baseLR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseLRM_Full_BIT, str: "10000baseLRM/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_10000baseER_Full_BIT, str: "10000baseER/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_2500baseT_Full_BIT, str: "2500baseT/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_5000baseT_Full_BIT, str: "5000baseT/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_FEC_NONE_BIT, str: "FEC/NONE"},
	{bit: unix.ETHTOOL_LINK_MODE_FEC_RS_BIT, str: "FEC/RS"},
	{bit: unix.ETHTOOL_LINK_MODE_FEC_BASER_BIT, str: "FEC/BASER"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseKR_Full_BIT, str: "50000baseKR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseSR_Full_BIT, str: "50000baseSR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseCR_Full_BIT, str: "50000baseCR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseLR_ER_FR_Full_BIT, str: "50000baseLR/ER/FR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_50000baseDR_Full_BIT, str: "50000baseDR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseKR2_Full_BIT, str: "100000baseKR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseSR2_Full_BIT, str: "100000baseSR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseCR2_Full_BIT, str: "100000baseCR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseLR2_ER2_FR2_Full_BIT, str: "100000baseLR2/ER2/FR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseDR2_Full_BIT, str: "100000baseDR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseKR4_Full_BIT, str: "200000baseKR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseSR4_Full_BIT, str: "200000baseSR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseLR4_ER4_FR4_Full_BIT, str: "200000baseLR4/ER4/FR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseDR4_Full_BIT, str: "200000baseDR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseCR4_Full_BIT, str: "200000baseCR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100baseT1_Full_BIT, str: "100baseT1/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_1000baseT1_Full_BIT, str: "1000baseT1/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseKR8_Full_BIT, str: "400000baseKR8/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseSR8_Full_BIT, str: "400000baseSR8/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseLR8_ER8_FR8_Full_BIT, str: "400000baseLR8/ER8/FR8/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseDR8_Full_BIT, str: "400000baseDR8/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseCR8_Full_BIT, str: "400000baseCR8/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_FEC_LLRS_BIT, str: "FEC/LLRS"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseKR_Full_BIT, str: "100000baseKR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseSR_Full_BIT, str: "100000baseSR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseLR_ER_FR_Full_BIT, str: "100000baseLR/ER/FR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseCR_Full_BIT, str: "100000baseCR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100000baseDR_Full_BIT, str: "100000baseDR/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseKR2_Full_BIT, str: "200000baseKR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseSR2_Full_BIT, str: "200000baseSR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseLR2_ER2_FR2_Full_BIT, str: "200000baseLR2/ER2/FR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseDR2_Full_BIT, str: "200000baseDR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_200000baseCR2_Full_BIT, str: "200000baseCR2/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseKR4_Full_BIT, str: "400000baseKR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseSR4_Full_BIT, str: "400000baseSR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseLR4_ER4_FR4_Full_BIT, str: "400000baseLR4/ER4/FR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseDR4_Full_BIT, str: "400000baseDR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_400000baseCR4_Full_BIT, str: "400000baseCR4/Full"},
	{bit: unix.ETHTOOL_LINK_MODE_100baseFX_Half_BIT, str: "100baseFX/Half"},
	{bit: unix.ETHTOOL_LINK_MODE_100baseFX_Full_BIT, str: "100baseFX/Full"},
}
