// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/type/matcher/v3/number.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Specifies the way to match a double value.
type DoubleMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatchPattern:
	//
	//	*DoubleMatcher_Range
	//	*DoubleMatcher_Exact
	MatchPattern  isDoubleMatcher_MatchPattern `protobuf_oneof:"match_pattern"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DoubleMatcher) Reset() {
	*x = DoubleMatcher{}
	mi := &file_envoy_type_matcher_v3_number_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DoubleMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DoubleMatcher) ProtoMessage() {}

func (x *DoubleMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_type_matcher_v3_number_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DoubleMatcher.ProtoReflect.Descriptor instead.
func (*DoubleMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_type_matcher_v3_number_proto_rawDescGZIP(), []int{0}
}

func (x *DoubleMatcher) GetMatchPattern() isDoubleMatcher_MatchPattern {
	if x != nil {
		return x.MatchPattern
	}
	return nil
}

func (x *DoubleMatcher) GetRange() *v3.DoubleRange {
	if x != nil {
		if x, ok := x.MatchPattern.(*DoubleMatcher_Range); ok {
			return x.Range
		}
	}
	return nil
}

func (x *DoubleMatcher) GetExact() float64 {
	if x != nil {
		if x, ok := x.MatchPattern.(*DoubleMatcher_Exact); ok {
			return x.Exact
		}
	}
	return 0
}

type isDoubleMatcher_MatchPattern interface {
	isDoubleMatcher_MatchPattern()
}

type DoubleMatcher_Range struct {
	// If specified, the input double value must be in the range specified here.
	// Note: The range is using half-open interval semantics [start, end).
	Range *v3.DoubleRange `protobuf:"bytes,1,opt,name=range,proto3,oneof"`
}

type DoubleMatcher_Exact struct {
	// If specified, the input double value must be equal to the value specified here.
	Exact float64 `protobuf:"fixed64,2,opt,name=exact,proto3,oneof"`
}

func (*DoubleMatcher_Range) isDoubleMatcher_MatchPattern() {}

func (*DoubleMatcher_Exact) isDoubleMatcher_MatchPattern() {}

var File_envoy_type_matcher_v3_number_proto protoreflect.FileDescriptor

const file_envoy_type_matcher_v3_number_proto_rawDesc = "" +
	"\n" +
	"\"envoy/type/matcher/v3/number.proto\x12\x15envoy.type.matcher.v3\x1a\x19envoy/type/v3/range.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x9a\x01\n" +
	"\rDoubleMatcher\x122\n" +
	"\x05range\x18\x01 \x01(\v2\x1a.envoy.type.v3.DoubleRangeH\x00R\x05range\x12\x16\n" +
	"\x05exact\x18\x02 \x01(\x01H\x00R\x05exact:'\x9aň\x1e\"\n" +
	" envoy.type.matcher.DoubleMatcherB\x14\n" +
	"\rmatch_pattern\x12\x03\xf8B\x01B\x84\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.type.matcher.v3B\vNumberProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_type_matcher_v3_number_proto_rawDescOnce sync.Once
	file_envoy_type_matcher_v3_number_proto_rawDescData []byte
)

func file_envoy_type_matcher_v3_number_proto_rawDescGZIP() []byte {
	file_envoy_type_matcher_v3_number_proto_rawDescOnce.Do(func() {
		file_envoy_type_matcher_v3_number_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_number_proto_rawDesc), len(file_envoy_type_matcher_v3_number_proto_rawDesc)))
	})
	return file_envoy_type_matcher_v3_number_proto_rawDescData
}

var file_envoy_type_matcher_v3_number_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_type_matcher_v3_number_proto_goTypes = []any{
	(*DoubleMatcher)(nil),  // 0: envoy.type.matcher.v3.DoubleMatcher
	(*v3.DoubleRange)(nil), // 1: envoy.type.v3.DoubleRange
}
var file_envoy_type_matcher_v3_number_proto_depIdxs = []int32{
	1, // 0: envoy.type.matcher.v3.DoubleMatcher.range:type_name -> envoy.type.v3.DoubleRange
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_type_matcher_v3_number_proto_init() }
func file_envoy_type_matcher_v3_number_proto_init() {
	if File_envoy_type_matcher_v3_number_proto != nil {
		return
	}
	file_envoy_type_matcher_v3_number_proto_msgTypes[0].OneofWrappers = []any{
		(*DoubleMatcher_Range)(nil),
		(*DoubleMatcher_Exact)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_type_matcher_v3_number_proto_rawDesc), len(file_envoy_type_matcher_v3_number_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_type_matcher_v3_number_proto_goTypes,
		DependencyIndexes: file_envoy_type_matcher_v3_number_proto_depIdxs,
		MessageInfos:      file_envoy_type_matcher_v3_number_proto_msgTypes,
	}.Build()
	File_envoy_type_matcher_v3_number_proto = out.File
	file_envoy_type_matcher_v3_number_proto_goTypes = nil
	file_envoy_type_matcher_v3_number_proto_depIdxs = nil
}
