// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/trace/v3/service.proto

package tracev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration structure.
type TraceServiceConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The upstream gRPC cluster that hosts the metrics service.
	GrpcService   *v3.GrpcService `protobuf:"bytes,1,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TraceServiceConfig) Reset() {
	*x = TraceServiceConfig{}
	mi := &file_envoy_config_trace_v3_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TraceServiceConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TraceServiceConfig) ProtoMessage() {}

func (x *TraceServiceConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_trace_v3_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TraceServiceConfig.ProtoReflect.Descriptor instead.
func (*TraceServiceConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_trace_v3_service_proto_rawDescGZIP(), []int{0}
}

func (x *TraceServiceConfig) GetGrpcService() *v3.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

var File_envoy_config_trace_v3_service_proto protoreflect.FileDescriptor

const file_envoy_config_trace_v3_service_proto_rawDesc = "" +
	"\n" +
	"#envoy/config/trace/v3/service.proto\x12\x15envoy.config.trace.v3\x1a'envoy/config/core/v3/grpc_service.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x95\x01\n" +
	"\x12TraceServiceConfig\x12N\n" +
	"\fgrpc_service\x18\x01 \x01(\v2!.envoy.config.core.v3.GrpcServiceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vgrpcService:/\x9aň\x1e*\n" +
	"(envoy.config.trace.v2.TraceServiceConfigB\x83\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.config.trace.v3B\fServiceProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3b\x06proto3"

var (
	file_envoy_config_trace_v3_service_proto_rawDescOnce sync.Once
	file_envoy_config_trace_v3_service_proto_rawDescData []byte
)

func file_envoy_config_trace_v3_service_proto_rawDescGZIP() []byte {
	file_envoy_config_trace_v3_service_proto_rawDescOnce.Do(func() {
		file_envoy_config_trace_v3_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_service_proto_rawDesc), len(file_envoy_config_trace_v3_service_proto_rawDesc)))
	})
	return file_envoy_config_trace_v3_service_proto_rawDescData
}

var file_envoy_config_trace_v3_service_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_trace_v3_service_proto_goTypes = []any{
	(*TraceServiceConfig)(nil), // 0: envoy.config.trace.v3.TraceServiceConfig
	(*v3.GrpcService)(nil),     // 1: envoy.config.core.v3.GrpcService
}
var file_envoy_config_trace_v3_service_proto_depIdxs = []int32{
	1, // 0: envoy.config.trace.v3.TraceServiceConfig.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_config_trace_v3_service_proto_init() }
func file_envoy_config_trace_v3_service_proto_init() {
	if File_envoy_config_trace_v3_service_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_trace_v3_service_proto_rawDesc), len(file_envoy_config_trace_v3_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_trace_v3_service_proto_goTypes,
		DependencyIndexes: file_envoy_config_trace_v3_service_proto_depIdxs,
		MessageInfos:      file_envoy_config_trace_v3_service_proto_msgTypes,
	}.Build()
	File_envoy_config_trace_v3_service_proto = out.File
	file_envoy_config_trace_v3_service_proto_goTypes = nil
	file_envoy_config_trace_v3_service_proto_depIdxs = nil
}
