// Code generated by smithy-go-codegen DO NOT EDIT.

package types

import (
	smithydocument "github.com/aws/smithy-go/document"
)

// This structure contains Amazon Web Services-specific parameter extensions and
// the [identity context].
//
// [identity context]: https://docs.aws.amazon.com/singlesignon/latest/userguide/trustedidentitypropagation-overview.html
type AwsAdditionalDetails struct {

	// The trusted context assertion is signed and encrypted by STS. It provides
	// access to sts:identity_context claim in the idToken without JWT parsing
	//
	// Identity context comprises information that Amazon Web Services services use to
	// make authorization decisions when they receive requests.
	IdentityContext *string

	noSmithyDocumentSerde
}

type noSmithyDocumentSerde = smithydocument.NoSerde
