// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/metrics/v3/stats.proto

package metricsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for pluggable stats sinks.
type StatsSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the stats sink to instantiate. The name must match a supported
	// stats sink.
	// See the :ref:`extensions listed in typed_config below <extension_category_envoy.stats_sinks>` for the default list of available stats sink.
	// Sinks optionally support tagged/multiple dimensional metrics.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Stats sink specific configuration which depends on the sink being instantiated. See
	// :ref:`StatsdSink <envoy_v3_api_msg_config.metrics.v3.StatsdSink>` for an example.
	// [#extension-category: envoy.stats_sinks]
	//
	// Types that are valid to be assigned to ConfigType:
	//
	//	*StatsSink_TypedConfig
	ConfigType    isStatsSink_ConfigType `protobuf_oneof:"config_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StatsSink) Reset() {
	*x = StatsSink{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatsSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatsSink) ProtoMessage() {}

func (x *StatsSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatsSink.ProtoReflect.Descriptor instead.
func (*StatsSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{0}
}

func (x *StatsSink) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *StatsSink) GetConfigType() isStatsSink_ConfigType {
	if x != nil {
		return x.ConfigType
	}
	return nil
}

func (x *StatsSink) GetTypedConfig() *anypb.Any {
	if x != nil {
		if x, ok := x.ConfigType.(*StatsSink_TypedConfig); ok {
			return x.TypedConfig
		}
	}
	return nil
}

type isStatsSink_ConfigType interface {
	isStatsSink_ConfigType()
}

type StatsSink_TypedConfig struct {
	TypedConfig *anypb.Any `protobuf:"bytes,3,opt,name=typed_config,json=typedConfig,proto3,oneof"`
}

func (*StatsSink_TypedConfig) isStatsSink_ConfigType() {}

// Statistics configuration such as tagging.
type StatsConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Each stat name is independently processed through these tag specifiers. When a tag is
	// matched, the first capture group is not immediately removed from the name, so later
	// :ref:`TagSpecifiers <envoy_v3_api_msg_config.metrics.v3.TagSpecifier>` can also match that
	// same portion of the match. After all tag matching is complete, a tag-extracted version of
	// the name is produced and is used in stats sinks that represent tags, such as Prometheus.
	StatsTags []*TagSpecifier `protobuf:"bytes,1,rep,name=stats_tags,json=statsTags,proto3" json:"stats_tags,omitempty"`
	// Use all default tag regexes specified in Envoy. These can be combined with
	// custom tags specified in :ref:`stats_tags
	// <envoy_v3_api_field_config.metrics.v3.StatsConfig.stats_tags>`. They will be processed before
	// the custom tags.
	//
	// See :repo:`well_known_names.h <source/common/config/well_known_names.h>` for a list of the
	// default tags in Envoy.
	//
	// If not provided, the value is assumed to be true.
	UseAllDefaultTags *wrapperspb.BoolValue `protobuf:"bytes,2,opt,name=use_all_default_tags,json=useAllDefaultTags,proto3" json:"use_all_default_tags,omitempty"`
	// Inclusion/exclusion matcher for stat name creation. If not provided, all stats are instantiated
	// as normal. Preventing the instantiation of certain families of stats can improve memory
	// performance for Envoys running especially large configs.
	//
	// .. warning::
	//
	//	Excluding stats may affect Envoy's behavior in undocumented ways. See
	//	`issue #8771 <https://github.com/envoyproxy/envoy/issues/8771>`_ for more information.
	//	If any unexpected behavior changes are observed, please open a new issue immediately.
	StatsMatcher *StatsMatcher `protobuf:"bytes,3,opt,name=stats_matcher,json=statsMatcher,proto3" json:"stats_matcher,omitempty"`
	// Defines rules for setting the histogram buckets. Rules are evaluated in order, and the first
	// match is applied. If no match is found (or if no rules are set), the following default buckets
	// are used:
	//
	//	.. code-block:: json
	//
	//	  [
	//	    0.5,
	//	    1,
	//	    5,
	//	    10,
	//	    25,
	//	    50,
	//	    100,
	//	    250,
	//	    500,
	//	    1000,
	//	    2500,
	//	    5000,
	//	    10000,
	//	    30000,
	//	    60000,
	//	    300000,
	//	    600000,
	//	    1800000,
	//	    3600000
	//	  ]
	HistogramBucketSettings []*HistogramBucketSettings `protobuf:"bytes,4,rep,name=histogram_bucket_settings,json=histogramBucketSettings,proto3" json:"histogram_bucket_settings,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *StatsConfig) Reset() {
	*x = StatsConfig{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatsConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatsConfig) ProtoMessage() {}

func (x *StatsConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatsConfig.ProtoReflect.Descriptor instead.
func (*StatsConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{1}
}

func (x *StatsConfig) GetStatsTags() []*TagSpecifier {
	if x != nil {
		return x.StatsTags
	}
	return nil
}

func (x *StatsConfig) GetUseAllDefaultTags() *wrapperspb.BoolValue {
	if x != nil {
		return x.UseAllDefaultTags
	}
	return nil
}

func (x *StatsConfig) GetStatsMatcher() *StatsMatcher {
	if x != nil {
		return x.StatsMatcher
	}
	return nil
}

func (x *StatsConfig) GetHistogramBucketSettings() []*HistogramBucketSettings {
	if x != nil {
		return x.HistogramBucketSettings
	}
	return nil
}

// Configuration for disabling stat instantiation.
type StatsMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to StatsMatcher:
	//
	//	*StatsMatcher_RejectAll
	//	*StatsMatcher_ExclusionList
	//	*StatsMatcher_InclusionList
	StatsMatcher  isStatsMatcher_StatsMatcher `protobuf_oneof:"stats_matcher"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StatsMatcher) Reset() {
	*x = StatsMatcher{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatsMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatsMatcher) ProtoMessage() {}

func (x *StatsMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatsMatcher.ProtoReflect.Descriptor instead.
func (*StatsMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{2}
}

func (x *StatsMatcher) GetStatsMatcher() isStatsMatcher_StatsMatcher {
	if x != nil {
		return x.StatsMatcher
	}
	return nil
}

func (x *StatsMatcher) GetRejectAll() bool {
	if x != nil {
		if x, ok := x.StatsMatcher.(*StatsMatcher_RejectAll); ok {
			return x.RejectAll
		}
	}
	return false
}

func (x *StatsMatcher) GetExclusionList() *v3.ListStringMatcher {
	if x != nil {
		if x, ok := x.StatsMatcher.(*StatsMatcher_ExclusionList); ok {
			return x.ExclusionList
		}
	}
	return nil
}

func (x *StatsMatcher) GetInclusionList() *v3.ListStringMatcher {
	if x != nil {
		if x, ok := x.StatsMatcher.(*StatsMatcher_InclusionList); ok {
			return x.InclusionList
		}
	}
	return nil
}

type isStatsMatcher_StatsMatcher interface {
	isStatsMatcher_StatsMatcher()
}

type StatsMatcher_RejectAll struct {
	// If “reject_all“ is true, then all stats are disabled. If “reject_all“ is false, then all
	// stats are enabled.
	RejectAll bool `protobuf:"varint,1,opt,name=reject_all,json=rejectAll,proto3,oneof"`
}

type StatsMatcher_ExclusionList struct {
	// Exclusive match. All stats are enabled except for those matching one of the supplied
	// StringMatcher protos.
	ExclusionList *v3.ListStringMatcher `protobuf:"bytes,2,opt,name=exclusion_list,json=exclusionList,proto3,oneof"`
}

type StatsMatcher_InclusionList struct {
	// Inclusive match. No stats are enabled except for those matching one of the supplied
	// StringMatcher protos.
	InclusionList *v3.ListStringMatcher `protobuf:"bytes,3,opt,name=inclusion_list,json=inclusionList,proto3,oneof"`
}

func (*StatsMatcher_RejectAll) isStatsMatcher_StatsMatcher() {}

func (*StatsMatcher_ExclusionList) isStatsMatcher_StatsMatcher() {}

func (*StatsMatcher_InclusionList) isStatsMatcher_StatsMatcher() {}

// Designates a tag name and value pair. The value may be either a fixed value
// or a regex providing the value via capture groups. The specified tag will be
// unconditionally set if a fixed value, otherwise it will only be set if one
// or more capture groups in the regex match.
type TagSpecifier struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Attaches an identifier to the tag values to identify the tag being in the
	// sink. Envoy has a set of default names and regexes to extract dynamic
	// portions of existing stats, which can be found in :repo:`well_known_names.h
	// <source/common/config/well_known_names.h>` in the Envoy repository. If a :ref:`tag_name
	// <envoy_v3_api_field_config.metrics.v3.TagSpecifier.tag_name>` is provided in the config and
	// neither :ref:`regex <envoy_v3_api_field_config.metrics.v3.TagSpecifier.regex>` or
	// :ref:`fixed_value <envoy_v3_api_field_config.metrics.v3.TagSpecifier.fixed_value>` were specified,
	// Envoy will attempt to find that name in its set of defaults and use the accompanying regex.
	//
	// .. note::
	//
	//	A stat name may be spelled in such a way that it matches two different
	//	tag extractors for the same tag name. In that case, all but one of the
	//	tag values will be dropped. It is not specified which tag value will be
	//	retained. The extraction will only occur for one of the extractors, and
	//	only the matched extraction will be removed from the tag name.
	TagName string `protobuf:"bytes,1,opt,name=tag_name,json=tagName,proto3" json:"tag_name,omitempty"`
	// Types that are valid to be assigned to TagValue:
	//
	//	*TagSpecifier_Regex
	//	*TagSpecifier_FixedValue
	TagValue      isTagSpecifier_TagValue `protobuf_oneof:"tag_value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TagSpecifier) Reset() {
	*x = TagSpecifier{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TagSpecifier) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TagSpecifier) ProtoMessage() {}

func (x *TagSpecifier) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TagSpecifier.ProtoReflect.Descriptor instead.
func (*TagSpecifier) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{3}
}

func (x *TagSpecifier) GetTagName() string {
	if x != nil {
		return x.TagName
	}
	return ""
}

func (x *TagSpecifier) GetTagValue() isTagSpecifier_TagValue {
	if x != nil {
		return x.TagValue
	}
	return nil
}

func (x *TagSpecifier) GetRegex() string {
	if x != nil {
		if x, ok := x.TagValue.(*TagSpecifier_Regex); ok {
			return x.Regex
		}
	}
	return ""
}

func (x *TagSpecifier) GetFixedValue() string {
	if x != nil {
		if x, ok := x.TagValue.(*TagSpecifier_FixedValue); ok {
			return x.FixedValue
		}
	}
	return ""
}

type isTagSpecifier_TagValue interface {
	isTagSpecifier_TagValue()
}

type TagSpecifier_Regex struct {
	// Designates a tag to strip from the tag extracted name and provide as a named
	// tag value for all statistics. This will only occur if any part of the name
	// matches the regex provided with one or more capture groups.
	//
	// The first capture group identifies the portion of the name to remove. The
	// second capture group (which will normally be nested inside the first) will
	// designate the value of the tag for the statistic. If no second capture
	// group is provided, the first will also be used to set the value of the tag.
	// All other capture groups will be ignored.
	//
	// Example 1. a stat name “cluster.foo_cluster.upstream_rq_timeout“ and
	// one tag specifier:
	//
	// .. code-block:: json
	//
	//	{
	//	  "tag_name": "envoy.cluster_name",
	//	  "regex": "^cluster\\.((.+?)\\.)"
	//	}
	//
	// Note that the regex will remove “foo_cluster.“ making the tag extracted
	// name “cluster.upstream_rq_timeout“ and the tag value for
	// “envoy.cluster_name“ will be “foo_cluster“ (note: there will be no
	// “.“ character because of the second capture group).
	//
	// Example 2. a stat name
	// “http.connection_manager_1.user_agent.ios.downstream_cx_total“ and two
	// tag specifiers:
	//
	// .. code-block:: json
	//
	//	[
	//	  {
	//	    "tag_name": "envoy.http_user_agent",
	//	    "regex": "^http(?=\\.).*?\\.user_agent\\.((.+?)\\.)\\w+?$"
	//	  },
	//	  {
	//	    "tag_name": "envoy.http_conn_manager_prefix",
	//	    "regex": "^http\\.((.*?)\\.)"
	//	  }
	//	]
	//
	// The two regexes of the specifiers will be processed from the elaborated
	// stat name.
	//
	// The first regex will save “ios.“ as the tag value for “envoy.http_user_agent“. It will
	// leave it in the name for potential matching with additional tag specifiers. After all tag
	// specifiers are processed the tags will be removed from the name.
	//
	// The second regex will populate tag “envoy.http_conn_manager_prefix“ with value
	// “connection_manager_1.“, based on the original stat name.
	//
	// As a final step, the matched tags are removed, leaving
	// “http.user_agent.downstream_cx_total“ as the tag extracted name.
	Regex string `protobuf:"bytes,2,opt,name=regex,proto3,oneof"`
}

type TagSpecifier_FixedValue struct {
	// Specifies a fixed tag value for the “tag_name“.
	FixedValue string `protobuf:"bytes,3,opt,name=fixed_value,json=fixedValue,proto3,oneof"`
}

func (*TagSpecifier_Regex) isTagSpecifier_TagValue() {}

func (*TagSpecifier_FixedValue) isTagSpecifier_TagValue() {}

// Specifies a matcher for stats and the buckets that matching stats should use.
type HistogramBucketSettings struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The stats that this rule applies to. The match is applied to the original stat name
	// before tag-extraction, for example “cluster.exampleclustername.upstream_cx_length_ms“.
	Match *v3.StringMatcher `protobuf:"bytes,1,opt,name=match,proto3" json:"match,omitempty"`
	// Each value is the upper bound of a bucket. Each bucket must be greater than 0 and unique.
	// The order of the buckets does not matter.
	Buckets []float64 `protobuf:"fixed64,2,rep,packed,name=buckets,proto3" json:"buckets,omitempty"`
	// Initial number of bins for the “circllhist“ thread local histogram per time series. Default value is 100.
	Bins          *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=bins,proto3" json:"bins,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HistogramBucketSettings) Reset() {
	*x = HistogramBucketSettings{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HistogramBucketSettings) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HistogramBucketSettings) ProtoMessage() {}

func (x *HistogramBucketSettings) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HistogramBucketSettings.ProtoReflect.Descriptor instead.
func (*HistogramBucketSettings) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{4}
}

func (x *HistogramBucketSettings) GetMatch() *v3.StringMatcher {
	if x != nil {
		return x.Match
	}
	return nil
}

func (x *HistogramBucketSettings) GetBuckets() []float64 {
	if x != nil {
		return x.Buckets
	}
	return nil
}

func (x *HistogramBucketSettings) GetBins() *wrapperspb.UInt32Value {
	if x != nil {
		return x.Bins
	}
	return nil
}

// Stats configuration proto schema for built-in “envoy.stat_sinks.statsd“ sink. This sink does not support
// tagged metrics.
// [#extension: envoy.stat_sinks.statsd]
type StatsdSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to StatsdSpecifier:
	//
	//	*StatsdSink_Address
	//	*StatsdSink_TcpClusterName
	StatsdSpecifier isStatsdSink_StatsdSpecifier `protobuf_oneof:"statsd_specifier"`
	// Optional custom prefix for StatsdSink. If
	// specified, this will override the default prefix.
	// For example:
	//
	// .. code-block:: json
	//
	//	{
	//	  "prefix" : "envoy-prod"
	//	}
	//
	// will change emitted stats to
	//
	// .. code-block:: cpp
	//
	//	envoy-prod.test_counter:1|c
	//	envoy-prod.test_timer:5|ms
	//
	// Note that the default prefix, "envoy", will be used if a prefix is not
	// specified.
	//
	// Stats with default prefix:
	//
	// .. code-block:: cpp
	//
	//	envoy.test_counter:1|c
	//	envoy.test_timer:5|ms
	Prefix        string `protobuf:"bytes,3,opt,name=prefix,proto3" json:"prefix,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StatsdSink) Reset() {
	*x = StatsdSink{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatsdSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatsdSink) ProtoMessage() {}

func (x *StatsdSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatsdSink.ProtoReflect.Descriptor instead.
func (*StatsdSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{5}
}

func (x *StatsdSink) GetStatsdSpecifier() isStatsdSink_StatsdSpecifier {
	if x != nil {
		return x.StatsdSpecifier
	}
	return nil
}

func (x *StatsdSink) GetAddress() *v31.Address {
	if x != nil {
		if x, ok := x.StatsdSpecifier.(*StatsdSink_Address); ok {
			return x.Address
		}
	}
	return nil
}

func (x *StatsdSink) GetTcpClusterName() string {
	if x != nil {
		if x, ok := x.StatsdSpecifier.(*StatsdSink_TcpClusterName); ok {
			return x.TcpClusterName
		}
	}
	return ""
}

func (x *StatsdSink) GetPrefix() string {
	if x != nil {
		return x.Prefix
	}
	return ""
}

type isStatsdSink_StatsdSpecifier interface {
	isStatsdSink_StatsdSpecifier()
}

type StatsdSink_Address struct {
	// The UDP address of a running `statsd <https://github.com/etsy/statsd>`_
	// compliant listener. If specified, statistics will be flushed to this
	// address.
	Address *v31.Address `protobuf:"bytes,1,opt,name=address,proto3,oneof"`
}

type StatsdSink_TcpClusterName struct {
	// The name of a cluster that is running a TCP `statsd
	// <https://github.com/etsy/statsd>`_ compliant listener. If specified,
	// Envoy will connect to this cluster to flush statistics.
	TcpClusterName string `protobuf:"bytes,2,opt,name=tcp_cluster_name,json=tcpClusterName,proto3,oneof"`
}

func (*StatsdSink_Address) isStatsdSink_StatsdSpecifier() {}

func (*StatsdSink_TcpClusterName) isStatsdSink_StatsdSpecifier() {}

// Stats configuration proto schema for built-in “envoy.stat_sinks.dog_statsd“ sink.
// The sink emits stats with `DogStatsD <https://docs.datadoghq.com/guides/dogstatsd/>`_
// compatible tags. Tags are configurable via :ref:`StatsConfig
// <envoy_v3_api_msg_config.metrics.v3.StatsConfig>`.
// [#extension: envoy.stat_sinks.dog_statsd]
type DogStatsdSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to DogStatsdSpecifier:
	//
	//	*DogStatsdSink_Address
	DogStatsdSpecifier isDogStatsdSink_DogStatsdSpecifier `protobuf_oneof:"dog_statsd_specifier"`
	// Optional custom metric name prefix. See :ref:`StatsdSink's prefix field
	// <envoy_v3_api_field_config.metrics.v3.StatsdSink.prefix>` for more details.
	Prefix string `protobuf:"bytes,3,opt,name=prefix,proto3" json:"prefix,omitempty"`
	// Optional max datagram size to use when sending UDP messages. By default Envoy
	// will emit one metric per datagram. By specifying a max-size larger than a single
	// metric, Envoy will emit multiple, new-line separated metrics. The max datagram
	// size should not exceed your network's MTU.
	//
	// Note that this value may not be respected if smaller than a single metric.
	MaxBytesPerDatagram *wrapperspb.UInt64Value `protobuf:"bytes,4,opt,name=max_bytes_per_datagram,json=maxBytesPerDatagram,proto3" json:"max_bytes_per_datagram,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *DogStatsdSink) Reset() {
	*x = DogStatsdSink{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DogStatsdSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DogStatsdSink) ProtoMessage() {}

func (x *DogStatsdSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DogStatsdSink.ProtoReflect.Descriptor instead.
func (*DogStatsdSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{6}
}

func (x *DogStatsdSink) GetDogStatsdSpecifier() isDogStatsdSink_DogStatsdSpecifier {
	if x != nil {
		return x.DogStatsdSpecifier
	}
	return nil
}

func (x *DogStatsdSink) GetAddress() *v31.Address {
	if x != nil {
		if x, ok := x.DogStatsdSpecifier.(*DogStatsdSink_Address); ok {
			return x.Address
		}
	}
	return nil
}

func (x *DogStatsdSink) GetPrefix() string {
	if x != nil {
		return x.Prefix
	}
	return ""
}

func (x *DogStatsdSink) GetMaxBytesPerDatagram() *wrapperspb.UInt64Value {
	if x != nil {
		return x.MaxBytesPerDatagram
	}
	return nil
}

type isDogStatsdSink_DogStatsdSpecifier interface {
	isDogStatsdSink_DogStatsdSpecifier()
}

type DogStatsdSink_Address struct {
	// The UDP address of a running DogStatsD compliant listener. If specified,
	// statistics will be flushed to this address.
	Address *v31.Address `protobuf:"bytes,1,opt,name=address,proto3,oneof"`
}

func (*DogStatsdSink_Address) isDogStatsdSink_DogStatsdSpecifier() {}

// Stats configuration proto schema for built-in “envoy.stat_sinks.hystrix“ sink.
// The sink emits stats in `text/event-stream
// <https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events>`_
// formatted stream for use by `Hystrix dashboard
// <https://github.com/Netflix-Skunkworks/hystrix-dashboard/wiki>`_.
//
// Note that only a single HystrixSink should be configured.
//
// Streaming is started through an admin endpoint :http:get:`/hystrix_event_stream`.
// [#extension: envoy.stat_sinks.hystrix]
type HystrixSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The number of buckets the rolling statistical window is divided into.
	//
	// Each time the sink is flushed, all relevant Envoy statistics are sampled and
	// added to the rolling window (removing the oldest samples in the window
	// in the process). The sink then outputs the aggregate statistics across the
	// current rolling window to the event stream(s).
	//
	// “rolling_window(ms)“ = “stats_flush_interval(ms)“ * “num_of_buckets“
	//
	// More detailed explanation can be found in `Hystrix wiki
	// <https://github.com/Netflix/Hystrix/wiki/Metrics-and-Monitoring#hystrixrollingnumber>`_.
	NumBuckets    int64 `protobuf:"varint,1,opt,name=num_buckets,json=numBuckets,proto3" json:"num_buckets,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HystrixSink) Reset() {
	*x = HystrixSink{}
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HystrixSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HystrixSink) ProtoMessage() {}

func (x *HystrixSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_metrics_v3_stats_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HystrixSink.ProtoReflect.Descriptor instead.
func (*HystrixSink) Descriptor() ([]byte, []int) {
	return file_envoy_config_metrics_v3_stats_proto_rawDescGZIP(), []int{7}
}

func (x *HystrixSink) GetNumBuckets() int64 {
	if x != nil {
		return x.NumBuckets
	}
	return 0
}

var File_envoy_config_metrics_v3_stats_proto protoreflect.FileDescriptor

const file_envoy_config_metrics_v3_stats_proto_rawDesc = "" +
	"\n" +
	"#envoy/config/metrics/v3/stats.proto\x12\x17envoy.config.metrics.v3\x1a\"envoy/config/core/v3/address.proto\x1a\"envoy/type/matcher/v3/string.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xa1\x01\n" +
	"\tStatsSink\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x129\n" +
	"\ftyped_config\x18\x03 \x01(\v2\x14.google.protobuf.AnyH\x00R\vtypedConfig:(\x9aň\x1e#\n" +
	"!envoy.config.metrics.v2.StatsSinkB\r\n" +
	"\vconfig_typeJ\x04\b\x02\x10\x03R\x06config\"\x86\x03\n" +
	"\vStatsConfig\x12D\n" +
	"\n" +
	"stats_tags\x18\x01 \x03(\v2%.envoy.config.metrics.v3.TagSpecifierR\tstatsTags\x12K\n" +
	"\x14use_all_default_tags\x18\x02 \x01(\v2\x1a.google.protobuf.BoolValueR\x11useAllDefaultTags\x12J\n" +
	"\rstats_matcher\x18\x03 \x01(\v2%.envoy.config.metrics.v3.StatsMatcherR\fstatsMatcher\x12l\n" +
	"\x19histogram_bucket_settings\x18\x04 \x03(\v20.envoy.config.metrics.v3.HistogramBucketSettingsR\x17histogramBucketSettings:*\x9aň\x1e%\n" +
	"#envoy.config.metrics.v2.StatsConfig\"\x98\x02\n" +
	"\fStatsMatcher\x12\x1f\n" +
	"\n" +
	"reject_all\x18\x01 \x01(\bH\x00R\trejectAll\x12Q\n" +
	"\x0eexclusion_list\x18\x02 \x01(\v2(.envoy.type.matcher.v3.ListStringMatcherH\x00R\rexclusionList\x12Q\n" +
	"\x0einclusion_list\x18\x03 \x01(\v2(.envoy.type.matcher.v3.ListStringMatcherH\x00R\rinclusionList:+\x9aň\x1e&\n" +
	"$envoy.config.metrics.v2.StatsMatcherB\x14\n" +
	"\rstats_matcher\x12\x03\xf8B\x01\"\xa8\x01\n" +
	"\fTagSpecifier\x12\x19\n" +
	"\btag_name\x18\x01 \x01(\tR\atagName\x12 \n" +
	"\x05regex\x18\x02 \x01(\tB\b\xfaB\x05r\x03(\x80\bH\x00R\x05regex\x12!\n" +
	"\vfixed_value\x18\x03 \x01(\tH\x00R\n" +
	"fixedValue:+\x9aň\x1e&\n" +
	"$envoy.config.metrics.v2.TagSpecifierB\v\n" +
	"\ttag_value\"\xcf\x01\n" +
	"\x17HistogramBucketSettings\x12D\n" +
	"\x05match\x18\x01 \x01(\v2$.envoy.type.matcher.v3.StringMatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05match\x12/\n" +
	"\abuckets\x18\x02 \x03(\x01B\x15\xfaB\x12\x92\x01\x0f\x18\x01\"\v\x12\t!\x00\x00\x00\x00\x00\x00\x00\x00R\abuckets\x12=\n" +
	"\x04bins\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueB\v\xfaB\b*\x06\x18\x82\xe8\x02 \x00R\x04bins\"\xcf\x01\n" +
	"\n" +
	"StatsdSink\x129\n" +
	"\aaddress\x18\x01 \x01(\v2\x1d.envoy.config.core.v3.AddressH\x00R\aaddress\x12*\n" +
	"\x10tcp_cluster_name\x18\x02 \x01(\tH\x00R\x0etcpClusterName\x12\x16\n" +
	"\x06prefix\x18\x03 \x01(\tR\x06prefix:)\x9aň\x1e$\n" +
	"\"envoy.config.metrics.v2.StatsdSinkB\x17\n" +
	"\x10statsd_specifier\x12\x03\xf8B\x01\"\x8f\x02\n" +
	"\rDogStatsdSink\x129\n" +
	"\aaddress\x18\x01 \x01(\v2\x1d.envoy.config.core.v3.AddressH\x00R\aaddress\x12\x16\n" +
	"\x06prefix\x18\x03 \x01(\tR\x06prefix\x12Z\n" +
	"\x16max_bytes_per_datagram\x18\x04 \x01(\v2\x1c.google.protobuf.UInt64ValueB\a\xfaB\x042\x02 \x00R\x13maxBytesPerDatagram:,\x9aň\x1e'\n" +
	"%envoy.config.metrics.v2.DogStatsdSinkB\x1b\n" +
	"\x14dog_statsd_specifier\x12\x03\xf8B\x01J\x04\b\x02\x10\x03\"Z\n" +
	"\vHystrixSink\x12\x1f\n" +
	"\vnum_buckets\x18\x01 \x01(\x03R\n" +
	"numBuckets:*\x9aň\x1e%\n" +
	"#envoy.config.metrics.v2.HystrixSinkB\x87\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"%io.envoyproxy.envoy.config.metrics.v3B\n" +
	"StatsProtoP\x01ZHgithub.com/envoyproxy/go-control-plane/envoy/config/metrics/v3;metricsv3b\x06proto3"

var (
	file_envoy_config_metrics_v3_stats_proto_rawDescOnce sync.Once
	file_envoy_config_metrics_v3_stats_proto_rawDescData []byte
)

func file_envoy_config_metrics_v3_stats_proto_rawDescGZIP() []byte {
	file_envoy_config_metrics_v3_stats_proto_rawDescOnce.Do(func() {
		file_envoy_config_metrics_v3_stats_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_metrics_v3_stats_proto_rawDesc), len(file_envoy_config_metrics_v3_stats_proto_rawDesc)))
	})
	return file_envoy_config_metrics_v3_stats_proto_rawDescData
}

var file_envoy_config_metrics_v3_stats_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_envoy_config_metrics_v3_stats_proto_goTypes = []any{
	(*StatsSink)(nil),               // 0: envoy.config.metrics.v3.StatsSink
	(*StatsConfig)(nil),             // 1: envoy.config.metrics.v3.StatsConfig
	(*StatsMatcher)(nil),            // 2: envoy.config.metrics.v3.StatsMatcher
	(*TagSpecifier)(nil),            // 3: envoy.config.metrics.v3.TagSpecifier
	(*HistogramBucketSettings)(nil), // 4: envoy.config.metrics.v3.HistogramBucketSettings
	(*StatsdSink)(nil),              // 5: envoy.config.metrics.v3.StatsdSink
	(*DogStatsdSink)(nil),           // 6: envoy.config.metrics.v3.DogStatsdSink
	(*HystrixSink)(nil),             // 7: envoy.config.metrics.v3.HystrixSink
	(*anypb.Any)(nil),               // 8: google.protobuf.Any
	(*wrapperspb.BoolValue)(nil),    // 9: google.protobuf.BoolValue
	(*v3.ListStringMatcher)(nil),    // 10: envoy.type.matcher.v3.ListStringMatcher
	(*v3.StringMatcher)(nil),        // 11: envoy.type.matcher.v3.StringMatcher
	(*wrapperspb.UInt32Value)(nil),  // 12: google.protobuf.UInt32Value
	(*v31.Address)(nil),             // 13: envoy.config.core.v3.Address
	(*wrapperspb.UInt64Value)(nil),  // 14: google.protobuf.UInt64Value
}
var file_envoy_config_metrics_v3_stats_proto_depIdxs = []int32{
	8,  // 0: envoy.config.metrics.v3.StatsSink.typed_config:type_name -> google.protobuf.Any
	3,  // 1: envoy.config.metrics.v3.StatsConfig.stats_tags:type_name -> envoy.config.metrics.v3.TagSpecifier
	9,  // 2: envoy.config.metrics.v3.StatsConfig.use_all_default_tags:type_name -> google.protobuf.BoolValue
	2,  // 3: envoy.config.metrics.v3.StatsConfig.stats_matcher:type_name -> envoy.config.metrics.v3.StatsMatcher
	4,  // 4: envoy.config.metrics.v3.StatsConfig.histogram_bucket_settings:type_name -> envoy.config.metrics.v3.HistogramBucketSettings
	10, // 5: envoy.config.metrics.v3.StatsMatcher.exclusion_list:type_name -> envoy.type.matcher.v3.ListStringMatcher
	10, // 6: envoy.config.metrics.v3.StatsMatcher.inclusion_list:type_name -> envoy.type.matcher.v3.ListStringMatcher
	11, // 7: envoy.config.metrics.v3.HistogramBucketSettings.match:type_name -> envoy.type.matcher.v3.StringMatcher
	12, // 8: envoy.config.metrics.v3.HistogramBucketSettings.bins:type_name -> google.protobuf.UInt32Value
	13, // 9: envoy.config.metrics.v3.StatsdSink.address:type_name -> envoy.config.core.v3.Address
	13, // 10: envoy.config.metrics.v3.DogStatsdSink.address:type_name -> envoy.config.core.v3.Address
	14, // 11: envoy.config.metrics.v3.DogStatsdSink.max_bytes_per_datagram:type_name -> google.protobuf.UInt64Value
	12, // [12:12] is the sub-list for method output_type
	12, // [12:12] is the sub-list for method input_type
	12, // [12:12] is the sub-list for extension type_name
	12, // [12:12] is the sub-list for extension extendee
	0,  // [0:12] is the sub-list for field type_name
}

func init() { file_envoy_config_metrics_v3_stats_proto_init() }
func file_envoy_config_metrics_v3_stats_proto_init() {
	if File_envoy_config_metrics_v3_stats_proto != nil {
		return
	}
	file_envoy_config_metrics_v3_stats_proto_msgTypes[0].OneofWrappers = []any{
		(*StatsSink_TypedConfig)(nil),
	}
	file_envoy_config_metrics_v3_stats_proto_msgTypes[2].OneofWrappers = []any{
		(*StatsMatcher_RejectAll)(nil),
		(*StatsMatcher_ExclusionList)(nil),
		(*StatsMatcher_InclusionList)(nil),
	}
	file_envoy_config_metrics_v3_stats_proto_msgTypes[3].OneofWrappers = []any{
		(*TagSpecifier_Regex)(nil),
		(*TagSpecifier_FixedValue)(nil),
	}
	file_envoy_config_metrics_v3_stats_proto_msgTypes[5].OneofWrappers = []any{
		(*StatsdSink_Address)(nil),
		(*StatsdSink_TcpClusterName)(nil),
	}
	file_envoy_config_metrics_v3_stats_proto_msgTypes[6].OneofWrappers = []any{
		(*DogStatsdSink_Address)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_metrics_v3_stats_proto_rawDesc), len(file_envoy_config_metrics_v3_stats_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_metrics_v3_stats_proto_goTypes,
		DependencyIndexes: file_envoy_config_metrics_v3_stats_proto_depIdxs,
		MessageInfos:      file_envoy_config_metrics_v3_stats_proto_msgTypes,
	}.Build()
	File_envoy_config_metrics_v3_stats_proto = out.File
	file_envoy_config_metrics_v3_stats_proto_goTypes = nil
	file_envoy_config_metrics_v3_stats_proto_depIdxs = nil
}
